/*	X Window Videum Tool
 *	gcc -o xvideum -L/usr/X11R6/lib/ -lXt -lXaw -Wall xvideum.c
 *
 *	For testing the Video for Linux Videum driver
 *
 *	This program was written by Bill Dirks.
 *	This program is in the public domain.
 */

/*  Set these according to your set-up */
#define MY_DEVICE "/dev/video0"
#define MY_INPUT  0	       /*-> 0=MXC, 1=COMPOSITE, 2=SVIDEO */
#define MY_STD	  "PAL"        /*-> "PAL" or "NTSC" */
#define MY_WIDTH  320
#define MY_HEIGHT 240
#define MY_PIXELFORMAT PIX_FMT_RGB565
#define MY_CHROMACOLOR 0xff00ff /* magenta */
/* if you are running 8 bit color or 24 bit color, you will have to
 modify the following const; I suppose the RIGHT way to do it is
 through X calls, but I don_t know X well enough yet   patb */
#define MY_CHROMACOLOR16 0xf81f /* in rgb 5-6-5 format */
/* PIX_FMT_YUYV */
/* PIX_FMT_GREY */
/* PIX_FMT_YUV420 */
/* PIX_FMT_RGB555 */
/* PIX_FMT_RGB565 */
/* PIX_FMT_RGB24 */
/* PIX_FMT_RGB32 */

#define STREAMBUFS	4

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/mman.h>
#include <errno.h>

/* These are needed to use the Videum driver */
#include <linux/fs.h>
#include <linux/kernel.h>
#include "videodev.h"

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/Porthole.h>

int bShowReads = 1;
int bShowSingle = 0;
int cur_width = MY_WIDTH;
int cur_height = MY_HEIGHT;

void MoveVidcapWindow(int vid, int x, int y)
{
	struct display_window dw;
	
fprintf(stderr, "MoveVidcapWindow is called: x=%d, y=%d\n", x, y);

	if (-1 == ioctl(vid, VIDIOC_COREL_GDWIN, &dw))
	{
		fprintf(stderr, "Error getting window parameters.\n");
	}

	dw.dest_win.x = x;
	dw.dest_win.y = y;

	if (-1 == ioctl(vid, VIDIOC_COREL_SDWIN, &dw))
	{
		fprintf(stderr, "Error setting window parameters.\n");
	}

#if 0
    struct video_window vw = { x,y,cur_width,cur_height, MY_CHROMACOLOR, 0, 0};
    ioctl(vid, VIDIOC_S_WIN, &vw);
#endif

#if 0
	if (-1 == ioctl(vid, VIDIOC_COREL_PREVIEW, 1))
	{
		fprintf(stderr, "Error preview.\n");
	}
#endif
}

typedef struct tag_vimage
{
	char								*data;
	XImage							*ximage;
}	VIMAGE;

//void quit(Widget w, XtPointer client, XtPointer call)
//{
//	exit(0);
//}

#if 0
void printctrl(struct video_queryctrl *qc)
{	
	printf("  %-12s %-6s %8d %8d %7d %8d\n",
	       qc->name,
	       (qc->step == -1) ? "(enum)" : (qc->step == 0) ? "(bool)" :
	       "(int)",
	       qc->minimum, qc->maximum, qc->step, qc->default_value);
}
#endif

int
main(int argc, char *argv[])
{
	XtAppContext								app;
	VIMAGE											vimage;
	Widget											toplevel;
  Widget											read_copy_viewport;
	int													vid;
	int													err;
	int													i;
  GC													imagegc;

	/* Open video device */
	vid = open(MY_DEVICE, O_RDWR);
	if (vid < 0)
	{
		printf("No video device\n");
		return 1;
	}

	/* Setting video channel to use */
	{
		int nChannel = 1;
		if (-1 == ioctl(vid, VIDIOCSCHAN, nChannel))
		{
			fprintf(stderr, "Error selecting channel.\n");
		}
	}

/* 	close(vid);	*/ /* temp */

	/* Setting default capture properties */
	{
		struct video_picture vp;

		vp.brightness = 70 * 65535 / 100;
		vp.hue = 0 * 65535 / 100;
		vp.colour = 70 * 65535 / 100;
		vp.contrast = 60 * 65535 / 100;
		vp.whiteness = 0 * 65535 / 100;
		vp.palette = VIDEO_PALETTE_RGB565;

		if (-1 == ioctl(vid, VIDIOCSPICT, &vp))
		{
			fprintf(stderr, "Error setting capture properties.\n");
		}
	}

	/* Setting default capture window size */
	{
		struct video_window vw;

		vw.x = 0;
		vw.y = 0;
		vw.width = 176;
		vw.height = 144;
		vw.chromakey = 0xffffff;
		vw.flags = 0;
		vw.clips = (void*)0;
		vw.clipcount = 0;

		if (-1 == ioctl(vid, VIDIOCSWIN, &vw))
		{
			fprintf(stderr, "Error setting capture window.\n");
		}
	}

	/* Start video capture */
	{
		if (-1 == ioctl(vid, VIDIOCCAPTURE, 1))
		{
			fprintf(stderr, "Error starting video capture.\n");
		}
	}

	/* Set overlay window Main */
	{
		struct display_window dw;

		dw.SrcType = 0;
		dw.SrcPalette = VIDEO_PALETTE_RGB565;
		dw.SrcWidth = 176;
		dw.SrcHeight = 144;
		dw.UseChromaKey = 1;
		dw.dest_win.x = 100;
		dw.dest_win.y = 200;
		dw.dest_win.width = 176;
		dw.dest_win.height = 144;
		dw.dest_win.chromakey = 0xF8FCF8 & 0xFF00FF;

		if (-1 == ioctl(vid, VIDIOC_COREL_SDWIN, &dw))
		{
			fprintf(stderr, "Error setting video window.\n");
		}
	}

	/* Start Main overlay display */
	{
		ioctl(vid, VIDIOC_COREL_PREVIEW, 1);
	}

	vimage.data = malloc(176*144*2);
	if (vimage.data == NULL)
	{
		printf("malloc(%d) failed\n", 176*144*2);
		return 1;
	}

	toplevel = XtAppInitialize(	&app, "xvideum", NULL, 0,
															&argc, argv, NULL, NULL, 0);


	/* I create a window that is twice as wide as we need;
			the read() data goes into the left hand size;
			the live image will be mapped into the right hand side */
	{
		XColor xc = {MY_CHROMACOLOR,
									0x8000, 0x8000, 0x8000, 255};
            
		read_copy_viewport = XtVaCreateManagedWidget(	"port",
																									portholeWidgetClass, toplevel,
																									XtNwidth, 176*2,
																									XtNheight, 144,
																									XtNbackground, 
																									/* MY_CHROMACOLOR16, NULL);*/
																									MY_CHROMACOLOR16, NULL);
	}

	XtRealizeWidget(toplevel);

	imagegc = XCreateGC(XtDisplay(read_copy_viewport),
											DefaultRootWindow(XtDisplay(read_copy_viewport)),
											0, NULL);

	vimage.ximage = XCreateImage(	XtDisplay(read_copy_viewport),
																DefaultVisual(XtDisplay(read_copy_viewport), 0),
																16, ZPixmap,
																0, vimage.data,
																176, 144, 16, 0);
	if (vimage.ximage == NULL)
	{
		printf("No XImage\n");
		return 1;
	}

	for (;;)
	{
		XEvent event;

		err = 0;
		if (XtIsRealized(read_copy_viewport))
		{
			if (err == 0)
			{
				if (bShowReads)
				{
#if 0
					while(-1 == read(vid, vimage.data, 176*144*2))
					{
						fprintf(stderr, "Error reading video data.\n");
					}
#endif
					read(vid, vimage.data, 176*144*2);
				}
				XPutImage(XtDisplay(read_copy_viewport),
									XtWindow(read_copy_viewport),
									imagegc,
									vimage.ximage,
									0, 0,
									0, 0,
									176,144);
			}
		}

		ioctl(vid, VIDIOC_COREL_PREVIEW, 1);	

		while (XtAppPending(app))
		{
			XtAppNextEvent(app, &event);
			switch (event.type)
			{
				case ConfigureNotify:
				{
					XConfigureEvent *ce = (XConfigureEvent*)&event;
																cur_width = ce->width - 176; /* MY_WIDTH; */
																cur_height = ce->height;
																MoveVidcapWindow(vid,ce->x+176+1/*MY_WIDTH*/,ce->y+1);
				}
				break;

				case FocusIn:
					bShowReads = 1;
				break;

				case FocusOut:
					bShowReads = 0;
				break;

				case ResizeRequest:
				{
					XResizeRequestEvent* re = (XResizeRequestEvent*)&event;

					bShowSingle = 1;
					printf("got resize %d %d\n",re->width,re->height);
									cur_width = re->width;
									cur_height = re->height;
				}
				break;

				default:
				break;
			}
			XtDispatchEvent(&event);
		}

#if 0
		ioctl(vid, VIDIOC_COREL_PREVIEW, 1);
#endif
	}
	close(vid);

	return 0;
}
