#include <linux/delay.h>
#include "igs.h"
#include "video.h"
#include "fifotab.h"

unsigned short  iVFilterOn[2]           = { 0 };
unsigned short  iReg74[2]               = { 0 };
unsigned short  iReg75[2]               = { 0 };
unsigned short  iRegD9[2]               = { 0 };
unsigned short  iRegDA[2]               = { 0 };
unsigned short  iRegDD[2]               = { 0 };

unsigned short  savedReg74              = 0;
unsigned short  savedReg75              = 0;
unsigned short  savedRegD9              = 0;
unsigned short  savedRegDA              = 0;
unsigned short  savedRegDD              = 0;

/*** 
		U T I L I T I E S   F U C T I O N S 
***/
/*=========================================================
 This routine is used internally
 in:   none
 =========================================================*/
void ResetSeq(void)
{
  unsigned char reg_70;
  outb_p(0x70, IGSEXTINDEX);
  reg_70 = inb_p(IGSEXTDATA) | 0x80;
  outb_p(reg_70, IGSEXTDATA);
  reg_70 &= 0x7F;
  outb_p(reg_70, IGSEXTDATA);
}

/*=========================================================
 This routine is used internally
 in:   none

  Byte OUTPORT with bit mask
  in: bValue=bit/bits to be set/clear, untouched bit/bits must be zero
      bMask=bit/bits to be masked, 1:mask, 0:to be set/clear
  e.g. set <4,3>=<1,0>, bValue=0x10 and bMask=E7
 =========================================================*/
void Out_Video_Reg_M(	unsigned char bIndex, 
											unsigned char bValue, 
											unsigned char bMask)
{
  outb_p(bIndex, IGSEXTINDEX);
  outb_p((inb_p(IGSEXTDATA) & bMask) | bValue, IGSEXTDATA);
}

/*=========================================================
 This routine is used internally
 in:   none
 =========================================================*/
void Out_Video_Reg(unsigned char bIndex, unsigned char bValue)
{
  outb_p(bIndex, IGSEXTINDEX);
  outb_p(bValue, IGSEXTDATA);

}

/*=========================================================
 This routine is used internally
 in:   none
 =========================================================*/
unsigned char In_Video_Reg (unsigned char bIndex) {
    outb_p(bIndex, IGSEXTINDEX);
    return inb_p(IGSEXTDATA);
}

/*=========================================================
 This routine is used internally
 in:
 =========================================================*/
unsigned char ReadReg(unsigned short wReg, unsigned char bIndex)
{
  unsigned char bReturn;

   outb_p(bIndex, wReg);
   bReturn = inb_p(wReg+1);

  return bReturn;
}

/*=========================================================
 This routine is used internally
 in:
 =========================================================*/
void IGS_SetFrameFreezeAsync (int OnOff)
{
	if (OnOff == ON)		/*note Freeze is On (i.e capture is off) */
	{
		/* turn on sync freeze bits */
		Out_Video_Reg_M (CAPTURE_MODE_II, 0x80, 0x7f);
	}
	else
	{
		/* turn them both off */
		Out_Video_Reg_M (CAPTURE_MODE_II, 0x00, 0x7f);
	}
}

/*=========================================================
 This routine is used internally
 in:
 =========================================================*/
int IGS_IsFreezeDone(void) {
	outb_p (VIDEO_ROM_UCB4G_HI, IGSEXTINDEX);
	return (inb_p (IGSEXTDATA) & 0x02);
}

/*=========================================================
	Initialize, Expanding or Relocate Temp region
		nDataSize - the desired temp region size
		return 0 if success
		return -1 if fails
 =========================================================*/
int Set_Temp_Region(int nDataSize)
{
	int nReturnValue = 0;
	unsigned long ulTemp = 0L;
	int nRegion = -1;

	if (0 == VDev.gAMI.gulDisplayStart[0])
	{
#ifdef DEBUG
printk("igs.c: No display region has been initialized yet.\n");
#endif
		/* No display region has been initialized yet */
		ulTemp = (VDev.gAMI.gulAvailableMemoryStart +
							VDev.gAMI.gunTotalAvailableMemory) - VDev.gAMI.gulCaptureEnd;

		if (ulTemp >= nDataSize)
		{
			VDev.gAMI.gulTempStart = VDev.gAMI.gulCaptureEnd;
			VDev.gAMI.gulTempSize = nDataSize;
			VDev.gAMI.gulTempEnd = VDev.gAMI.gulTempStart + VDev.gAMI.gulTempSize;
#ifdef DEBUG
printk("igs.c: TEMP start: %ld\n", VDev.gAMI.gulTempStart);
printk("igs.c: TEMP end: %ld\n", VDev.gAMI.gulTempEnd);
printk("igs.c: TEMP size: %ld\n", VDev.gAMI.gulTempSize);
#endif
		}
		else
		{
#ifdef DEBUG
printk("igs.c: Not enough memory to set TEMP region.\n");
#endif
			nReturnValue = -1;
		}
	}
	else /* some display region has already been initialized */
	{
		for (nRegion = (MAX_NUM_OF_DISPLAY_REGION-1); nRegion>=0; nRegion--)
		{
			if (0 != VDev.gAMI.gulDisplayStart[nRegion])
			{
#ifdef DEBUG
printk("igs.c: Some display region has already been initialized.\n");
printk("igs.c: Region %d is the last initialized display region.\n", nRegion);
#endif
				/* Found the last initialized region, do we have enough room? */
				if ((VDev.gAMI.gulDisplayStart[nRegion] - VDev.gAMI.gulCaptureEnd) >=
						nDataSize)
				{
					VDev.gAMI.gulTempStart = VDev.gAMI.gulCaptureEnd;
					VDev.gAMI.gulTempSize = nDataSize;
					VDev.gAMI.gulTempEnd = VDev.gAMI.gulTempStart + VDev.gAMI.gulTempSize;
#ifdef DEBUG
printk("igs.c: TEMP start: %ld\n", VDev.gAMI.gulTempStart);
printk("igs.c: TEMP end: %ld\n", VDev.gAMI.gulTempEnd);
printk("igs.c: TEMP size: %ld\n", VDev.gAMI.gulTempSize);
#endif
				}
				else
				{
#ifdef DEBUG
printk("igs.c: Not enough memory to set TEMP region.\n");
#endif
					nReturnValue = -1;
				}
				break;
			}
		}
	}

	return (nReturnValue);
}

/*=========================================================
	Initialize Display region
	If succeed, global values will be changed.
	If failed, global values will not be changed.
		nMinor - which video window are we talking about?
		nDataSize - the desired display region size
		return 0 if success
		return -1 if fails
 =========================================================*/
int Init_Display_Region(int nMinor, int nDataSize)
{
	int nReturnValue = 0;
	unsigned long ulTemp = 0L;
	unsigned long ulEndPointer = 0L;
	int nRegion = -1;

	/* If we have temp region, we should check for temp region end pointer 
			instead of capture region end pointer */
	if (0 == (ulEndPointer = VDev.gAMI.gulTempEnd))
	{
		ulEndPointer = VDev.gAMI.gulCaptureEnd;
	}

	for (nRegion = (MAX_NUM_OF_DISPLAY_REGION-1); nRegion>=0; nRegion--)
	{
		if (0 != VDev.gAMI.gulDisplayStart[nRegion])
		{
			if (MAX_NUM_OF_DISPLAY_REGION == nRegion+1)
			{
#ifdef DEBUG
printk("igs.c: All display regions are already initialized.\n");
#endif
				nReturnValue = -1;
				break;
			}
			else
			{
#ifdef DEBUG
printk("igs.c: Some display regions has already been initialized.\n");
printk("igs.c: Region %d is the last initialized display region.\n", nRegion);
#endif
				/* Found the last initialized region, do we have enough room? */
				if ((VDev.gAMI.gulDisplayStart[nRegion] - ulEndPointer) >=
						nDataSize)
				{
					VDev.gAMI.gulDisplayStart[nRegion+1] = 
															VDev.gAMI.gulDisplayStart[nRegion] - nDataSize;
					VDev.gAMI.gulRegionSize[nRegion+1] = nDataSize;
					VDev.gDWinRegion[nMinor] = nRegion+1;	
#ifdef DEBUG
printk("igs.c: Display region %d start: %ld\n", nRegion+1,
				VDev.gAMI.gulDisplayStart[nRegion+1]);
printk("igs.c: Display region %d size: %ld\n", nRegion+1, 
				VDev.gAMI.gulRegionSize[nRegion+1]);
printk("igs.c: /dev/video%d is now associated to region %d.\n", nMinor,
				nRegion+1);
#endif

					break;
				}
				else
				{
#ifdef DEBUG
printk("igs.c: Not enough memory to initialize a display region.\n");
#endif
					nReturnValue = -1;
					break;
				}
			}
		}
		else if (0 == nRegion)	/* No region has been initialized yet */
		{
#ifdef DEBUG
printk("igs.c: No display region has been initialized yet.\n");
#endif
			ulTemp = (VDev.gAMI.gulAvailableMemoryStart + 
								VDev.gAMI.gunTotalAvailableMemory) - nDataSize;
			if (ulTemp >= ulEndPointer)
			{
				VDev.gAMI.gulDisplayStart[0] = ulTemp;
				VDev.gAMI.gulRegionSize[0] = nDataSize;
				VDev.gDWinRegion[nMinor] = 0;	
#ifdef DEBUG
printk("igs.c: Display region 0 starts: %ld\n", VDev.gAMI.gulDisplayStart[0]);
printk("igs.c: Display region 0 size: %ld\n", VDev.gAMI.gulRegionSize[0]);
printk("igs.c: /dev/video%d is now associated to region 0.\n", nMinor);
#endif
				break;
			}
			else
			{
#ifdef DEBUG
printk("igs.c: Not enough memory to initialize a display region.\n");
#endif
				nReturnValue = -1;
				break;
			}
		}
	}

	return (nReturnValue);
}

/*=========================================================
	Expanding Display region
	If succeed, global values will be changed.
	If failed, global values will not be changed.
		nMinor - which video window are we talking about?
		nNewDataSize - the desired display region size
		return 0 if success
		return -1 if fails
 =========================================================*/
int Expand_Display_Region(int nMinor, int nNewDataSize)
{
	int nReturnValue = 0;
	int nCurrentRegion = VDev.gDWinRegion[nMinor];
	int nRegion = 0;
	unsigned long ulSizeNeeded = 0L;
	unsigned long ulEndPointer = 0L;
	unsigned long ulTemp       = 0L;

	/* If we have temp region, we should check for temp region end pointer 
			instead of capture region end pointer */
	if (0 == (ulEndPointer = VDev.gAMI.gulTempEnd))
	{
		ulEndPointer = VDev.gAMI.gulCaptureEnd;
	}

	/* If the higher display regions has not been initialized,
			expansion will need to consider the capture region ending
			location or temp region ending location as well.
	*/
	if (MAX_NUM_OF_DISPLAY_REGION == nCurrentRegion+1)
	{
#ifdef DEBUG
printk("igs.c: Expanding highest display region #%d\n", nCurrentRegion);
#endif
		ulTemp = nNewDataSize - VDev.gAMI.gulRegionSize[nCurrentRegion];

		if ((VDev.gAMI.gulDisplayStart[nCurrentRegion] - ulTemp) < ulEndPointer)
		{
#ifdef DEBUG
printk("igs.c: Failed to expand highest region.\n");
#endif
			nReturnValue = -1;
		}
		else
		{
			VDev.gAMI.gulDisplayStart[nCurrentRegion] -= ulTemp;
			VDev.gAMI.gulRegionSize[nCurrentRegion] = nNewDataSize;
#ifdef DEBUG
printk("igs.c: region #%d new starting location %ld\n", nCurrentRegion,
        VDev.gAMI.gulDisplayStart[nCurrentRegion]);
printk("igs.c: region #%d new size %ld\n", nCurrentRegion,
        VDev.gAMI.gulRegionSize[nCurrentRegion]);
#endif
		}
	}
	else if (0 == VDev.gAMI.gulDisplayStart[nCurrentRegion + 1])
	{
#ifdef DEBUG
printk("igs.c: expanding highest display region #%d\n", nCurrentRegion);
#endif
		ulTemp = nNewDataSize - VDev.gAMI.gulRegionSize[nCurrentRegion];

		if ((VDev.gAMI.gulDisplayStart[nCurrentRegion] - ulTemp) < ulEndPointer)
		{
#ifdef DEBUG
printk("igs.c: Failed to expand lower regions.\n");
#endif
			nReturnValue = -1;
		}
		else
		{
			VDev.gAMI.gulDisplayStart[nCurrentRegion] -= ulTemp;
			VDev.gAMI.gulRegionSize[nCurrentRegion] = nNewDataSize;
#ifdef DEBUG
printk("igs.c: region #%d new starting location %ld\n", nCurrentRegion,
        VDev.gAMI.gulDisplayStart[nCurrentRegion]);
printk("igs.c: region #%d new size %ld\n", nCurrentRegion,
        VDev.gAMI.gulRegionSize[nCurrentRegion]);
#endif
		}
	}
	else /* Higher display regions have already been initialized */
	{
		ulSizeNeeded = nNewDataSize - VDev.gAMI.gulRegionSize[nCurrentRegion];	
		nRegion = 2;
		while(nRegion >= 0)
		{
			if (VDev.gAMI.gulDisplayStart[nRegion] != 0)
			{
#ifdef DEBUG
printk("igs.c: The highest initialized region is: %d\n", nRegion);
#endif
				if ((VDev.gAMI.gulDisplayStart[nRegion] - ulSizeNeeded) < ulEndPointer)
				{
#ifdef DEBUG
printk("igs.c: Cannot expand display region.\n");
#endif
					nReturnValue = -1;
					break;
				}
				else	/* Reset the pointers */
				{
					VDev.gAMI.gulDisplayStart[nRegion] -= ulSizeNeeded;
					if (nRegion == nCurrentRegion)
					{
						VDev.gAMI.gulRegionSize[nRegion] = nNewDataSize;
#ifdef DEBUG
printk("igs.c: Region %d new start address: %ld\n", nRegion,
				VDev.gAMI.gulDisplayStart[nRegion]);
printk("igs.c: Region %d new region size: %ld\n", nRegion,
				VDev.gAMI.gulRegionSize[nRegion]);
#endif
						break;
					}
#ifdef DEBUG
printk("igs.c: Region %d new start address: %ld\n", nRegion,
				VDev.gAMI.gulDisplayStart[nRegion]);
printk("igs.c: Region %d new region size: %ld\n", nRegion,
				VDev.gAMI.gulRegionSize[nRegion]);
#endif
				}
			}
			nRegion--;
		}

		if (-1 != nReturnValue)
		{
			/* Setting new memory start address for the affected regions */
			for (nRegion=0; nRegion < MAX_NUM_OF_DISPLAY_REGION; nRegion++)
			{
				if (-1 != VDev.gDWinRegion[nRegion])
				{
					switch(nRegion)
					{
						case 0:
#ifdef DEBUG
printk("igs.c: Resetting memory start address for MAIN window.\n");
#endif
							IGS_AdjustVideoWindow(VDev.gDisplayWindow[0].dest_win.x,
																		VDev.gDisplayWindow[0].dest_win.y,
																		VDev.gDisplayWindow[0].dest_win.width,
																		VDev.gDisplayWindow[0].dest_win.height);
						break;

						case 1:
#ifdef DEBUG
printk("igs.c: Resetting memory start address for V2 window.\n");
#endif
							IGS_AdjustVideoWindow_V2(	VDev.gDisplayWindow[1].dest_win.x,
																				VDev.gDisplayWindow[1].dest_win.y,
																				VDev.gDisplayWindow[1].dest_win.width,
																				VDev.gDisplayWindow[1].dest_win.height);
						break;

						case 2:
#ifdef DEBUG
printk("igs.c: Resetting memory start address for X2 window.\n");
#endif
							IGS_AdjustVideoWindow_X2(	VDev.gDisplayWindow[2].dest_win.x,
																				VDev.gDisplayWindow[2].dest_win.y,
																				VDev.gDisplayWindow[2].dest_win.width,
																				VDev.gDisplayWindow[2].dest_win.height);
						break;
					}
				}
			}
		}
	}
	
	return (nReturnValue);
}

/*=========================================================
	Scale down a large video to a smaller one
	Source region is the TEMP region.
	Destination region is the corresponding video window
		indicated by minor number.
	nMinor - which video window are we refering to
	Note: Scale down can only be done properly in the 
				standard resolutions:
				352x288 -> 176x144 -> 88x72 -> 44x36 -> 22x18
				352x240 -> 176x120 -> 88x60 -> 44x30 -> 22x15
				176x144 ->  88x72  -> 44x36 -> 22x18
				128x96  ->  64x48  -> 32x24
				 88x72  ->  44x36  -> 22x18
				 44x36  ->  cannot be scaled down anymore
 =========================================================*/
void Corel_ScaleDown(int nMinor)
{
	int loopcount = 0;
	int i = 0;
	int bytecount = VDev.gInputBPP[nMinor];
	int sim_bytes_per_pixel = 0;
	unsigned long ulSimSrcStart = 0L;
	unsigned long ulSimDstStart = 0L;

	/* The following codes are needed because there's problem scaling down
			YUYV formated data under 24bit color mode.  Quick work around is
			to simulate scaling down the YUYV data in 16 bit color mode */
	if ((VIDEO_PALETTE_YUYV == VDev.gDisplayWindow[nMinor].SrcPalette) &&
			(3 == VDev.gScreen.bytes_per_pixel))
	{
		sim_bytes_per_pixel = 2;		/* simulate 16 bit color mode */
		ulSimSrcStart = VDev.gAMI.gulTempStart /2;
		ulSimDstStart = VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[nMinor]] /2;
	}
	else
	{
		sim_bytes_per_pixel = VDev.gScreen.bytes_per_pixel;
		ulSimSrcStart = VDev.gAMI.gulTempStart;
		ulSimDstStart = VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[nMinor]];
	}

	Set_IGS_Px_Map( VDev.gScreen.x_ext-1,
									VDev.gScreen.bytes_per_pixel-1);
	Corel_Set_IGS_Px_Map((VDev.gnScaleDownFactor[nMinor]*bytecount - 1),
												bytecount-1,	
												sim_bytes_per_pixel-1);
	loopcount = (VDev.gDisplayWindow[nMinor].SrcWidth *
							VDev.gDisplayWindow[nMinor].SrcHeight) /
							6020;	// = 86 * 70
							//(88 * 72);
							//(44 * 36 * 2);

#if 0 	// original
	for (i = 0; i< loopcount; i++)
#endif 
	for (i = 0; i < (loopcount*(sim_bytes_per_pixel - 1)); i++)
	{
		Corel_Setup_SrcCopy(ulSimSrcStart, ulSimDstStart);	
		//Corel_SrcCopy(0, i*395, 0, i*395, 2-1, 790-1);	
		Corel_SrcCopy(0, i*790, 0, i*790, bytecount-1, 1580-1);	
	}

	Set_IGS_Px_Map( VDev.gScreen.x_ext-1,
									VDev.gScreen.bytes_per_pixel-1);
}

/*=========================================================
	First kind of PIP
	Source region: one of the three video window (nMinor).
	Destination region: the capture region.
	nMinor - which video window are we referring to
	Note - first kind of PIP can only support input data type
					of YUYV, since the background data is always YUYV
 =========================================================*/
void Corel_PIP1(int nMinor)
{
	int nFactor1 = 1;
	int nFactor2 = 1;

	if (3 == VDev.gScreen.bytes_per_pixel)
	{
		nFactor1 = 2;
		nFactor2 = 3;
	}

	Corel_Set_IGS_Px_Map(	VDev.gnActualCaptureWidth * nFactor1 - 1,
												VDev.gnActualCaptureWidth * nFactor1 - 1,
												VDev.gScreen.bytes_per_pixel-1);

	Corel_Setup_SrcCopy(VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[nMinor]],
											(VDev.gAMI.gulCaptureStart +
											(VDev.gnActualCaptureWidth * 2 *
											VDev.gnGarbageLines)));

	Corel_SrcCopy_1(0, 0,
									VDev.gPIP[nMinor].x * nFactor1,
									VDev.gPIP[nMinor].y,
									(VDev.gPIP[nMinor].width * nFactor1) / nFactor2 -1,
									VDev.gPIP[nMinor].height-1);

	Set_IGS_Px_Map( VDev.gScreen.x_ext-1,
									VDev.gScreen.bytes_per_pixel-1);
}

/*=========================================================
	Second kind of PIP
	Source region:
		background - TEMP region.
		foreground - one of the video window (nMinor)
	Destination region:
		background - one of the video window (nMinor) 
		foreground - all the video windows which are background
									of PIP2.
	nMinor - which video window are we referring to.
	order - order=0 background, order=1 foreground
 =========================================================*/
void Corel_PIP2(int nMinor, int order)
{
	int nTemp    = 0;
	int nFactor1 = 0;
	int nFactor2 = 0;
	int nFactor3 = 0;
	int nNumber  = 0;

	if (order)	/* foreground */
	{
		switch(VDev.gVideoPicture.palette)
		{
			case VIDEO_PALETTE_YUYV:
			case VIDEO_PALETTE_RGB565:
			{
				if (2 == VDev.gScreen.bytes_per_pixel)				/* 16bpp color mode */
				{
					nFactor1 = 1;
					nFactor2 = 1;
					nFactor3 = 1;
				}
				else if (3 == VDev.gScreen.bytes_per_pixel)		/* 24bpp color mode */
				{
					nFactor1 = 2;
					nFactor2 = 1;
					nFactor3 = 3;
					nNumber  = 2;
				}
			}
			break;
		
			case VIDEO_PALETTE_RGB24:
			{
				if (2 == VDev.gScreen.bytes_per_pixel)				/* 16bpp color mode */
				{
					nFactor1 = 3;
					nFactor2 = 2;
					nFactor3 = 1;
				}
				else if (3 == VDev.gScreen.bytes_per_pixel)		/* 24bpp color mode */
				{
					nFactor1 = 3;
					nFactor2 = 1;
					nFactor3 = 3;
					nNumber  = 2;
				}
			}
			break;
		}

		for (nTemp = 0; nTemp < 3; nTemp++)
		{
			if (!VDev.gPIP[nTemp].order && VDev.gnPIPFlag[nTemp])
			{
				Corel_Set_IGS_Px_Map(	VDev.gDisplayWindow[nTemp].SrcWidth * nFactor1 /
															nFactor2 -1,
															VDev.gDisplayWindow[nTemp].SrcWidth * nFactor1 /
															nFactor2 -1,
															VDev.gScreen.bytes_per_pixel-1);
				Corel_Setup_SrcCopy(VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[nMinor]],
														VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[nTemp]]);
				Corel_SrcCopy_1(0, 0,
												VDev.gPIP[nTemp].x * nFactor1 / nFactor2,
												VDev.gPIP[nTemp].y + nNumber,
												VDev.gPIP[nTemp].width * nFactor1 / nFactor2 
												/ nFactor3 -1,
												VDev.gPIP[nTemp].height-1 - nNumber);
				Set_IGS_Px_Map( VDev.gScreen.x_ext-1,
												VDev.gScreen.bytes_per_pixel-1);
			}
		}
	}
	else	/* background */
	{
		switch(VDev.gVideoPicture.palette)
		{
			case VIDEO_PALETTE_YUYV:
			case VIDEO_PALETTE_RGB565:
			{
				if (2 == VDev.gScreen.bytes_per_pixel)				/* 16bpp color mode */
				{
					nFactor1 = 1;
					nFactor2 = 1;
					nFactor3 = 1;
				}
				else if (3 == VDev.gScreen.bytes_per_pixel)		/* 24bpp color mode */
				{
					nFactor1 = 2;
					nFactor2 = 1;
					nFactor3 = 3;
				}
			}
			break;
		
			case VIDEO_PALETTE_RGB24:
			{
				if (2 == VDev.gScreen.bytes_per_pixel)				/* 16bpp color mode */
				{
					nFactor1 = 3;
					nFactor2 = 2;
					nFactor3 = 1;
				}
				else if (3 == VDev.gScreen.bytes_per_pixel)		/* 24bpp color mode */
				{
					nFactor1 = 3;
					nFactor2 = 1;
					nFactor3 = 3;
				}
			}
			break;
		}

		Corel_Set_IGS_Px_Map(	VDev.gDisplayWindow[nTemp].SrcWidth * nFactor1 /
													nFactor2 -1,
													VDev.gDisplayWindow[nTemp].SrcWidth * nFactor1 /
													nFactor2 -1,
													VDev.gScreen.bytes_per_pixel-1);

		if (VDev.gPIP[nMinor].y > 1)
		{
			Corel_Setup_SrcCopy(VDev.gAMI.gulTempStart,
													VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[nMinor]]);
			Corel_SrcCopy_1(0, 0, 0, 0,
											(VDev.gDisplayWindow[nMinor].SrcWidth * nFactor1) /
											nFactor2 -1,
											VDev.gPIP[nMinor].y-1);
		}

		if (VDev.gPIP[nMinor].x > 1)
		{
			Corel_Setup_SrcCopy(VDev.gAMI.gulTempStart,
													VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[nMinor]]);
			Corel_SrcCopy_1(0, VDev.gPIP[nMinor].y,
											0, VDev.gPIP[nMinor].y,
											(VDev.gPIP[nMinor].x * nFactor1) / nFactor2 / nFactor3 -1,
											VDev.gPIP[nMinor].height-1);
		}

		nTemp = VDev.gPIP[nMinor].x + VDev.gPIP[nMinor].width;
		if (nTemp < VDev.gDisplayWindow[nMinor].SrcWidth)
		{
			Corel_Setup_SrcCopy(VDev.gAMI.gulTempStart,
													VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[nMinor]]);
			Corel_SrcCopy_1(nTemp * nFactor1 / nFactor2, 
											VDev.gPIP[nMinor].y, 
											nTemp * nFactor1 / nFactor2, 
											VDev.gPIP[nMinor].y,
											(VDev.gDisplayWindow[nMinor].SrcWidth - nTemp) * nFactor1/
											nFactor2 / nFactor3 - 1,
											VDev.gPIP[nMinor].height-1);
		}

		nTemp = VDev.gPIP[nMinor].y + VDev.gPIP[nMinor].height;
		if (nTemp < VDev.gDisplayWindow[nMinor].SrcHeight)
		{
			Corel_Setup_SrcCopy(VDev.gAMI.gulTempStart,
													VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[nMinor]]);
			Corel_SrcCopy_1(0, nTemp, 0, nTemp,
											(VDev.gDisplayWindow[nMinor].SrcWidth * nFactor1) /
											nFactor2 / nFactor3 -1,
											VDev.gDisplayWindow[nMinor].SrcHeight - nTemp -1);
		}

		Set_IGS_Px_Map( VDev.gScreen.x_ext-1,
										VDev.gScreen.bytes_per_pixel-1);
	}
}

/*=========================================================
	Icon Video
	Source region: one of the video window (nMinor)
	Destination region: 
		base display window (VDev.gIconBase)
		particular icon (VDev.gIconCurrent)
	nMinor - which video window are we referring to.
 =========================================================*/
void Corel_Icon(int nMinor)
{
	int y = 0;
	int nFactor1 = 0;
	int nFactor2 = 0;
	int nFactor3 = 0;

	switch(VDev.gVideoPicture.palette)
	{
		case VIDEO_PALETTE_YUYV:
		case VIDEO_PALETTE_RGB565:
		{
			if (2 == VDev.gScreen.bytes_per_pixel)				/* 16bpp color mode */
			{
				nFactor1 = 1;
				nFactor2 = 1;
				nFactor3 = 1;
			}
			else if (3 == VDev.gScreen.bytes_per_pixel)		/* 24bpp color mode */
			{
				nFactor1 = 2;
				nFactor2 = 1;
				nFactor3 = 3;
			}
		}
		break;
	
		case VIDEO_PALETTE_RGB24:
		{
			if (2 == VDev.gScreen.bytes_per_pixel)				/* 16bpp color mode */
			{
				nFactor1 = 3;
				nFactor2 = 2;
				nFactor3 = 1;
			}
			else if (3 == VDev.gScreen.bytes_per_pixel)		/* 24bpp color mode */
			{
				nFactor1 = 3;
				nFactor2 = 1;
				nFactor3 = 3;
			}
		}
		break;
	}

	Corel_Set_IGS_Px_Map(	VDev.gDisplayWindow[nMinor].SrcWidth * nFactor1 /
												nFactor2 - 1,
												VDev.gDisplayWindow[VDev.gIconBase].SrcWidth*nFactor1 /
												nFactor2 - 1,
												VDev.gScreen.bytes_per_pixel-1);

	Corel_Setup_SrcCopy(VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[nMinor]],
									VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[VDev.gIconBase]]);

	y = (VDev.gIconCurrent-1) * (VDev.gIcon.height + 0); //2);

	Corel_SrcCopy_1(0, 0, 0, y, 
									VDev.gIcon.width * nFactor1 / nFactor2 / nFactor3 -1, 
									VDev.gIcon.height -1);

	Set_IGS_Px_Map(VDev.gScreen.x_ext-1, VDev.gScreen.bytes_per_pixel-1);
}

/*=========================================================
	Corel_DisplayFreeze

	Source region: 			TEMP region
	Destination region: One of the video window (nMinor)
	Parameters:
		nMinor - which video window are we referring to.
		type   - 0 - selected area freezed.
						 1 - surrounding area freezed.
 =========================================================*/
void Corel_DisplayFreeze(int nMinor, int type)
{
	int nTemp = 0;
	int nActualSrcWidth = 0;
	int nActualFreezeX = 0;
	int nActualFreezeWidth = 0;
	int nFactor1 = 0;
	int nFactor2 = 0;
	int nFactor3 = 0;

	if (type)	/* type 1 */
	{
		switch(VDev.gVideoPicture.palette)
		{
			case VIDEO_PALETTE_YUYV:
			case VIDEO_PALETTE_RGB565:
			{
				if (2 == VDev.gScreen.bytes_per_pixel)	/* 16bpp color mode */
				{
					nActualSrcWidth = VDev.gDisplayWindow[nMinor].SrcWidth;
					nActualFreezeX = VDev.gVideoFreeze[nMinor].x;
					nActualFreezeWidth = VDev.gVideoFreeze[nMinor].width;
				}
				else if (3 == VDev.gScreen.bytes_per_pixel)		/* 24bpp color mode */
				{
					nActualSrcWidth = VDev.gDisplayWindow[nMinor].SrcWidth * 2;
					nActualFreezeX = VDev.gVideoFreeze[nMinor].x * 2;
					nActualFreezeWidth = (VDev.gVideoFreeze[nMinor].width * 2) /3;
				}
			}
			break;
		
			case VIDEO_PALETTE_RGB24:
			{
				if (2 == VDev.gScreen.bytes_per_pixel)	/* 16bpp color mode */
				{
					nActualSrcWidth = (VDev.gDisplayWindow[nMinor].SrcWidth * 3) /2;
					nActualFreezeX = (VDev.gVideoFreeze[nMinor].x * 3) /2;
					nActualFreezeWidth = (VDev.gVideoFreeze[nMinor].width * 3) /2;
				}
				else if (3 == VDev.gScreen.bytes_per_pixel)		/* 24bpp color mode */
				{
					nActualSrcWidth = VDev.gDisplayWindow[nMinor].SrcWidth * 3;
					nActualFreezeX = VDev.gVideoFreeze[nMinor].x * 3;
					nActualFreezeWidth = VDev.gVideoFreeze[nMinor].width;
				}
			}
			break;
		}

		Corel_Set_IGS_Px_Map(	nActualSrcWidth-1, nActualSrcWidth-1, 
													VDev.gScreen.bytes_per_pixel-1);
		Corel_Setup_SrcCopy(VDev.gAMI.gulTempStart,
												VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[nMinor]]);
		Corel_SrcCopy_1(//VDev.gVideoFreeze[nMinor].x,
										nActualFreezeX,
										VDev.gVideoFreeze[nMinor].y,
										//VDev.gVideoFreeze[nMinor].x,
										nActualFreezeX,
										VDev.gVideoFreeze[nMinor].y,
										//VDev.gVideoFreeze[nMinor].width-1,
										nActualFreezeWidth-1,
										VDev.gVideoFreeze[nMinor].height-1);
		Set_IGS_Px_Map( VDev.gScreen.x_ext-1,
										VDev.gScreen.bytes_per_pixel-1);
	}
	else	/* type 0 */
	{
		switch(VDev.gVideoPicture.palette)
		{
			case VIDEO_PALETTE_YUYV:
			case VIDEO_PALETTE_RGB565:
			{
				if (2 == VDev.gScreen.bytes_per_pixel)	/* 16bpp color mode */
				{
					nFactor1 = 1;
					nFactor2 = 1;
					nFactor3 = 1;
				}
				else if (3 == VDev.gScreen.bytes_per_pixel)		/* 24bpp color mode */
				{
					nFactor1 = 2;
					nFactor2 = 1;
					nFactor3 = 3;
				}
			}
			break;
		
			case VIDEO_PALETTE_RGB24:
			{
				if (2 == VDev.gScreen.bytes_per_pixel)	/* 16bpp color mode */
				{
					nFactor1 = 3;
					nFactor2 = 2;
					nFactor3 = 1;
				}
				else if (3 == VDev.gScreen.bytes_per_pixel)		/* 24bpp color mode */
				{
					nFactor1 = 3;
					nFactor2 = 1;
					nFactor3 = 3;
				}
			}
			break;
		}

		Corel_Set_IGS_Px_Map(	(VDev.gDisplayWindow[nMinor].SrcWidth*nFactor1) /
													nFactor2-1, 
													(VDev.gDisplayWindow[nMinor].SrcWidth*nFactor1) /
													nFactor2-1, 
													VDev.gScreen.bytes_per_pixel-1);

		if (VDev.gVideoFreeze[nMinor].y > 1)
		{
			Corel_Setup_SrcCopy(VDev.gAMI.gulTempStart,
													VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[nMinor]]);
			Corel_SrcCopy_1(0, 0, 0, 0,
											(VDev.gDisplayWindow[nMinor].SrcWidth*nFactor1) /
											nFactor2 - 1,
											VDev.gVideoFreeze[nMinor].y-1);
		}

		if (VDev.gVideoFreeze[nMinor].x > 1)
		{
			Corel_Setup_SrcCopy(VDev.gAMI.gulTempStart,
													VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[nMinor]]);
			Corel_SrcCopy_1(0, VDev.gVideoFreeze[nMinor].y,
											0, VDev.gVideoFreeze[nMinor].y,
											(VDev.gVideoFreeze[nMinor].x * nFactor1)/ nFactor2 /
											nFactor3 -1,
											VDev.gVideoFreeze[nMinor].height-1);
		}

		nTemp = VDev.gVideoFreeze[nMinor].x + VDev.gVideoFreeze[nMinor].width;
		if (nTemp < VDev.gDisplayWindow[nMinor].SrcWidth)
		{
			Corel_Setup_SrcCopy(VDev.gAMI.gulTempStart,
													VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[nMinor]]);
			Corel_SrcCopy_1(nTemp * nFactor1 / nFactor2,
											VDev.gVideoFreeze[nMinor].y, 
											nTemp * nFactor1 / nFactor2, 
											VDev.gVideoFreeze[nMinor].y,
											(VDev.gDisplayWindow[nMinor].SrcWidth - nTemp)*nFactor1/
											nFactor2/nFactor3 -1,
											VDev.gVideoFreeze[nMinor].height-1);
		}

		nTemp = VDev.gVideoFreeze[nMinor].y + VDev.gVideoFreeze[nMinor].height;
		if (nTemp < VDev.gDisplayWindow[nMinor].SrcHeight)
		{
			Corel_Setup_SrcCopy(VDev.gAMI.gulTempStart,
													VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[nMinor]]);
			Corel_SrcCopy_1(0, nTemp, 0, nTemp,
											(VDev.gDisplayWindow[nMinor].SrcWidth*nFactor1)/
											nFactor2/nFactor3 -1,
											VDev.gDisplayWindow[nMinor].SrcHeight - nTemp -1);
		}

		Set_IGS_Px_Map( VDev.gScreen.x_ext-1,
										VDev.gScreen.bytes_per_pixel-1);
	}
}

/*=========================================================
	Corel_ShiftVideo

	Source region: 	One of the video window (source)
	Destination region: Designated region (VDev.gShiftDest)
	Parameters:
		source - where do video data come from?
 =========================================================*/
void Corel_ShiftVideo(int source)
{
	int nFactor1 = 0;
	int nFactor2 = 0;
	int nFactor3 = 0;
	int nNumber  = 0;

	switch(VDev.gVideoPicture.palette)
	{
		case VIDEO_PALETTE_YUYV:
		case VIDEO_PALETTE_RGB565:
		{
			if (2 == VDev.gScreen.bytes_per_pixel)				/* 16bpp color mode */
			{
				nFactor1 = 1;
				nFactor2 = 1;
				nFactor3 = 1;
				nNumber  = 0;
			}
			else if (3 == VDev.gScreen.bytes_per_pixel)		/* 24bpp color mode */
			{
				nFactor1 = 2;
				nFactor2 = 1;
				nFactor3 = 3;
				nNumber  = 1;
			}
		}
		break;
	
		case VIDEO_PALETTE_RGB24:
		{
			if (2 == VDev.gScreen.bytes_per_pixel)				/* 16bpp color mode */
			{
				nFactor1 = 3;
				nFactor2 = 2;
				nFactor3 = 1;
				nNumber  = 0;
			}
			else if (3 == VDev.gScreen.bytes_per_pixel)		/* 24bpp color mode */
			{
				nFactor1 = 3;
				nFactor2 = 1;
				nFactor3 = 3;
				nNumber  = 0;
			}
		}
		break;
	}

	Corel_Set_IGS_Px_Map(	VDev.gDisplayWindow[source].SrcWidth * nFactor1 /
												nFactor2 - 1,
												VDev.gDisplayWindow[VDev.gShiftDest].SrcWidth*nFactor1/
												nFactor2 - 1,
												VDev.gScreen.bytes_per_pixel-1);
	Corel_Setup_SrcCopy(VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[source]],
									VDev.gAMI.gulDisplayStart[VDev.gDWinRegion[VDev.gShiftDest]]);
	Corel_SrcCopy_1(VDev.gVideoShift.x * nFactor1 / nFactor2,
									VDev.gVideoShift.y,
									0, 0,
									VDev.gVideoShift.width * nFactor1 / nFactor2 
									/ nFactor3 -1 + nNumber,
									VDev.gVideoShift.height-1);
	Set_IGS_Px_Map( VDev.gScreen.x_ext-1,
									VDev.gScreen.bytes_per_pixel-1);
}

/***
		I G S   F U N C T I O N S
***/

/*
 *	G E N E R A L   F U N C T I O N S
 */

/* return TRUE if the VGA is in graphics mode */
int IGS_IsGraphics(void)
{
	/* easy: just check bit 0 of 3cf/6 */
	return In_Video_Reg(6) & 1;
}

/**********************************************************
 Setup color key registers
 in: cref - 32 bit values for rgb color, R,G,B,x with R in lower byte

 Note: . This function is only useful when color key format is used.
       . At 256 color mode, inb_put data should be R=G=B.
       . At 16 Bpp mode, inb_put data should be RGB = RGB & 0xF8FCF8
      which is RGB565 format.
 **********************************************************/
void IGS_SetRGBKeyColor (unsigned long cref)
{
  unsigned char bData;

  bData = (unsigned char) cref;
  Out_Video_Reg(COLOR_CMP_RED,  bData);
  bData = (unsigned char) (cref >> 8);
  Out_Video_Reg(COLOR_CMP_GREEN,bData);
  bData = (unsigned char) (cref >> 16);
  Out_Video_Reg(COLOR_CMP_BLUE, bData);
}

/**********************************************************
 Get current graphics mode color depth
 in: none
 return 8 - 8 Bpp (256 color mode)
       16 - 16 Bpp (High color mode)
       24 - 24 Bpp (True color mode)
 **********************************************************/
int GetBpp(void)
{
  unsigned char modeBpp;

  outb_p(ExtSeqMisc, IGSEXTINDEX);
  modeBpp = inb_p(IGSEXTDATA) & 0x07;

  switch (modeBpp)
  {
    case 2:
    case 6: modeBpp = 16; break;
    case 3:
    case 4: modeBpp = 24; break;
    case 1:
    default:modeBpp =  8; break;
  }
  return ((int) modeBpp);
}

/*=============================================
  in:  iScreen is X_SCREEN or Y_SCREEN
  out: return current x_resolution or y_resolution
       (1 based value)
=============================================*/
int GetCurrentScreenSize(int iScreen)
{
  int iDE;         /*H or V Display Enable*/
  unsigned char bCrt01, bCrt12, bCrt07, bExt11;

  if (iScreen == Y_SCREEN)
  {
    outb_p(0x07, CRTINDEX);
    bCrt07 = inb_p(CRTDATA);

    outb_p(0x12, CRTINDEX);
    bCrt12 = inb_p(CRTDATA);

    outb_p(0x11, IGSEXTINDEX);
    bExt11 = inb_p(IGSEXTDATA);

    iDE   =  bCrt12 + (((unsigned short)bExt11 & 0x02) << 9) + ((bCrt07 & 0x02)
<< 7) + ((bCrt07 & 0x40) << 3) + 1;
  }
  else
  {
    outb_p(0x01, CRTINDEX);
    bCrt01 = inb_p(CRTDATA);
    iDE   = ((int)bCrt01 + 1) * 8;
  }

  return iDE;
}

/*=============================================
  in:  None
  out: return current video memory size
       0: 1MB, 1: 2MB, 2: 4MB
=============================================*/
int GetVMemSize(void)
{
  unsigned char bData;
  outb_p(0x72, IGSEXTINDEX);
  bData = inb_p(IGSEXTDATA) & 0x03;  /*0: 1MB, 1: 2MB, 2: 4MB.*/
  return bData;
}

/*=============================================
  This routine is used internally:
  Get Video FIFO and other informations
  return 0: error, 1: success
=============================================*/
int GetVideoFIFO(void)
{
  int nReturnValue = 0;
  int wTableIndex, iMpeg1, iMpeg2;
  unsigned short *wPtr;

  /*save FIFO control regs*/
  outb_p(ExtFIFOReg0, IGSEXTINDEX);
  savedReg74 = ((inb_p(IGSEXTDATA)) << 8) + ExtFIFOReg0;

  outb_p(ExtFIFOReg1, IGSEXTINDEX);
  savedReg75 = ((inb_p(IGSEXTDATA)) << 8) + ExtFIFOReg1;

  outb_p(FIFO_TIMING_CTL_L, IGSEXTINDEX);
  savedRegD9 = ((inb_p(IGSEXTDATA)) << 8) + FIFO_TIMING_CTL_L;

  outb_p(FIFO_TIMING_CTL_H, IGSEXTINDEX);
  savedRegDA = ((inb_p(IGSEXTDATA)) << 8) + FIFO_TIMING_CTL_H;

  outb_p(FIFO_CTL_I, IGSEXTINDEX);
  savedRegDD = ((inb_p(IGSEXTDATA)) << 8) + FIFO_CTL_I;

  /* switch (VDev.gScreen.x_ext) */
	switch (VDev.gScreen.width)
  {
    default:
    case 640:
      wTableIndex = 0;
    break;

    case 800:
      wTableIndex = 1;
    break;

    case 1024:
      wTableIndex = 2;
    break;

    case 1280:
      wTableIndex = 3;
    break;

    case 1600:
      wTableIndex = 4;
    break;
  }

  if (GetVMemSize() == 0)
    iMpeg1 =  (unsigned short)
              ( dwMPEG1_1MB[wTableIndex] >>
                ((VDev.gScreen.bytes_per_pixel * 8) - 8)
              );
  else
    iMpeg1 =  (unsigned short)
              ( dwMPEG1_2MB[wTableIndex] >>
                ((VDev.gScreen.bytes_per_pixel * 8) - 8)
              );

  iMpeg1 &= 0x00FF;   /* got the index to the FIFO table (MPEG1) */

  iMpeg2 =  (unsigned short)
            ( dwMPEG2_2MB[wTableIndex] >>
              ((VDev.gScreen.bytes_per_pixel * 8) - 8)
            );
  iMpeg2 &= 0x00FF;   /* got the index to the FIFO table (MPEG2) */

  /*Copy data to working sets for MPEG1 and MPEG2 size images*/
  if (0x00FF != iMpeg1)
  {
    wPtr = wFIFOTable[iMpeg1]->wFIFOData;

    iVFilterOn[0] = *wPtr++;
    iReg74[0]   = *wPtr++;
    iReg75[0]   = *wPtr++;
    iRegD9[0]   = *wPtr++;
    iRegDA[0]   = *wPtr++;
    iRegDD[0]   = *wPtr++;

    if (0x00FF != iMpeg2)
    {
      wPtr = wFIFOTable[iMpeg2]->wFIFOData;

      iVFilterOn[1] = *wPtr++;
      iReg74[1]   = *wPtr++;
      iReg75[1]   = *wPtr++;
      iRegD9[1]   = *wPtr++;
      iRegDA[1]   = *wPtr++;
      iRegDD[1]   = *wPtr++;

      VDev.gnFullScreenOk = 1;   /* Capture width > 352 pixels is supported */
    }
    else
    {
      VDev.gnFullScreenOk = 0;   /* Capture width > 352 pixels is NOT supported */

      /*  Newly added */
      iVFilterOn[1] = iVFilterOn[0];
      iReg74[1] = iReg74[0];
      iReg75[1] = iReg75[0];
      iRegD9[1] = iRegD9[0];
      iRegDA[1] = iRegDA[0];
      iRegDD[1] = iRegDD[0];
    }

    nReturnValue = 1;     /* Successful */
  }
  else
  {
    nReturnValue = 0;     /* Failed.  No capturing can be supported in this
                             mode. */
  }

  return (nReturnValue);
}

/*=============================================
  This routine is used internally:
  Routine: to change Video FIFO
  in: which=table to be used
=============================================*/
void Change_VideoFIFO( int which )
{
  if (iVFilterOn[which] == 0)
    Out_Video_Reg_M( DISP_CTL_I, 0x20, 0xDF );
  else
    Out_Video_Reg_M( DISP_CTL_I, 0x00, 0xDF );

  outw_p(iReg74[which], IGSEXTINDEX);
  outw_p(iReg75[which], IGSEXTINDEX);
  outw_p(iRegD9[which], IGSEXTINDEX);
  outw_p(iRegDA[which], IGSEXTINDEX);
  outw_p(iRegDD[which], IGSEXTINDEX);
}

/**********************************************************
 Initialize IGS Video Hardware and Decoder to a default state
 in:   none

 Note: This function should not be called before DetectDecoder,
       since there is decoder dependant code in this routine
       This is one time only initialization routine.
 **********************************************************/
void IGS_InitVideo(void)
{
  /* --- Initialize IGS Video Registers --- */

  Out_Video_Reg(DISP_CTL_I, 0x00);

  Out_Video_Reg(VFAC_CTL_MODE_III, 0x00);
  Out_Video_Reg(VFAC_CTL_MODE_II,  0x10);

  outw_p((0x0100 + CAPTURE_BANK_CTL), IGSEXTINDEX);
  outw_p(((iRegD9[0] & 0xFF00) | V_DST_Y1_LO), IGSEXTINDEX);
  outw_p(((iRegDA[0] & 0xFF00) | V_DST_Y1_HI), IGSEXTINDEX);

  Out_Video_Reg(DEST_RECT_LEFT_L, 0x20);
  Out_Video_Reg(DEST_RECT_LEFT_H, 0x00);
  Out_Video_Reg(DEST_RECT_RIGHT_L, 0x60);
  Out_Video_Reg(DEST_RECT_RIGHT_H, 0x01);
  Out_Video_Reg(DEST_RECT_TOP_L, 0x00);
  Out_Video_Reg(DEST_RECT_TOP_H, 0x00);     // <--- default to 0x08 ????
  Out_Video_Reg(DEST_RECT_BOTTOM_L, 0xe0);
  Out_Video_Reg(DEST_RECT_BOTTOM_H, 0x00);

  Out_Video_Reg(MEMORY_OFFSET_PHASE, 0x50);

  Out_Video_Reg(COLOR_CMP_RED,  0x00);      // uses??
  Out_Video_Reg(COLOR_CMP_GREEN,0x00);
  Out_Video_Reg(COLOR_CMP_BLUE, 0x00);

  Out_Video_Reg(VIDEO_FORMAT, 0x00);

  Out_Video_Reg(MISC_CTL_I, 0xc0);
  Out_Video_Reg(MISC_CTL_I, 0x01);

  outw_p((0x0000 + CAPTURE_DDA_X_INIT_L), IGSEXTINDEX);
  outw_p((0x0800 + CAPTURE_DDA_X_INIT_H), IGSEXTINDEX);
  outw_p((0x0000 + CAPTURE_DDA_Y_INIT_L), IGSEXTINDEX);
  outw_p((0x0800 + CAPTURE_DDA_Y_INIT_H), IGSEXTINDEX);

  Out_Video_Reg(DDA_X_INC_L, 0x00);
  Out_Video_Reg(DDA_X_INC_H, 0x10);
  Out_Video_Reg(DDA_Y_INC_L, 0x00);
  Out_Video_Reg(DDA_Y_INC_H, 0x10);

  outw_p((0x3F00 + CAPTURE_NEW_CTL_I), IGSEXTINDEX);

  outw_p((0xC000 + CAPTURE_NEW_CTL_II), IGSEXTINDEX);

  outw_p((0x1000 + CAPTURE_MODE_I), IGSEXTINDEX);

  outw_p((0x5000 + CAPTURE_MODE_II), IGSEXTINDEX);

  outw_p((0x8000 + CAPTURE_TV_CTL), IGSEXTINDEX);

  outw_p((0xB400 + CAPTURE_CTL_MISC), IGSEXTINDEX);

  outw_p((0x0500 + CAPTURE_BANK_CTL), IGSEXTINDEX);
  outw_p((0x4000 + CAPTURE_X2_CTL_I), IGSEXTINDEX);

  // Set Philips to automatic field detection mode
  sendOneByte(P7111Port, 0x08, 0x88);

  switch (VDev.gnWhichDecoder)
  {
    default:
    case PHILIPS7111:
    {
      unsigned char uchRegValue = 0;

      Out_Video_Reg_M(CAPTURE_MODE_I, 0x01, 0xfe);    /*Enable 8 bit video*/
      sendOneByte(P7111Port, 0x10, 0xC8);             /* 7111 - 8 bit*/
                                                      /* YUYV CCIR656 8bits */

      /* Get the status register value from Philips */
      readOneByte(P7111Port, 0x1F, &uchRegValue);

      if (0x20 & uchRegValue)
      {
printk("igs.c: ...NTSC camera detected...\n");
        VDev.gnVideoStandard						= NTSC;
        VDev.gVideoCapability.maxwidth  = MAX_NTSC_CAPTURE_WIDTH;
        VDev.gVideoCapability.maxheight = MAX_NTSC_CAPTURE_HEIGHT;

        /* Setting default clipping area (full image) */
        Out_Video_Reg(CAPTURE_H_START_L, 0x0F);
        Out_Video_Reg(CAPTURE_H_START_H, 0x00);
        Out_Video_Reg(CAPTURE_H_END_L,   0x90);
        Out_Video_Reg(CAPTURE_H_END_H,   0x05);

        Out_Video_Reg(CAPTURE_V_START_L, 0x0A);
        Out_Video_Reg(CAPTURE_V_START_H, 0x00);
        Out_Video_Reg(CAPTURE_V_END_L,   0xFD);
        Out_Video_Reg(CAPTURE_V_END_H,   0x00);
      }
      else
      {
printk("igs.c: ...PAL camera detected...\n");
        VDev.gnVideoStandard            = PAL;
        VDev.gVideoCapability.maxwidth  = MAX_PAL_CAPTURE_WIDTH;
        VDev.gVideoCapability.maxheight = MAX_PAL_CAPTURE_HEIGHT;

        /* Setting default clipping area (full image) */
        Out_Video_Reg(CAPTURE_H_START_L, 0x13);
        Out_Video_Reg(CAPTURE_H_START_H, 0x00);
        Out_Video_Reg(CAPTURE_H_END_L,   0x93);
        Out_Video_Reg(CAPTURE_H_END_H,   0x05);

        Out_Video_Reg(CAPTURE_V_START_L, 0x0B);
        Out_Video_Reg(CAPTURE_V_START_H, 0x00);
        Out_Video_Reg(CAPTURE_V_END_L,   0x2F);
        Out_Video_Reg(CAPTURE_V_END_H,   0x01);
      }
    }
    break;
  }

  Out_Video_Reg( DISP_CTL_I, 0x04 );
}

/*
 *	C A P T U R E   F U N C T I O N S
 */
/*=========================================================
 This routine is used internally
 in:   OnOff = ON or OFF
 Note: This routine is used only at
   1) anytime when we touch capture address registers (0xEB,0xEC,0xED)
   2) anytime when we touch video capture enable register (0xE8)
 =========================================================*/
void IGS_SetFreezeVid (int OnOff)
{
  if (OnOff==ON)  /*note Freeze is On (i.e capture is off)*/
  {
    Out_Video_Reg_M(VFAC_CTL_MODE_I, 0x0C, 0xF3);
    outb_p(VIDEO_ROM_UCB4G_HI, IGSEXTINDEX);
    while ( !(inb_p(IGSEXTDATA) & 0x02) );
  }
  else
  {
    udelay(TimeDelay1);
    Out_Video_Reg_M(VFAC_CTL_MODE_I, 0x00, 0xF3);
  }

  ResetSeq();
}

/**********************************************************
 Setup capture window address (Setup incoming video address)
 in: none

 Note:
    1. It is only necessary to call this routine once.
    2. Please note that the capture address varies for
       different graphics modes and the address is defined in
       the variable dwOffsetOffScreen which must be set by
       application before calling this routine.
    3. dwOffsetOffScreen has been set a default value in
       routine Init_ConfigData,
    4. application can call dwSetOffScreenPtr() to overwite
       the 32 variable dwOffsetOffScreen.
 **********************************************************/
void IGS_SetCaptureAddr(void)
{
  switch (VDev.gnWhichDecoder)
  {
    default:
    case PHILIPS7111:
    {
      if ((VDev.gCaptureWindow.width * 2) <= VDev.gusHighResThreshold)
			{
#ifdef DEBUG
printk("video.c: (Capture) Set MPEG 1 FIFO.\n\n");
#endif
        Change_VideoFIFO(0);        /*set MPEG1 FIFO*/
			}
      else
			{
#ifdef DEBUG
printk("video.c: (Capture) Set MPEG 2 FIFO.\n\n");
#endif
        Change_VideoFIFO(1);        /*set MPEG2 FIFO*/
			}
    }
    break;
  }

	if (NTSC == VDev.gnVideoStandard)
	{
		if (VDev.gCaptureWindow.height <= 240)
		{
      Set_Capture_Scale(MAX_NTSC_CAPTURE_WIDTH,       /* 704 */
                        VDev.gCaptureWindow.width,
                        MAX_NTSC_CAPTURE_HEIGHT / 2,  /* 240 */
                        VDev.gCaptureWindow.height);

      VDev.gnGarbageLines = NTSC_GarbageLine/2;

			Out_Video_Reg_M(VFAC_CTL_MODE_III, 0x00, 0xFD);   /*Interlace bit=0*/
			Out_Video_Reg_M(CAPTURE_CTL_MISC, 0x04, 0xFB);    /*Data Good bit=1*/
		}
		else
		{
      Set_Capture_Scale(MAX_NTSC_CAPTURE_WIDTH,				/* 704 */
                        VDev.gCaptureWindow.width,
                        MAX_NTSC_CAPTURE_HEIGHT,			/* 480 */
                        VDev.gCaptureWindow.height);

      VDev.gnGarbageLines = NTSC_GarbageLine;

			Out_Video_Reg_M(VFAC_CTL_MODE_III, 0x02, 0xFD);		/*Interlace bit=1*/
			Out_Video_Reg_M( CAPTURE_CTL_MISC, 0x00, 0xFB );	/*Data Good bit=0*/
		}
	}
	else if (PAL == VDev.gnVideoStandard)
	{
		if (VDev.gCaptureWindow.height <= 288)
		{
      Set_Capture_Scale(MAX_PAL_CAPTURE_WIDTH,        /* 704 */
                        VDev.gCaptureWindow.width,
                        MAX_PAL_CAPTURE_HEIGHT / 2,   /* 288 */
                        VDev.gCaptureWindow.height);

      VDev.gnGarbageLines = PAL_GarbageLine/2;

			Out_Video_Reg_M(VFAC_CTL_MODE_III, 0x00, 0xFD);   /*Interlace bit=0*/
			Out_Video_Reg_M(CAPTURE_CTL_MISC, 0x04, 0xFB);    /*Data Good bit=1*/
    }
		else
		{
      Set_Capture_Scale(MAX_PAL_CAPTURE_WIDTH,					/* 704 */
                        VDev.gCaptureWindow.width,
                        MAX_PAL_CAPTURE_HEIGHT,					/* 288 */
                        VDev.gCaptureWindow.height);

      VDev.gnGarbageLines = PAL_GarbageLine;

			Out_Video_Reg_M(VFAC_CTL_MODE_III, 0x02, 0xFD);   /*Interlace bit=1*/
			Out_Video_Reg_M(CAPTURE_CTL_MISC, 0x00, 0xFB);		/*Data Good bit=0*/
    }
  }

  Out_Video_Reg(CAPTURE_PITCH_L, (unsigned char)(VDev.gwWidthBytes >> 3) );

  IGS_SetXYMirror(VDev.gnCaptureXMirror, VDev.gnCaptureYMirror);
}

/*=============================================
 This routine is used internally
 in wSrcXExt, wDstXExt = MAX_NTSC_CAPTURE_WIDTH  (640/720)
    wSrcYExt, wDstYExt = 480/2 or 480
===============================================*/
void Set_Capture_Scale( unsigned short wSrcXExt,
                        unsigned short wDstXExt,
                        unsigned short wSrcYExt,
                        unsigned short wDstYExt )
{
  unsigned long dwScale;

  if ( wSrcXExt == wDstXExt )
    dwScale = 4096;
  else
  {
    dwScale = ((unsigned long)(wDstXExt) * 4096) / (unsigned long)wSrcXExt;
  }

  Out_Video_Reg( CAPTURE_DDA_X_INC_L, LOBYTE( (unsigned short)dwScale ) );
  Out_Video_Reg( CAPTURE_DDA_X_INC_H, HIBYTE( (unsigned short)dwScale ) );

  if ( wSrcYExt == wDstYExt )
    dwScale = 4096;
  else
  {
    dwScale = ((unsigned long)wDstYExt * 4096) / (unsigned long)wSrcYExt;
  }

  Out_Video_Reg( CAPTURE_DDA_Y_INC_L, LOBYTE( (unsigned short)dwScale ) );
  Out_Video_Reg( CAPTURE_DDA_Y_INC_H, HIBYTE( (unsigned short)dwScale ) );
}

/**********************************************************
  To mirror capture image and set/reset the capture starting
  address (FOR NTSC only) (PAL code needed to be added)

  in:
  gwXMirror: 1: x-mirror, 0: x-normal
  gwYMirror: 1: y-mirror, 0: y-normal
 **********************************************************/
void IGS_SetXYMirror(int gwXMirror, int gwYMirror)
{
  unsigned long     dwL;

  dwL = VDev.gAMI.gulCaptureStart;

  if (gwYMirror)
  {
    if (VDev.gnActualCaptureHeight > 240)
    {
      dwL +=  (unsigned long)
              (VDev.gnActualCaptureHeight + NTSC_GarbageLine + 2) *
              VDev.gnActualCaptureWidth * 2;
    }
    else
    {
      if (VDev.gnSubCaptureOn)
      {
        if (VDev.gCaptureWindow.height > 240)
        {
          dwL += (unsigned long)(VDev.gnActualCaptureHeight + 5) *
                  VDev.gnActualCaptureWidth * 2;
        }
        else
        {
          dwL += (unsigned long)(VDev.gnActualCaptureHeight + 1) *
                  VDev.gnActualCaptureWidth * 2;
        }
      }
      else
      {
        dwL += (unsigned long)(VDev.gnActualCaptureHeight + 1) *
                VDev.gnActualCaptureWidth * 2;
      }
    }

    Out_Video_Reg_M(CAPTURE_MODE_I, 0x40, 0xBF);   /*Y mirror*/
    Out_Video_Reg_M(VFAC_CTL_MODE_II, 0x20, 0xDF); /*O-E polarity reverse*/
  }
  else
  {
    Out_Video_Reg_M(CAPTURE_MODE_I, 0x00, 0xBF);   /*Y normal*/
    Out_Video_Reg_M(VFAC_CTL_MODE_II, 0x00, 0xDF); /*O-E polarity normal*/
  }

  if (gwXMirror)
  {
    dwL += (unsigned long)(VDev.gnActualCaptureWidth) * 2 - 2;

    Out_Video_Reg_M(CAPTURE_MODE_I, 0x80, 0x7F);   /*X mirror*/
  }
  else
    Out_Video_Reg_M(CAPTURE_MODE_I, 0x00, 0x7F);   /*X normal*/

  /* Setting capture starting address */
  IGS_SetFreezeVid(ON);   /* only need freeze when set capture addr */

  Set_Capture_addr(dwL);

  while(!(inb_p(0x3da) & 0x08));
	/* Introducing a little delay */
	{
		int i=0; 
		for (i=0;i<20;i++)
			udelay(1000);
	}

  IGS_SetFreezeVid(OFF);

  ResetSeq();
}

/*=============================================
  This routine is used internally:
  To set Capture Address
  in: dwOffAddr points to off-screen address with garbage line adjustment
  Note: Freeze and Unfreeze should be used before and after this function.
=============================================*/
void Set_Capture_addr(unsigned long dwOffAddr)
{
  unsigned long dwL;

  dwL = dwOffAddr >> 2;

  Out_Video_Reg(CAPTURE_ADDR_L, (unsigned char) ( dwL & 0x0000ffL));
  Out_Video_Reg(CAPTURE_ADDR_M, (unsigned char) ((dwL & 0x00ff00L) >> 8));
  Out_Video_Reg(CAPTURE_ADDR_H, (unsigned char) ((dwL & 0xff0000L) >> 16));
}

/**********************************************************
 Freeze incoming video (Trun off External In)
 in: None

 Note: This routine together and IGS_UnfreezeVideo should be
       used to capture a frame, i.e.
     IGS_FreezeVideo   (freeze in-coming video stream)
     Capture-One-Frame
     IGS_UnfreezeVideo (unfreeze in-coming video stream)
 **********************************************************/
void IGS_FreezeVideo (void)
{
  Out_Video_Reg_M(CAPTURE_MODE_II, 0x80, 0x7F);

  outb_p(VIDEO_ROM_UCB4G_HI, IGSEXTINDEX);
  while ( !(inb_p(IGSEXTDATA) & 0x02) );

  ResetSeq();
}

/**********************************************************
 Unfreeze incoming video (Trun on External In)
 in: None
 **********************************************************/
void IGS_UnfreezeVideo (void)
{
  Out_Video_Reg_M(CAPTURE_MODE_II, 0x00, 0x7F);
  ResetSeq();
}

/**********************************************************
 Enable capture ONLY.  Display will be enabled separately. 
 in: None
 **********************************************************/
void IGS_VideoCaptureOn(void)
{
  IGS_SetFreezeVid(ON);

  Out_Video_Reg_M(0xb5, 0x01, 0xFE);  /* Enable Vafc */

  Out_Video_Reg(MISC_CTL_I, 0xC0);
  Out_Video_Reg(MISC_CTL_I, 0x01);

  Out_Video_Reg_M(VFAC_CTL_MODE_I, 0x41, 0xBE); /* Video capture enable */

  while(!(inb_p(0x3da) & 0x08));
	/* Introducing a little delay */
	{
		int i=0; 
		for (i=0;i<100;i++)
			udelay(1000);
	}

  IGS_SetFreezeVid(OFF);

  ResetSeq();
}

/**********************************************************
 Disable capture ONLY.  Display will be disabled separately. 
 in: None
 **********************************************************/
void IGS_VideoCaptureOff(void)
{
  /* Milind: */
  while(!(inb_p(0x3da) & 0x08));    /* wait for Vertical Retrace to begin */
  Out_Video_Reg(VFAC_CTL_MODE_I, 0x40);
  Out_Video_Reg_M(0xb5, 0x00, 0xFE);      /* Disable Vafc */

  ResetSeq();
}

/*
 *	V I D E O   O N E   (M A I N)
 */
/*************************************************************
 Turn on IGS video (Turn on both External Out and External In)
 in:   none
 Note: . This function should be called at least once to enable
   IGS video hardware.
       . In case video stops accidently, this function together
   with IGS_VideoOff can be used to reenable the video hardware.
 **********************************************************/
void IGS_VideoOn(void)
{
  while(!(inb_p(0x3da) & 0x08));
	/* Introducing a little delay */
	{
		int i=0; 
		for (i=0;i<20;i++)
			udelay(1000);
	}
  Out_Video_Reg_M(DISP_CTL_I, 0x84, 0x7B);  // enable display of video
}

/**********************************************************
 Turn off IGS video (Turn off both External Out and External In)
 in:   none
 **********************************************************/
void IGS_VideoOff(void)
{
  Out_Video_Reg( DISP_CTL_I, 0x00 );/* Video disabled*/

  ResetSeq();
}

/**********************************************************
 Setup captured video format
 in:  nFormat = YUV422
       else: not support yet
 **********************************************************/
void IGS_SetCaptureFormat(unsigned char nFormat)
{
  switch (nFormat)
  {
		default:
    case YUV422:
    {
      Out_Video_Reg_M(VIDEO_FORMAT, 0x00, 0xF8);
    }
    break;
		
		case RGB565:
		{
			Out_Video_Reg_M(VIDEO_FORMAT, 0x02, 0xF8);
		}
		break;

		case RGB24:
		{
			Out_Video_Reg_M(VIDEO_FORMAT, 0x03, 0xF8);
		}
		break;
  }
}

/**********************************************************
 Setup video overlay format
 in:  nFormat = COLORKEY
              = WINDOWKEY
 **********************************************************/
void IGS_SetOverlayFormat(unsigned char nFormat)
{
  switch (nFormat)
  {
    case COLORKEY:
    Out_Video_Reg_M(DISP_CTL_I, 0x00, 0xFD);
    break;

    case WINDOWKEY:
    default:
    Out_Video_Reg_M(DISP_CTL_I, 0x02, 0xFD);
    break;
  }
}

/**********************************************************
 Scale up the overlay window (source window unchange)
 in:
     wSrcXExt - source window width
     wDstXExt - destination window width
     wSrcYExt - source window height
     wDstYExt - destination window height
 **********************************************************/
void IGS_SetScale(unsigned short wSrcXExt,
                  unsigned short wDstXExt,
                  unsigned short wSrcYExt,
                  unsigned short wDstYExt )
{
  unsigned long dwScale;

  Out_Video_Reg(DDA_X_INIT_L, 0x0);   /*0x800*/
  Out_Video_Reg(DDA_X_INIT_H, 0x8);

  if (wSrcXExt == wDstXExt)
    dwScale = 4096;                   /*0x1000*/
  else
    dwScale = ((unsigned long)(wSrcXExt - 2) * 4096) / (unsigned long)wDstXExt;

  Out_Video_Reg(DDA_X_INC_L, LOBYTE((unsigned short)dwScale));
  Out_Video_Reg(DDA_X_INC_H, HIBYTE((unsigned short)dwScale));

  Out_Video_Reg(DDA_Y_INIT_L, 0x0);   /*0x800*/
  Out_Video_Reg(DDA_Y_INIT_H, 0x8);

  if ( wSrcYExt == wDstYExt )
    dwScale = 4096;                   /*0x1000*/
  else
    dwScale = ((unsigned long)(wSrcYExt - 2) * 4096) / (unsigned long)wDstYExt;

  Out_Video_Reg(DDA_Y_INC_L, LOBYTE((unsigned short)dwScale));
  Out_Video_Reg(DDA_Y_INC_H, HIBYTE((unsigned short)dwScale));
}

/**********************************************************
 Create and Position the overlay window with adjustment
 if overlay windows is moved to far left or far right
 in:  Rectangle to be overlayed to the screen:
          cx - co-ordinate x  of the new window
          cy - co-ordinate y  of the new window
  wWinWidth  - desired width  of the new window
  wWinHeight - desired height of the new window
 **********************************************************/
void IGS_AdjustVideoWindow(int cx, int cy, int wWinWidth, int wWinHeight)
{
  int i = 0, j = 0, wX = 0, wY = 0, iDstWidth = 0;

#if 0
printk("Inputs:  cx=%d, cy=%d, wWinWidth=%d, wWinHeight=%d\n", cx, cy, 
				wWinWidth, wWinHeight);
printk("Values: x_orig=%d, y_orig=%d, width=%d, height=%d\n", 
				VDev.gScreen.x_orig, VDev.gScreen.y_orig, VDev.gScreen.width,
				VDev.gScreen.height);
#endif

	cx -= VDev.gScreen.x_orig;
	cy -= VDev.gScreen.y_orig;

	if (cx > 0) {
		wX = 0;
		i = cx;
	}
	else {
		i = 0;
		wX = abs(cx);
		wX =	(unsigned short)
					(	((unsigned long)wX * VDev.gDisplayWindow[0].SrcWidth) /
						(unsigned long)wWinWidth
					);
	}

	if (cy > 0) {
		wY = 0;
		j = cy;
	}
	else {
		j = 0;
		wY = abs(cy);
		wY =	(unsigned short)
					( ((unsigned long)wY * VDev.gDisplayWindow[0].SrcHeight) /
						(unsigned long)wWinHeight
					);
	}	
	
  /*move to far right, need to adjust source fetch (also need consider DDA)*/
  if ((cx+wWinWidth) > VDev.gScreen.width)
  {
    iDstWidth = VDev.gScreen.width - cx;
    if ((unsigned short)wWinWidth != (VDev.gwWidthBytes / 2))
      iDstWidth = (unsigned short)
                  ( ((unsigned long)iDstWidth *
                    (VDev.gwWidthBytes / 2)) / wWinWidth
                  );
  }
  else
    iDstWidth = VDev.gwWidthBytes / 2;

#if 0
printk("Before calling IGS_SetSrcAddr(): wX=%d, wY=%d, iDstWidth=%d, SrcWidth=%d\n", wX, wY, iDstWidth, VDev.gDisplayWindow[0].SrcWidth);
#endif

  /*The following line will support Video Screen being moved to far left*/
  IGS_SetSrcAddr(	wX, wY, iDstWidth, 
									VDev.gDisplayWindow[0].SrcWidth);

  /* if move to far right, need to adjust destination display end */
  if ((cx+wWinWidth) > VDev.gScreen.width)
    iDstWidth = VDev.gScreen.width - cx;
  else
    iDstWidth = wWinWidth;

#if 0
printk("Before SetOverlayWindow: left:%d, top:%d, right:%d, bottom:%d\n", 
				i, j, cx+iDstWidth, cy+wWinHeight);
#endif

  IGS_SetOverlayWindow(i, j, cx+iDstWidth, cy+wWinHeight);
}

/**********************************************************
 Setup video overlay window (destination window)
 in:  Rectangle to be overlayed to the screen:
  wLeft (x1)
  wTop  (y1)
  wRight  (x2)
  wBottom (y2)
 **********************************************************/
void IGS_SetOverlayWindow(unsigned short wLeft,
                          unsigned short wTop,
                          unsigned short wRight,
                          unsigned short wBottom)
{
  int iDropLine, iRemainder;

#if SUPPORTVUNDERSCAN
  if (VDev.wV_Underscan==1)
  {
    iDropLine = wTop / VDev.iMaxLines;
    iRemainder= wTop % VDev.iMaxLines;

    if (iRemainder > 0)
      iDropLine ++;

    wTop = wTop - iDropLine;
  }
#endif

  Out_Video_Reg( DEST_RECT_LEFT_L,  LOBYTE( wLeft  ) );
  Out_Video_Reg( DEST_RECT_LEFT_H,  HIBYTE( wLeft  ) );
  Out_Video_Reg( DEST_RECT_RIGHT_L, LOBYTE( wRight ) );
  Out_Video_Reg( DEST_RECT_RIGHT_H, HIBYTE( wRight ) );

  Out_Video_Reg( DEST_RECT_TOP_L, LOBYTE( wTop   ) );
  Out_Video_Reg( DEST_RECT_TOP_H, HIBYTE( wTop   ) );
  Out_Video_Reg( DEST_RECT_BOTTOM_L, LOBYTE( wBottom) );
  Out_Video_Reg( DEST_RECT_BOTTOM_H, HIBYTE( wBottom) );
}

/**********************************************************
 Setup source video window address (source window to be overlayed)
 in:
     wX    - source start x
     wY    - source start y
     wPixelWidth - how many pixel in one line of the physical data will be
                   fetched from memory and displayed on screen.
     wMapWidth   - how many pixels per line are in the physical video data

     Note: Source height is not specifically defined and it is embedded
     in destination window and DDA parameters (in IGS_SetScale)
 **********************************************************/
void IGS_SetSrcAddr(unsigned short wX,
                    unsigned short wY,
                    unsigned short wPixelWidth,
                    unsigned short wMapWidth)
{
  unsigned long     dwL;
  unsigned char     bHigh;
  unsigned short    wFetchPitch;
	int								nBytesPerPixel = 0;
  unsigned short    wBytePitch = 0;

	/* find out how many bytes per pixel should be used to display the video 
			data.  Since local captured data are always in YUV422 2bytes/pixel 
			format, local display should always use nBytesPerPixel = 2.
	*/
	if (!VDev.gDisplayWindow[0].SrcType)	/* displaying local image */
	{
		nBytesPerPixel = 2;

  	wBytePitch = wMapWidth * nBytesPerPixel;

    /* Reduce some scanlines to avoid garbages from decoder */
    dwL = ( VDev.gAMI.gulCaptureStart +
            (VDev.gnGarbageLines + wY) * wBytePitch + wX * nBytesPerPixel
          ) >> 2;
	}
	else
	{
		nBytesPerPixel = VDev.gInputBPP[0];

  	wBytePitch = wMapWidth * nBytesPerPixel;

		/* need to check the width bytes here again because if app dumps too much
				data to main video say, 352x288x3, it will over the threshold (352x2)
				and driver should then use MPEG2 FIFO.
				
				when X2 is activated, it will need to check FIFO again since X2 will
				increase the total width bytes. i.e. Main Video displaying 176x144x2
				local capture video and app dumps 176x144x3 into X2, total width bytes
				become (176x2 + 176x3) = 880 which is over threshold and driver should
				use MPEG2 FIFO settings.
		*/
  	if (wBytePitch <= VDev.gusHighResThreshold)
		{
#ifdef DEBUG
printk("video.c: (Main) Set MPEG 1 FIFO.\n");
#endif
			Change_VideoFIFO(0);        /*set MPEG1 FIFO*/
		}
		else
		{
#ifdef DEBUG
printk("video.c: (Main) Set MPEG 2 FIFO.\n");
#endif
			Change_VideoFIFO(1);        /*set MPEG2 FIFO*/
		}

    dwL = VDev.gAMI.gulDisplayStart[(VDev.gDWinRegion[0])] >> 2;
	}


  /* Playback start addr */
  Out_Video_Reg( MEMORY_START_L, (unsigned char) (  dwL & 0x0000ffL) );
  Out_Video_Reg( MEMORY_START_M, (unsigned char) ( (dwL & 0x00ff00L) >> 8 ) );
  Out_Video_Reg( MEMORY_START_H, (unsigned char) ( (dwL & 0xff0000L) >> 16 ) );

  /* pitch is a multiple of 64 bits */
  wBytePitch = wBytePitch >> 3;
  wFetchPitch = ( wPixelWidth * nBytesPerPixel + 7 ) >> 3;

  bHigh = HIBYTE( wBytePitch );
  bHigh = bHigh | ( ( HIBYTE( wFetchPitch ) ) << 4 );

  Out_Video_Reg( MEMORY_PITCH_L, LOBYTE( wBytePitch ) );
  Out_Video_Reg( MEMORY_PITCH_H, bHigh );
  Out_Video_Reg( MEMORY_OFFSET_PHASE, LOBYTE( wFetchPitch ) );
}

/**********************************************************
 Clean up some status which were set during the video overlay/capture
 in:   none

 Note: This function should be called just before the application exits
 **********************************************************/
void IGS_VideoCleanUp(void)
{
  /*restore FIFO control regs*/
  outw_p(savedReg74, IGSEXTINDEX);
  outw_p(savedReg75, IGSEXTINDEX);
  outw_p(savedRegD9, IGSEXTINDEX);
  outw_p(savedRegDA, IGSEXTINDEX);
}

/*
 *	V I D E O   T W O   (V 2   V I D E O)
 */
/**********************************************************
 Setup source video 2 window address (source window to be overlayed)
 in:
     wX    - source start x
     wY    - source start y
     wPixelWidth - source window width (similar to XGA dimension 1)
     wMapWidth   - source window map width

     Note: Source height is not specifically defined and it is embedded
     in destination window and DDA parameters (in IGS_SetScale)
 **********************************************************/
void IGS_SetSrcAddr_V2(	unsigned short wX, 
												unsigned short wY, 
												unsigned short wPixelWidth, 
												unsigned short wMapWidth)
{
	unsigned long   dwL;
  unsigned char   bHigh;
  unsigned short  wFetchPitch;
	int							nBytesPerPixel = 0;
	unsigned short	wBytePitch = 0;

	/* find out how many bytes per pixel should be used to display the video 
			data.  Since local captured data are always in YUV422 2bytes/pixel 
			format, local display should always use nBytesPerPixel = 2.
	*/
	if (!VDev.gDisplayWindow[1].SrcType)	/* displaying local image */
	{
		nBytesPerPixel = 2;

	  wBytePitch = wMapWidth * nBytesPerPixel;

		/* Reduce some scalines to avoid garbages from decoder */
		dwL = ( VDev.gAMI.gulCaptureStart +
						(VDev.gnGarbageLines + wY) * wBytePitch + wX * nBytesPerPixel
					) >> 2;
	}
	else
	{
		nBytesPerPixel = VDev.gInputBPP[1];

	  wBytePitch = wMapWidth * nBytesPerPixel;

		dwL = VDev.gAMI.gulDisplayStart[(VDev.gDWinRegion[1])] >> 2;
	}

  /* playback start addr */
  Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);

  Out_Video_Reg(V_SRC_START_LO, (unsigned char)(dwL & 0x0000ffL));
  Out_Video_Reg(V_SRC_START_MI, (unsigned char)((dwL & 0x00ff00L) >> 8));
  Out_Video_Reg(V_SRC_START_HI, (unsigned char)((dwL & 0xff0000L) >> 16));

  /* pitch is a multiple of 64 bits */
  wBytePitch = wBytePitch >> 3; /*64 bit address field*/
  wFetchPitch = (wPixelWidth * nBytesPerPixel + 7) >> 3;

  bHigh = HIBYTE(wBytePitch);
  bHigh = bHigh | ((HIBYTE(wFetchPitch)) << 4);

  Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);

  Out_Video_Reg(V_SRC_STRIDE_LO, LOBYTE(wBytePitch));
  Out_Video_Reg(V_SRC_STRIDE_HI, bHigh);
  Out_Video_Reg(V_SRC_FETCH_LO, LOBYTE(wFetchPitch));

  Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);
}

/**********************************************************
 Setup video 2 overlay window (destination window)
 in:  Rectangle to be overlayed to the screen:
  wLeft (x1)
  wTop  (y1)
  wRight  (x2)
  wBottom (y2)
 **********************************************************/
void IGS_SetOverlayWindow_V2(	unsigned short wLeft, 
															unsigned short wTop, 
															unsigned short wRight, 
															unsigned short wBottom)
{
  int iDropLine, iRemainder;

  if (wRight > (unsigned short) VDev.gScreen.x_ext)
    wRight = VDev.gScreen.x_ext;

#if SUPPORTVUNDERSCAN
  if (VDev.wV_Underscan==1)
  {
    iDropLine = wTop / VDev.iMaxLines;
    iRemainder= wTop % VDev.iMaxLines;
    if (iRemainder > 0)
      iDropLine ++;

    wTop = wTop - iDropLine;
  }
#endif

  Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);

  Out_Video_Reg(V_DST_X1_LO, LOBYTE(wLeft));
  Out_Video_Reg(V_DST_X1_HI, HIBYTE(wLeft));
  Out_Video_Reg(V_DST_X2_LO, LOBYTE(wRight));
  Out_Video_Reg(V_DST_X2_HI, HIBYTE(wRight));

  Out_Video_Reg(V_DST_Y1_LO, LOBYTE(wTop));
  Out_Video_Reg(V_DST_Y1_HI, HIBYTE(wTop));
  Out_Video_Reg(V_DST_Y2_LO, LOBYTE(wBottom));
  Out_Video_Reg(V_DST_Y2_HI, HIBYTE(wBottom));
}

/**********************************************************
 Trun on video 2 overlay
 in:   none
 **********************************************************/
void IGS_OverlayOn_V2(void)
{
  Out_Video_Reg(CAPTURE_BANK_CTL, 0x01);
  Out_Video_Reg_M(0x4D, 0x80, 0x7F);
}

/**********************************************************
 Trun off video 2 overlay
 in:   none
 **********************************************************/
void IGS_OverlayOff_V2(void)
{
  Out_Video_Reg(CAPTURE_BANK_CTL, 0x01);
  Out_Video_Reg_M(0x4D, 0x00, 0x7F);
}

/**********************************************************
 Clean up some status for video 2 overlay hardware
 in:   none

 Note: This function should be called right after application
       closes video 2.
 **********************************************************/
void IGS_VideoCleanUp_V2(void)
{
  int i;
  int iStart=0x40;
  int iMaxNum=0x4D;

  /*clear Video_2 registers*/
  Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);
  for (i=iStart; i<iMaxNum; i++)
    Out_Video_Reg((unsigned char) i, 0x00);
}

/**********************************************************
 Create and Position the 2nd overlay window with adjustment
 if overlay windows is moved to far left or far right
 in:  Rectangle to be overlayed to the screen:
  cx  - current x of the window
  cy  - current y of the window
  wWinWidth  - current window width
  wWinHeight - current window height
 **********************************************************/
void IGS_AdjustVideoWindow_V2(int cx, int cy, int wWinWidth, int wWinHeight)
{
  int i, j, wX, wY;

  if ( cx > 0) {
    wX = 0;
    i = cx;
  } else {
    i = 0;
    wX = abs(cx);
    wX = 	(unsigned short)
					(	((unsigned long)wX * VDev.gDisplayWindow[1].SrcWidth) /
						(unsigned long)wWinWidth
					);
  }

  if ( cy > 0) {
    wY = 0;
    j = cy;
  } else {
    j = 0;
    wY = abs(cy);
    wY = 	(unsigned short)
					(	((unsigned long)wY * VDev.gDisplayWindow[1].SrcHeight) /
						(unsigned long)wWinHeight
					);
  }

  /*The following line will support Video Screen being moved to far left*/
  IGS_SetSrcAddr_V2(wX, wY, VDev.gDisplayWindow[1].SrcWidth, 
										VDev.gDisplayWindow[1].SrcWidth);
  IGS_SetOverlayWindow_V2(i, j, cx+wWinWidth, cy+wWinHeight);
}

/**********************************************************
 Scale up video 2 overlay window (video 2 source window unchange)
 in:
     wSrcXExt - source window width
     wDstXExt - destination window width
     wSrcYExt - source window height
     wDstYExt - destination window height
 **********************************************************/
void IGS_SetScale_V2(	unsigned short wSrcXExt, 
											unsigned short wDstXExt, 
											unsigned short wSrcYExt,
											unsigned short wDstYExt)
{
  unsigned long dwScale;

	Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);

  /* initial value registers are same as video 1 */
  Out_Video_Reg(DDA_X_INIT_L, 0x0);
  Out_Video_Reg(DDA_X_INIT_H, 0x8);

  if ( wSrcXExt == wDstXExt )
    dwScale = 4096;
  else
    dwScale = ((unsigned long)(wSrcXExt - 2) * 4096) / (unsigned long)wDstXExt;

  Out_Video_Reg(CAPTURE_BANK_CTL, 0x01);

  Out_Video_Reg( V_SRC_STRIDE_LO, LOBYTE( (unsigned short)dwScale ) );
  Out_Video_Reg( V_SRC_STRIDE_HI, HIBYTE( (unsigned short)dwScale ) );

	Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);

  /*initial value registers are same as video 1*/
  Out_Video_Reg( DDA_Y_INIT_L, 0x0 );
  Out_Video_Reg( DDA_Y_INIT_H, 0x8 );

  if ( wSrcYExt == wDstYExt )
    dwScale = 4096;
  else
    dwScale = ((unsigned long)(wSrcYExt - 2) * 4096) / (unsigned long)wDstYExt;

  Out_Video_Reg(CAPTURE_BANK_CTL, 0x01);

  Out_Video_Reg( V_DST_X2_LO, LOBYTE( (unsigned short)dwScale ) );
  Out_Video_Reg( V_DST_X2_HI, HIBYTE( (unsigned short)dwScale ) );

  Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);
}

/**********************************************************
 Setup video 2 overlay format
 in:  nFormat = COLORKEY
              = WINDOWKEY
 **********************************************************/
void IGS_SetOverlayFormat_V2(unsigned char nFormat)
{
  Out_Video_Reg(CAPTURE_BANK_CTL, 0x01);

  switch (nFormat)
  {
    case COLORKEY:
      Out_Video_Reg_M(0x4C, 0x00, 0xFD);
    break;

    case WINDOWKEY:
    default:
      Out_Video_Reg_M(0x4C, 0x02, 0xFD);
    break;
  }

  Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);

}

/**********************************************************
 Setup captured video format
 in:  nFormat = YUV422
       else: not support yet
 **********************************************************/
void IGS_SetCaptureFormat_V2(unsigned char nFormat)
{
  Out_Video_Reg(CAPTURE_BANK_CTL, 0x01);

  switch (nFormat)
  {
		default:
    case YUV422:
    {
      Out_Video_Reg_M(0x4B, 0x00, 0xF8);
    }
    break;
		
		case RGB565:
		{
			Out_Video_Reg_M(0x4B, 0x02, 0xF8);
		}
		break;

		case RGB24:
		{
			Out_Video_Reg_M(0x4B, 0x03, 0xF8);
		}
		break;
  }
	
  Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);
}

/*
 *	V I D E O   T H R E E   (X 2   V I D E O)
 */
/**********************************************************
 Setup source video 3 window address (source window to be overlayed)
 in:
     wX    - source start x
     wY    - source start y
     wPixelWidth - source window width (similar to XGA dimension 1)
     wMapWidth   - source window map width

     Note: Source height is not specifically defined and it is embedded
     in destination window and DDA parameters (in IGS_SetScale)
 **********************************************************/
void IGS_SetSrcAddr_X2(	unsigned short wX, 
												unsigned short wY, 
												unsigned short wPixelWidth, 
												unsigned short wMapWidth)
{
	unsigned long   dwL;
  unsigned char   bHigh;
  unsigned short  wFetchPitch;
	int							nBytesPerPixel = 0;
  unsigned short  wBytePitch = 0;

	/* find out how many bytes per pixel should be used to display the video 
			data.  Since local captured data are always in YUV422 2bytes/pixel 
			format, local display should always use nBytesPerPixel = 2.
	*/
	if (!VDev.gDisplayWindow[2].SrcType)	/* displaying local image */
	{
		nBytesPerPixel = 2;

  	wBytePitch = wMapWidth * nBytesPerPixel;

// Newly added
#if 0
		{
printk("video.c: (X2) Set MPEG 2 FIFO.\n");
			Change_VideoFIFO(1);        /*set MPEG2 FIFO*/
		}
#endif

		/* Reduce some scalines to avoid garbages from decoder */
		dwL = ( VDev.gAMI.gulCaptureStart +
						(VDev.gnGarbageLines + wY) * wBytePitch + wX * nBytesPerPixel
					) >> 2;
	}
	else
	{
		nBytesPerPixel = VDev.gInputBPP[2];

  	wBytePitch = wMapWidth * nBytesPerPixel;

		/* need to check the width bytes here again because if app dumps too much
				data to main video and X2 video, it will be over the threshold (352x2)
				and driver should then use MPEG2 FIFO.
				
				when X2 is activated, it will need to check FIFO again since X2 will
				increase the total width bytes. i.e. Main Video displaying 176x144x2
				local capture video and app dumps 176x144x3 into X2, total width bytes
				become (176x2 + 176x3) = 880 which is over threshold and driver should
				use MPEG2 FIFO settings.
				
				ideally, we should find out the main video width bytes and add it to
				X2 width bytes to check if they are over threshold but for now, just
				set to MPEG2 FIFO whenever X2 is activiated.
		*/
		{
#ifdef DEBUG
printk("video.c: (X2) Set MPEG 2 FIFO.\n");
#endif
			Change_VideoFIFO(1);        /*set MPEG2 FIFO*/
		}

		dwL = VDev.gAMI.gulDisplayStart[(VDev.gDWinRegion[2])] >> 2;
	}

  /* playback start addr */
  Out_Video_Reg(CAPTURE_BANK_CTL, 0x04);

  Out_Video_Reg(0x40, (unsigned char)(dwL & 0x0000ffL));
  Out_Video_Reg(0x41, (unsigned char)((dwL & 0x00ff00L) >> 8));
  Out_Video_Reg(0x42, (unsigned char)((dwL & 0xff0000L) >> 16));

  /* pitch is a multiple of 64 bits */
  wBytePitch = wBytePitch >> 3; /*64 bit address field*/
  wFetchPitch = (wPixelWidth * nBytesPerPixel + 7) >> 3;

  bHigh = HIBYTE(wBytePitch);
  bHigh = bHigh | ((HIBYTE(wFetchPitch)) << 4);

  Out_Video_Reg(0x43, LOBYTE(wBytePitch));
  Out_Video_Reg(0x44, bHigh);
  Out_Video_Reg(0x4F, LOBYTE(wFetchPitch));

  Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);
}

/**********************************************************
 Setup video 3 overlay window (destination window)
 in:  Rectangle to be overlayed to the screen:
  wLeft (x1)
  wTop  (y1)
  wRight  (x2)
  wBottom (y2)
 **********************************************************/
void IGS_SetOverlayWindow_X2(	unsigned short wLeft, 
															unsigned short wTop, 
															unsigned short wRight, 
															unsigned short wBottom)
{
  int iDropLine, iRemainder;
	int nXPipeStart = 0;
	int nTemp1 = 0;
	int nTemp2 = 0;
	
#if 0 // Before change 
	nXPipeStart = VDev.gDisplayWindow[0].dest_win.x + 
								VDev.gDisplayWindow[0].dest_win.width + 4;
#endif
/* Changes */
	/* only consider boundaries if the corresponding video window is turned on */
	if (VDev.gWindowOnOff[0])	
	{
		nTemp1 = 	VDev.gDisplayWindow[0].dest_win.x +
							VDev.gDisplayWindow[0].dest_win.width + 4;
	}

	if (VDev.gWindowOnOff[1])
	{
		nTemp2 =	VDev.gDisplayWindow[1].dest_win.x +
							VDev.gDisplayWindow[1].dest_win.width + 4;
	}

	if (nTemp1 >= nTemp2)
		nXPipeStart = nTemp1; 
	else
		nXPipeStart = nTemp2;
/* Changes end */

#if 0
	nXPipeStart = 400;
#endif
	
#if 0
	if (wLeft < (nXPipeStart + 12))
	{
		wLeft = nXPipeStart + 12;
	}
#endif
	if (wLeft < (nXPipeStart + 12))
	{
		wLeft = nXPipeStart + 12;
	}

	/* Depends on whether we are scaling up or scaling down */
	if (VDev.gDisplayWindow[2].dest_win.width > VDev.gDisplayWindow[2].SrcWidth)
	{
		wRight = wLeft + VDev.gDisplayWindow[2].dest_win.width - 
							(6 * 	(VDev.gDisplayWindow[2].dest_win.width /
							VDev.gDisplayWindow[2].SrcWidth));
	}
	else
	{
		wRight = wLeft + VDev.gDisplayWindow[2].dest_win.width - 
							(6 * 	(VDev.gDisplayWindow[2].SrcWidth /
							VDev.gDisplayWindow[2].dest_win.width)); 
	}

#if 0
printk("!!!!!!!!!! nXpipestart is:  %d\n", nXPipeStart);
printk("!!!!!!!!!! wLeft is:        %d\n", wLeft);
printk("!!!!!!!!!! wRight is:       %d\n", wRight);
#endif

  if (wRight > (unsigned short) VDev.gScreen.x_ext)
    wRight = VDev.gScreen.x_ext;

#if SUPPORTVUNDERSCAN
  if (VDev.wV_Underscan==1)
  {
    iDropLine = wTop / VDev.iMaxLines;
    iRemainder= wTop % VDev.iMaxLines;
    if (iRemainder > 0)
      iDropLine ++;

    wTop = wTop - iDropLine;
  }
#endif

  Out_Video_Reg(CAPTURE_BANK_CTL, 0x04);

	Out_Video_Reg(0x45, LOBYTE(nXPipeStart));
	Out_Video_Reg(0x46, HIBYTE(nXPipeStart));
	Out_Video_Reg(0x47, LOBYTE(wLeft));
	Out_Video_Reg(0x48, HIBYTE(wLeft));
	Out_Video_Reg(0x49, LOBYTE(wRight));
	Out_Video_Reg(0x4A, HIBYTE(wRight));

  Out_Video_Reg(0x4B, LOBYTE(wTop));
  Out_Video_Reg(0x4C, HIBYTE(wTop));
  Out_Video_Reg(0x4D, LOBYTE(wBottom));
  Out_Video_Reg(0x4E, HIBYTE(wBottom));

  Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);
}

/**********************************************************
 Trun on video 3 overlay
 in:   none
 **********************************************************/
void IGS_OverlayOn_X2(void)
{
  Out_Video_Reg(CAPTURE_BANK_CTL, 0x05);
  Out_Video_Reg_M(0x4D, 0x80, 0x7F);
	Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);
}

/**********************************************************
 Trun off video 3 overlay
 in:   none
 **********************************************************/
void IGS_OverlayOff_X2(void)
{
  Out_Video_Reg(CAPTURE_BANK_CTL, 0x05);
  Out_Video_Reg_M(0x4D, 0x00, 0x7F);
	Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);
}

/**********************************************************
 Clean up some status for video 3 overlay hardware
 in:   none

 Note: This function should be called right after application
       closes video 3.
 **********************************************************/
void IGS_VideoCleanUp_X2(void)
{
  int i;
  int iStart=0x40;
  int iMaxNum=0x4F;

  /*clear Video_3 registers*/
  Out_Video_Reg(CAPTURE_BANK_CTL, 0x04);
  for (i=iStart; i<iMaxNum; i++)
    Out_Video_Reg((unsigned char) i, 0x00);
}

/**********************************************************
 Create and Position the 3rd overlay window with adjustment
 if overlay windows is moved to far left or far right
 in:  Rectangle to be overlayed to the screen:
  cx  - current x of the window
  cy  - current y of the window
  wWinWidth  - current window width
  wWinHeight - current window height
 **********************************************************/
void IGS_AdjustVideoWindow_X2(int cx, int cy, int wWinWidth, int wWinHeight)
{
  int i, j, wX, wY;

  if ( cx > 0) {
    wX = 0;
    i = cx;
  } else {
    i = 0;
    wX = abs(cx);
    wX = 	(unsigned short)
					(	((unsigned long)wX * VDev.gDisplayWindow[2].SrcWidth) /
						(unsigned long)wWinWidth
					);
  }

  if ( cy > 0) {
    wY = 0;
    j = cy;
  } else {
    j = 0;
    wY = abs(cy);
    wY = 	(unsigned short)
					(	((unsigned long)wY * VDev.gDisplayWindow[2].SrcHeight) /
						(unsigned long)wWinHeight
					);
  }

  /*The following line will support Video Screen being moved to far left*/
  IGS_SetSrcAddr_X2(wX, wY, VDev.gDisplayWindow[2].SrcWidth, 
										VDev.gDisplayWindow[2].SrcWidth);
  IGS_SetOverlayWindow_X2(i, j, cx+wWinWidth, cy+wWinHeight);
}

/**********************************************************
 Scale up video 3 overlay window (video 3 source window unchange)
 in:
     wSrcXExt - source window width
     wDstXExt - destination window width
     wSrcYExt - source window height
     wDstYExt - destination window height
 **********************************************************/
void IGS_SetScale_X2(	unsigned short wSrcXExt, 
											unsigned short wDstXExt, 
											unsigned short wSrcYExt,
											unsigned short wDstYExt)
{
  unsigned long dwScale;

  Out_Video_Reg(CAPTURE_BANK_CTL, 0x05);

  Out_Video_Reg(0x40, 0x0);
  Out_Video_Reg(0x41, 0x8);
	//Out_Video_Reg(DDA_X_INIT_L, 0x0);
	//Out_Video_Reg(DDA_X_INIT_H, 0x8);

  if ( wSrcXExt == wDstXExt )
    dwScale = 4096;
  else
    dwScale = ((unsigned long)(wSrcXExt - 2) * 4096) / (unsigned long)wDstXExt;

  Out_Video_Reg(0x42, LOBYTE( (unsigned short)dwScale ) );
  Out_Video_Reg(0x43, HIBYTE( (unsigned short)dwScale ) );

  Out_Video_Reg(0x44, 0x0 );
  Out_Video_Reg(0x45, 0x8 );
	//Out_Video_Reg(DDA_Y_INIT_L, 0x0);
	//Out_Video_Reg(DDA_Y_INIT_H, 0x8);

  if ( wSrcYExt == wDstYExt )
    dwScale = 4096;
  else
    dwScale = ((unsigned long)(wSrcYExt - 2) * 4096) / (unsigned long)wDstYExt;

  Out_Video_Reg(CAPTURE_BANK_CTL, 0x05);

  Out_Video_Reg(0x46, LOBYTE( (unsigned short)dwScale ) );
  Out_Video_Reg(0x47, HIBYTE( (unsigned short)dwScale ) );

  Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);
}

/**********************************************************
 Setup video 3 overlay format
 in:  nFormat = COLORKEY
              = WINDOWKEY
 **********************************************************/
void IGS_SetOverlayFormat_X2(unsigned char nFormat)
{
  Out_Video_Reg(CAPTURE_BANK_CTL, 0x05);

  switch (nFormat)
  {
    case COLORKEY:
      Out_Video_Reg_M(0x49, 0x00, 0xFD);
    break;

    case WINDOWKEY:
    default:
      Out_Video_Reg_M(0x49, 0x02, 0xFD);
    break;
  }

  Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);

}

/**********************************************************
 Setup captured video format
 in:  nFormat = YUV422
       else: not support yet
 **********************************************************/
void IGS_SetCaptureFormat_X2(unsigned char nFormat)
{
  Out_Video_Reg(CAPTURE_BANK_CTL, 0x05);

  switch (nFormat)
  {
		default:
    case YUV422:
    {
      Out_Video_Reg_M(0x48, 0x00, 0xF8);
    }
    break;
		
		case RGB565:
		{
			Out_Video_Reg_M(0x48, 0x02, 0xF8);
		}
		break;

		case RGB24:
		{
			Out_Video_Reg_M(0x48, 0x03, 0xF8);
		}
		break;
  }
	
  Out_Video_Reg(CAPTURE_BANK_CTL, 0x00);
}

/***
		P I P   F U N C T I O N S
***/
/**********************************************************
  Enable/Disable Picture in Picture
  in:   ON  - enable PiP
  OFF - disable PiP
**********************************************************/
void IGS_EnablePIP(int OnOff)
{
  if (OnOff == ON)
    Out_Video_Reg_M(CAPTURE_NEW_CTL_I, 0x80, 0x7F);  /* enable PIP */
  else
    Out_Video_Reg_M(CAPTURE_NEW_CTL_I, 0x00, 0x7F);  /* disable PIP */
}

/**********************************************************
 Set Picture in Picture window
  in:   x0, y0, x1, y1 are (x,y) position relative to captured
        data area (not screen coordination).

  iCaptureW, iCaptureH are capture width and height.

  E.g. if capture size is 352x240, then (x0,y0) and (x1,y1)
  are relative to the area (0,0) and (351,239).
  iCaptureW, iCaptureH are 352 and 240.

**********************************************************/
void IGS_SetPIPWindow(int x0, int y0, int x1, int y1, int iCaptureW, int iCaptureH)
{
  unsigned long  wX0,wX1;
  unsigned long  wY0,wY1;

  unsigned short wReg6061;
  unsigned short wReg6465;

	/* Adjustment only */
	y1 += VDev.gnGarbageLines;		
	x0 += 2;

#if 0
  wReg6061 = ReadReg(IGSEXTINDEX, DEST_RECT_LEFT_H);
  wReg6061 = (wReg6061 << 8) + ReadReg(IGSEXTINDEX, DEST_RECT_LEFT_L);
  wReg6465 = ReadReg(IGSEXTINDEX, DEST_RECT_TOP_H);
  wReg6465 = (wReg6465 << 8) + ReadReg(IGSEXTINDEX, DEST_RECT_TOP_L);
#endif

  wReg6061 = ReadReg(IGSEXTINDEX, CAPTURE_H_START_H);
  wReg6061 = (wReg6061 << 8) + ReadReg(IGSEXTINDEX, CAPTURE_H_START_L);
  wReg6465 = ReadReg(IGSEXTINDEX, CAPTURE_V_START_H);
  wReg6465 = (wReg6465 << 8) + ReadReg(IGSEXTINDEX, CAPTURE_V_START_L);

  wX0 = (unsigned long) 2*x0;
  wX0 = (unsigned long)((wX0 * MAX_NTSC_CAPTURE_WIDTH)/iCaptureW);
  //wX0 = (unsigned long) wX0 * 2;
  wX0 += wReg6061;    /*add capture clip information*/


  wX1 = (unsigned long) 2*x1;
  wX1 = (unsigned long)((wX1 * MAX_NTSC_CAPTURE_WIDTH)/iCaptureW);
  //wX1 = (unsigned long) wX1 * 2;
  wX1 += wReg6061;    /*add capture clip information*/

  wY0 = (unsigned long)(((y0 * MAX_NTSC_CAPTURE_HEIGHT)/2) / iCaptureH);
  wY0 += wReg6465 + 4;    /*add capture clip information*/

  wY1 = (unsigned long)(((y1 * MAX_NTSC_CAPTURE_HEIGHT)/2) / iCaptureH);
  wY1 += wReg6465;    /* add capture clip information */

  Out_Video_Reg(CAP_PIP_HSTART_L, (unsigned char) wX0);
  Out_Video_Reg(CAP_PIP_HSTART_H, (unsigned char)(wX0 >> 8));
  Out_Video_Reg(CAP_PIP_HEND_L,   (unsigned char) wX1);
  Out_Video_Reg(CAP_PIP_HEND_H,   (unsigned char)(wX1 >> 8));

  Out_Video_Reg(CAP_PIP_VSTART_L, (unsigned char) wY0);
  Out_Video_Reg(CAP_PIP_VSTART_H, (unsigned char)(wY0 >> 8));
  Out_Video_Reg(CAP_PIP_VEND_L,   (unsigned char) wY1);
  Out_Video_Reg(CAP_PIP_VEND_H,   (unsigned char)(wY1 >> 8));
}

/***
		F U N C T I O N S   N O T   U S E D   
***/

#if 0	
/*=============================================
  This routine is used to set Capture Src Clip
  in:
    gnVideoStandard == NTSC or PAL
=============================================*/
void IGS_SetCaptureClip(int gnVideoStandard)
{
  switch (VDev.gnWhichDecoder)
  {
    default:
    case PHILIPS7111:
      if (gnVideoStandard == NTSC)
      {
        /*some garbage lines*/
        outw_p((0x0A00 + CAPTURE_V_START_L), IGSEXTINDEX  );
        outw_p((0x0000 + CAPTURE_V_START_H), IGSEXTINDEX  );

        /*480+GarbageLines=end-start*/
        outw_p((0xFD00 + CAPTURE_V_END_L), IGSEXTINDEX  );  // garyn
        outw_p((0x0000 + CAPTURE_V_END_H), IGSEXTINDEX  );
      }
      else if (gnVideoStandard == PAL)
      {
        outw_p((0x0B00 + CAPTURE_V_START_L),IGSEXTINDEX ); /*some garbage lin*/
        outw_p((0x0000 + CAPTURE_V_START_H), IGSEXTINDEX  );
        outw_p((0x2F00 + CAPTURE_V_END_L), IGSEXTINDEX  ); /*576+GarbageLines=end-start*/
        outw_p((0x0100 + CAPTURE_V_END_H), IGSEXTINDEX  );
      }
    break;
  }
}

/**********************************************************
 Setup width and Height of video source window
       (this routine does not set hardware registers)
 in:  nWidth  = width of captured video image in pixel.
      nHeight = height of captured video image in scan line.

 return  0 if successful
  -1 if error

  note: . full screen capture may not be available in some modes due to
          off-screen memory requirment. So make sure to check
          retuned value to see it is successful or not.
        . Application must call this routine to set gwWidthBytes
          before calling

          IGS_AdjustVideoWindow
          IGS_SetCaptureAddr
 **********************************************************/
int  IGS_SetSrcWindowSize(unsigned short wWidth, unsigned short wHeight)
{
  int iFlag;

  iFlag = 0;   /*to indicate successful*/
#if 0

  //if (wWidth % 2)
  if (wWidth & 1)
    iFlag = -1;  /*error*/

  if ( (wWidth > MAXMPEG1) && (VDev.gnFullScreenOk == 0) )
    iFlag = -1;  /*error*/

  wDEFAULT_SRC_WIDTH  = wWidth;
  wDEFAULT_SRC_HEIGHT = wHeight;

  gwWidthBytes = wDEFAULT_SRC_WIDTH*2;

#endif
  return (iFlag);
}

/**********************************************************
 Set pointer to video memory used to store captured data

 in: dwL - 32 bit value pointing to video memory, usually is
     off-screen to store YUV422 captured data.

 Note:
       . dwL must be physical memory pointer, not virtual.
   Please see Init_ConfigData as an example
       . With this pointer, application can access captured
   image data directly.
 **********************************************************/
void dwSetOffScreenPtr(unsigned long dwL)
{
   dwOffsetOffScreen = dwL;
}

int IGS_InitParam(void)
{
  if (FALSE == Init_ConfigData())
    return(1);

  //IGS_InitXlatTab();   /* first YUV->RGB xlat table */
  InitXlatTab();   /* first YUV->RGB xlat table */

  return(0);
}

/**********************************************************
 Setup width and Height of video source window
       (this routine does not set hardware registers)
 in:  nWidth  = width of captured video image in pixel.
      nHeight = height of captured video image in scan line.

 return  0 if successful
  -1 if error

 note  . video 2 may not be available in some modes due to
   off-screen memory requirment. So application should
   make sure the offscreen has enough space to hold 2nd
   video source data.
 **********************************************************/
int  IGS_SetSrcWindowSize_V2(unsigned short wWidth, unsigned short wHeight)
{
  int iFlag;

  iFlag = 0;   /*to indicate successful*/

  //if (wWidth % 2)
  if (wWidth & 1)
    iFlag = -1;  /*error*/

  wDEFAULT_SRC_WIDTH_V2  = wWidth;
  wDEFAULT_SRC_HEIGHT_V2 = wHeight;

  return (iFlag);
}
 
/**********************************************************
 Set pointer to video memory used by Video 2 overlay window

 in: dwL - 32 bit value pointing to video memory, usually is
     off-screen to store YUV422 overlay data.

 Note:
       . dwL must be physical memory pointer, not virtual.
 **********************************************************/
void dwSetOffScreenPtr_V2(unsigned long dwL)
{
  dwOffsetOffScreen_V2 = dwL;
}

#endif
