#include "igscop.h"
#include "igs.h"
#include "video.h"

/*-----------------------------------------------------
  Enable Graphics Engine
  This routine shows an example that enables graphics engine's memory
  mapped IO at base of 0xBF000 (notice this is working DOS program).
  However, enable to linear address is recommanded.
  E.g. video memory is located at 0xFE000000, we can use
       0xFE8BF000 as graphics engine's memory mapped IO base,
       We need set 3CE_33 bits 0,1,2,3, that is,
   OUTPORT(igs_index_port, 0x0F33);
  -----------------------------------------------------*/
void EnableCop()
{
  // orig : outw_p(0x1C33, IGSEXTINDEX);    //enable graphics engine at BF000
  outw_p(0x0f33, IGSEXTINDEX);    //enable graphics engine at BF000
}

/*----------------------------------------------
   IGS Graphics Engine Funtions

   Note: the Graphics engine routines are not necessary in overlay
   and capture programming. All overlay and capture data
   can be manipulated by using CPU.
  ----------------------------------------------*/
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  Basic bitblt - source copy (a color map as source)
  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
/*------------------------------------------------------
   copy a rectangle from src (sx, sy) to dst (dx, dy)
  ----------------------------------------------------*/
void SrcCopy(int sx, int sy, int dx, int dy, int w, int h)
{
  unsigned long op;
  wait_for_cop_ready();

  *Cop_control    = 0;

  *Fore_Mix   = (char) Mix_S;
  if ((VDev.gScreen.bytes_per_pixel-1) != 2)
  {
    *Op_Dimension_1 = w-1;
    *Dst_Start_Ptr  = (unsigned long)dy*(VDev.gScreen.x_ext) + dx;
    *Src_Start_Ptr  = (unsigned long)sy*(VDev.gScreen.x_ext) + sx;
  }
  else
  {
    *Op_Dimension_1 = w-1;
    *Dst_Start_Ptr  =((unsigned long)dy*(VDev.gScreen.x_ext) + dx)*3;
    *Dst_X_Phase   =((unsigned char)dy*(VDev.gScreen.x_ext) + dx)  ;
    *Src_Start_Ptr  =((unsigned long)sy*(VDev.gScreen.x_ext) + sx)*3;
  }
  *Op_Dimension_2   = h-1;

  op  = Fore_Src_PxMap;
  op |= PxBlt;
  op |= Pattern_Foreground | Right_Down;
  *Px_OpL = LOWORD(op);
  *Px_OpH = HIWORD(op);
}

/*------------------------------------------------------
   copy a rectangle from src (sx, sy) to dst (dx, dy)

   This routine play a trick to copy data in different
   dimension as the current screen resolution.

   The off-screen is used as base start pointer.

   This routine should be used together with IGS_SetPIPWindow
  ----------------------------------------------------*/
void SrcCopy_0(int sx, int sy, int dx, int dy, int w, int h)
{

  unsigned long op;
  wait_for_cop_ready();

  *Cop_control    = 0;

  *Fore_Mix   = (char) Mix_S;
  //*Op_Dimension_1 = w-1;
  *Op_Dimension_1 = w;

  *Dst_Start_Ptr  = (unsigned long)(VDev.gScreen.y_ext)*(VDev.gScreen.x_ext); // garyn
  //*Dst_Start_Ptr += (unsigned long)dy*(352*2) + dx;
  //*Dst_Start_Ptr += (unsigned long)640*480;     //garyn,on screen mem?

  *Src_Start_Ptr  = (unsigned long)(VDev.gScreen.y_ext)*(VDev.gScreen.x_ext)+ (unsigned long)640*480;
  //*Src_Start_Ptr += (unsigned long)sy*(352*2) + sx;
  //*Src_Start_Ptr += (unsigned long)sy*(176*2) + sx;

  *Op_Dimension_2   = h;

  op  = Fore_Src_PxMap;
  op |= PxBlt;
  op |= Pattern_Foreground | Right_Down;
  *Px_OpL = LOWORD(op);
  *Px_OpH = HIWORD(op);
}

/*------------------------------------------------------
   fill a rectangle with solid color
  ----------------------------------------------------*/
// garyn
void PaintWindow(int x, int y, int w, int h, unsigned long c)
{
  unsigned long op;
  wait_for_cop_ready();

  *Cop_control    = 0;

  *Fore_Mix   = (char) Mix_S;
  *Fore_Color   = c;
  if ((VDev.gScreen.bytes_per_pixel-1) != 2)
  {
    *Op_Dimension_1 = w-1;
    *Dst_Start_Ptr  = (unsigned long)y*(VDev.gScreen.x_ext) + x;
  }
  else
  {
    *Op_Dimension_1 = w-1;
    *Dst_Start_Ptr  =((unsigned long)y*(VDev.gScreen.x_ext) + x)*3;
    *Dst_X_Phase   =((unsigned char)y*(VDev.gScreen.x_ext) + x)  ;
  }
  *Op_Dimension_2   = h-1;

  op  = Fore_Src_color;
  op |= PxBlt;
  op |= Pattern_Foreground | Right_Down;
  *Px_OpL = LOWORD(op);   //this can be byte, 2 bytes or 4 bytes access,
  *Px_OpH = HIWORD(op);   //but, hi byte must be the last to access.
}

/*-----------------------------------------------------
  This routine waits until Graphics Engine is not busy.
  -----------------------------------------------------*/
void wait_for_cop_ready(void)
{
  unsigned long WaitCount;          //
                                    //
  WaitCount=0x000FFFF0;             //
  while(WaitCount!=0)               // <- with counter
  {                                 //
    if ((*Cop_control & 0x80)==0)   //
      return;                       //
    else                            //
      WaitCount --;                 //
  }                                 //

  *Cop_control = 0;                 // <- force to reset
}

/*-----------------------------------------------------
  Set some Graphics Engine registers which are not
  touched by most operations.
  -----------------------------------------------------*/
void Set_IGS_Px_Map(int w, int pixelsize)
{
  if (pixelsize != 2)
  {
    *Src_Map_Width  = w;
    *Dst_Map_Width  = w;
  }
  else  //24 bpp true color
  {
    *Src_Map_Width  = (w+1)*3-1;
    *Dst_Map_Width  = (w+1)*3-1;
  }
  *Px_Map_Format = pixelsize;
}

// !!!!!!!!!!!!!!!!!!!!!!!!
/*-----------------------------------------------------
	Functions added by Corel
-----------------------------------------------------*/
void Corel_Set_IGS_Px_Map(int sw, int dw, int pixelsize)
{
  if (pixelsize != 2)	
  {
    *Src_Map_Width  = sw;
    *Dst_Map_Width  = dw;
  }
  else  //24 bpp true color
  {
#if 0 // Original
    *Src_Map_Width  = (sw+1)*3-1;
    *Dst_Map_Width  = (dw+1)*3-1;
#endif 
    *Src_Map_Width  = sw;
    *Dst_Map_Width  = dw;
  }
  *Px_Map_Format = pixelsize;
}

void Corel_Setup_SrcCopy(unsigned long ulSrcPtr, unsigned long ulDstPtr)
{
	if (3 == VDev.gScreen.bytes_per_pixel)	/* using 24 bits color depth */
	{	
		*Src_Start_Ptr = ulSrcPtr;
		*Dst_Start_Ptr = ulDstPtr;
	}
	else																		/* using 16 bits color depth */
	{
		*Src_Start_Ptr = ulSrcPtr / 2;
		*Dst_Start_Ptr = ulDstPtr / 2;
	}
}

void Corel_SrcCopy(int sx, int sy, int dx, int dy, int w, int h)
{
        unsigned long op;
        wait_for_cop_ready();

        *Cop_control            = 0;

        *Fore_Mix               = (char) Mix_S;
        *Op_Dimension_1 = w;

//				*Dst_Start_Ptr = (unsigned long)(1023232);
        *Dst_Start_Ptr += (unsigned long)dy*(2*(*Dst_Map_Width +1)) + dx;

//				*Src_Start_Ptr = (unsigned long)(813536/*420320*/);
        *Src_Start_Ptr += (unsigned long)sy*(2*(*Src_Map_Width +1)) + sx;

        *Op_Dimension_2 = h;

        op  = Fore_Src_PxMap;
        op |= PxBlt;
        op |= Pattern_Foreground | Right_Down;
        *Px_OpL = LOWORD(op);
        *Px_OpH = HIWORD(op);
}

void Corel_SrcCopy_1(int sx, int sy, int dx, int dy, int w, int h)
{
        unsigned long op;
        wait_for_cop_ready();

        *Cop_control            = 0;

        *Fore_Mix               = (char) Mix_S;
        *Op_Dimension_1 = w;

        *Dst_Start_Ptr += (unsigned long)(dy*(*Dst_Map_Width +1)) + dx;
        *Src_Start_Ptr += (unsigned long)(sy*(*Src_Map_Width +1)) + sx;

        *Op_Dimension_2 = h;

        op  = Fore_Src_PxMap;
        op |= PxBlt;
        op |= Pattern_Foreground | Right_Down;
        *Px_OpL = LOWORD(op);
        *Px_OpH = HIWORD(op);
}
