/**
 * New Test program for video.
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <vga.h>
#include <vgagl.h>
#include <linux/types.h>
#include "videodev.h"

/*
 *	C O N S T A N T S
 */
#define CURRENT_VIDEO_CHANNEL 		1
#define TEMPSTRINGSIZE						30

/*
 *  G L O B A L   V A R I A B L E S
 */
char* TEMPSTRING = (void*)0;
int FD[3];						/* 3 handles for video devices */
int XX           = 0;	/* Text start coordinate */
int YY           = 0;
int XMIRROR      = 0;	/* keep track x,y mirroring */
int YMIRROR      = 0;
int CFREEZE      = 0; /* keep track capture freeze */
int BGDCOLOR     = 0;	/* current background color */
int WIN[3];						/* video windows flag 0-off 1-on */

/*
 *	F U N C T I O N S
 */
int  init_graph(int VideoMode);
void close_graph(void);
void capture_menu();
void display_menu();
int  get_value(int messagelen, int ypos);

int main(int argc, char** argv)
{
	unsigned char*	DataBuffer   = (void*)0;
	char						chInputKey   = 0; 
	int							Arg1         = 0;
	int							VideoMode    = 0;
	int							CurrentMenu  = 0;		/* 0-capture, 1-display */
	int							nExit        = 0;
	int							SubCaptureOn = 0;		/* using subcapture function? */

	FD[0] = -1;
	FD[1] = -1;
	FD[2] = -1;
	WIN[0] = 0;
	WIN[1] = 0;
	WIN[2] = 0;
	/* alloc some bytes */
	TEMPSTRING = malloc(TEMPSTRINGSIZE);
	memset(TEMPSTRING, 0, TEMPSTRINGSIZE);
	DataBuffer = malloc(704*480*3);
	memset(DataBuffer, 0, 704*480*3);

	if (2 != argc)
	{
		VideoMode = G800x600x64K;
		Arg1 = 1;
	}
	else
	{
		Arg1 = atoi(argv[1]);

		switch(Arg1)
		{
			case 0:
				VideoMode = G800x600x256;
			break;
			case 1:
				VideoMode = G800x600x64K;
			break;
			case 2:
				VideoMode = G800x600x16M;
			break;
			case 3:
				VideoMode = G1024x768x256;
			break;
			case 4:
				VideoMode = G1024x768x64K;
			break;
			case 5:
				VideoMode = G1024x768x16M;
			break;
			case 6:
				VideoMode = G1280x1024x256;
			break;
			case 9:
				VideoMode = G1600x1200x256;
			break;
			case 12:
				VideoMode = G640x480x256;
			break;
			case 13:
				VideoMode = G640x480x64K;
			break;
			case 14:
				VideoMode = G640x480x16M;
			break;
			case 15:
				VideoMode = G320x240x256;
			break;
			default:
				return 0;
		}
	}
	system("clear");

	if (init_graph(VideoMode) < 0) return -1;

	if (Arg1 < 3) {
		XX = 536;
		YY = 500;
	}
	else if (Arg1 < 6) {
		XX = 760;
		YY = 668;
	}
	else if (Arg1 < 9) {
		XX = 1016;
		YY = 924;
	}
	else if (Arg1 < 12) {
		XX = 1336;
		YY = 924;
	}
	else if (Arg1 < 15) {
		XX = 376;
		YY = 380;
	}
	else if (Arg1 < 18) {
		XX = 56;
		YY = 140;
	}

  /* Set screen background color (8/16/24 bits color have different setting) */
	if ((Arg1/3)*3 == Arg1)
		BGDCOLOR = 0x3;	 		/* 8 bits color */
	else if ((Arg1/2)*2 == Arg1)
  	BGDCOLOR = 0x58;	 		/* 24 bits color */
	else
		BGDCOLOR = 0x18C3;			/* 16 bits color */

	gl_clearscreen(BGDCOLOR); 

	/* USING COMPRESSED FONT */
	gl_setfont(8, 8, gl_font8x8);
	gl_setwritemode(FONT_COMPRESSED|WRITEMODE_MASKED);
	gl_setfontcolors(0x00000000, 0x00FFFFFF);

	/* OPENING VIDEO DEVICES */
	if (0 > (FD[0] = open("/dev/video0", O_RDWR)))
	{	
		fprintf(stderr, "Error opening /dev/video0.\n"); 
		close_graph();
		return 0;
	}	
	if (0 >	(FD[1] = open("/dev/video1", O_RDWR)))
	{	
		fprintf(stderr, "Error opening /dev/video1.\n"); 
		close_graph();
		return 0;
	}	
	if (0 >	(FD[2] = open("/dev/video2", O_RDWR)))
	{	
		fprintf(stderr, "Error opening /dev/video2.\n"); 
		close_graph();
		return 0;
	}	

	/* Setting channel 1 to be used as video input */
	{
		int nChannelNumber = CURRENT_VIDEO_CHANNEL;
 
		if (-1 == ioctl(FD[0], VIDIOCSCHAN, nChannelNumber))
		{
			fprintf(stderr, "Error! Cannot select channel #%d as video input.\n",
							nChannelNumber);
		}
	}

	/* Setting default capture image properties */
	{
		struct video_picture VideoPicture;

		VideoPicture.brightness = 50 * 65535 / 100;
		VideoPicture.hue				= 0 * 65535 / 100;
		VideoPicture.colour			= 50 * 65535 / 100;
		VideoPicture.contrast		= 55 * 65535 /100;
		VideoPicture.whiteness	= 0 * 65535 / 100;
		VideoPicture.palette		= VIDEO_PALETTE_YUYV;
		
		if (-1 == ioctl(FD[0], VIDIOCSPICT, &VideoPicture))
		{
			fprintf(stderr, "Error setting capture image properties.\n\n");
		}
	}

	/* Setting default capture window size */
	{
		struct video_window VideoWindow;

		VideoWindow.x					= 0;
		VideoWindow.y					= 0;
		VideoWindow.width			= 176;
		VideoWindow.height		= 144;
		VideoWindow.chromakey = 0xFFFFFF;
		VideoWindow.flags			= 0;
		VideoWindow.clips			= (void*)0;
		VideoWindow.clipcount = 0;

		if (-1 == ioctl(FD[0], VIDIOCSWIN, &VideoWindow))
		{
			fprintf(stderr, "Error setting capture window size.\n\n");
		}
	}

	/* Start capturing */
	{
		if (-1 == ioctl(FD[0], VIDIOCCAPTURE, 1))
		{
			fprintf(stderr, "Error starting capture.\n");
		}
	}

	/* Set overlay window MAIN parameters */
	{
		struct display_window DWin;

		DWin.SrcType = 0;	/* 0-local 1-remote */
		DWin.SrcPalette = VIDEO_PALETTE_YUYV;
		DWin.SrcWidth = 176;
		DWin.SrcHeight = 144;
		DWin.UseChromaKey = 0;	/* use window key for now */
		DWin.dest_win.x = 10;
		DWin.dest_win.y = 20;
		DWin.dest_win.width = 176;
		DWin.dest_win.height = 144;
		DWin.dest_win.chromakey = 0x00;

		if (-1 == ioctl(FD[0], VIDIOC_COREL_SDWIN, &DWin))
		{
			fprintf(stderr, "Error setting display window params.\n");
		}
	}

	/* Start Main overlay display */
	{
		if (-1 == ioctl(FD[0], VIDIOC_COREL_PREVIEW, 1))
		{
			fprintf(stderr, "Error starting preview.\n");
		}
		else
			WIN[0] = 1;
	}

#if 0
	/* Set overlay window V2 parameters */
	{
		struct display_window DWin;

		DWin.SrcType = 0;	/* 0-local 1-remote */
		DWin.SrcPalette = VIDEO_PALETTE_YUYV;
		DWin.SrcWidth = 176;
		DWin.SrcHeight = 144;
		DWin.UseChromaKey = 0;	/* use window key for now */
		DWin.dest_win.x = 10;
		DWin.dest_win.y = 200;
		DWin.dest_win.width = 176;
		DWin.dest_win.height = 144;
		DWin.dest_win.chromakey = 0x00;

		if (-1 == ioctl(FD[1], VIDIOC_COREL_SDWIN, &DWin))
		{
			fprintf(stderr, "Error setting display window params.\n");
		}
	}

	/* Start V2 overlay display */
	{
		if (-1 == ioctl(FD[1], VIDIOC_COREL_PREVIEW, 1))
		{
			fprintf(stderr, "Error starting preview.\n");
		}
		else
			WIN[1] = 1;
	}
#endif

	/* PRINT DEFAULT MENU (CAPTURE) */
	gl_write(10,6, "LOCAL MACHINE VIEW");
	//sleep(2);
	gl_write(XX-1,6, "VIDEO DEMO");
	//sleep(2);
	CurrentMenu = 0;
	//sleep(2);
	capture_menu();
	//gl_clearscreen(0x00000000);

	/* Main loop */
	while (nExit != 1)
	{
		chInputKey = vga_getkey();

		switch(chInputKey)
		{
			case '':		/* Refresh the screen (take out noise lines) */
			{
  			gl_clearscreen(BGDCOLOR);
				gl_write(10,6,"LOCAL MACHINE VIEW");
				gl_write(XX-1,6,"VIDEO DEMO");

				if (!CurrentMenu)
				{
					capture_menu();
				}
				else
				{
					display_menu();
				}
			}
			break;

			case 'Q':
			case 'q':
			{
				nExit = 1;
			}
			break;
			
			case 'B':		/* Increase brightness by 2% */
			{
				struct video_picture VideoPicture;
				int newvalue = 0;

				if (CurrentMenu != 0)break;

				ioctl(FD[0], VIDIOCGPICT, &VideoPicture);

				newvalue = (VideoPicture.brightness * 100) / 65535;
				if ((newvalue + 2) <= 100)
					newvalue += 2;
				VideoPicture.brightness = (newvalue * 65535) / 100;

				ioctl(FD[0], VIDIOCSPICT, &VideoPicture);

			 	/* printing the result */
				bzero(TEMPSTRING,TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, "Brightness is now: %d", newvalue);
				gl_fillbox(XX,348,260,80,BGDCOLOR);
				gl_write(XX,348,TEMPSTRING);
			}
			break;

			case 'b':		/* Decrease brightness by 2% */
			{
				struct video_picture VideoPicture;
				int newvalue = 0;
				
				if (CurrentMenu != 0)break;

				ioctl(FD[0], VIDIOCGPICT, &VideoPicture);

				newvalue = (VideoPicture.brightness * 100) / 65535;
				if ((newvalue - 2) >= 0)
					newvalue -= 2;
				VideoPicture.brightness = (newvalue * 65535) / 100;

				ioctl(FD[0], VIDIOCSPICT, &VideoPicture);
				
			 	/* printing the result */
				bzero(TEMPSTRING,TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, "Brightness is now: %d", newvalue);
				gl_fillbox(XX,348,260,80,BGDCOLOR);
				gl_write(XX,348,TEMPSTRING);
			}
			break;

			case 'H':		/* Increase hue by 2% */
			{
				struct video_picture VideoPicture;
				int newvalue = 0;

				if (CurrentMenu != 0)break;

				ioctl(FD[0], VIDIOCGPICT, &VideoPicture);

				newvalue = (VideoPicture.hue * 100) / 65535;
				if ((newvalue + 2) <= 100)
					newvalue += 2;
				VideoPicture.hue = (newvalue * 65535) / 100;

				ioctl(FD[0], VIDIOCSPICT, &VideoPicture);
				
			 	/* printing the result */
				bzero(TEMPSTRING,TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, "Hue is now: %d", newvalue);
				gl_fillbox(XX,348,260,80,BGDCOLOR);
				gl_write(XX,348,TEMPSTRING);
			}
			break;

			case 'h':		/* Decrease hue by 2% */
			{
				struct video_picture VideoPicture;
				int newvalue = 0;
				
				if (CurrentMenu != 0)break;

				ioctl(FD[0], VIDIOCGPICT, &VideoPicture);

				newvalue = (VideoPicture.hue * 100) / 65535;
				if ((newvalue - 2) >= 0)
					newvalue -= 2;
				VideoPicture.hue = (newvalue * 65535) / 100;

				ioctl(FD[0], VIDIOCSPICT, &VideoPicture);
				
			 	/* printing the result */
				bzero(TEMPSTRING,TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, "Hue is now: %d", newvalue);
				gl_fillbox(XX,348,260,80,BGDCOLOR);
				gl_write(XX,348,TEMPSTRING);
			}
			break;
			
			case 'S':		/* Increase saturation/color by 2% */
			{
				struct video_picture VideoPicture;
				int newvalue = 0;

				if (CurrentMenu != 0)break;

				ioctl(FD[0], VIDIOCGPICT, &VideoPicture);

				newvalue = (VideoPicture.colour * 100) / 65535;
				if ((newvalue + 2) <= 100)
					newvalue += 2;
				VideoPicture.colour = (newvalue * 65535) / 100;

				ioctl(FD[0], VIDIOCSPICT, &VideoPicture);
				
			 	/* printing the result */
				bzero(TEMPSTRING,TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, "Saturation is now: %d", newvalue);
				gl_fillbox(XX,348,260,80,BGDCOLOR);
				gl_write(XX,348,TEMPSTRING);
			}
			break;

			case 's':		/* Decrease saturation/color by 2% */
			{
				struct video_picture VideoPicture;
				int newvalue = 0;
				
				if (CurrentMenu != 0)break;

				ioctl(FD[0], VIDIOCGPICT, &VideoPicture);

				newvalue = (VideoPicture.colour * 100) / 65535;
				if ((newvalue - 2) >= 0)
					newvalue -= 2;
				VideoPicture.colour = (newvalue * 65535) / 100;

				ioctl(FD[0], VIDIOCSPICT, &VideoPicture);
				
			 	/* printing the result */
				bzero(TEMPSTRING,TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, "Saturation is now: %d", newvalue);
				gl_fillbox(XX,348,260,80,BGDCOLOR);
				gl_write(XX,348,TEMPSTRING);
			}
			break;

			case 'C':		/* Increase contrast by 2% */
			{
				struct video_picture VideoPicture;
				int newvalue = 0;

				if (CurrentMenu != 0)break;

				ioctl(FD[0], VIDIOCGPICT, &VideoPicture);

				newvalue = (VideoPicture.contrast * 100) / 65535;
				if ((newvalue + 2) <= 100)
					newvalue += 2;
				VideoPicture.contrast = (newvalue * 65535) / 100;

				ioctl(FD[0], VIDIOCSPICT, &VideoPicture);
				
			 	/* printing the result */
				bzero(TEMPSTRING,TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, "Contrast is now: %d", newvalue);
				gl_fillbox(XX,348,260,80,BGDCOLOR);
				gl_write(XX,348,TEMPSTRING);
			}
			break;

			case 'c':		/* Decrease contrast by 2% */
			{
				struct video_picture VideoPicture;
				int newvalue = 0;
				
				if (CurrentMenu != 0)break;

				ioctl(FD[0], VIDIOCGPICT, &VideoPicture);

				newvalue = (VideoPicture.contrast * 100) / 65535;
				if ((newvalue - 2) >= 0)
					newvalue -= 2;
				VideoPicture.contrast = (newvalue * 65535) / 100;

				ioctl(FD[0], VIDIOCSPICT, &VideoPicture);
				
			 	/* printing the result */
				bzero(TEMPSTRING,TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, "Contrast is now: %d", newvalue);
				gl_fillbox(XX,348,260,80,BGDCOLOR);
				gl_write(XX,348,TEMPSTRING);
			}
			break;

			case 'X':		/* x-axis mirroring	*/
			case 'x':	
			{
				if (CurrentMenu != 0)break;

				if (XMIRROR == ((XMIRROR /2)*2))
				{
					ioctl(FD[0], VIDIOC_COREL_MIRRORX, 1);
					gl_fillbox(XX,YY+80,120,10,BGDCOLOR);
					gl_write(XX, YY+80, "X Mirroring");
				}
				else
				{
					ioctl(FD[0], VIDIOC_COREL_MIRRORX, 0);
					gl_fillbox(XX,YY+80,120,10,BGDCOLOR);
					gl_write(XX, YY+80, "X Normal");
				}
	
				XMIRROR++;
			}
			break;

			case 'Y':		/* y-axis mirroring	*/ 
			case 'y':		
			{
				if (CurrentMenu != 0)break;

				if (YMIRROR == ((YMIRROR /2)*2))
				{
					ioctl(FD[0], VIDIOC_COREL_MIRRORY, 1);
					gl_fillbox(XX+120,YY+80,120,10,BGDCOLOR);
					gl_write(XX+120, YY+80, "Y Mirroring");
				}
				else
				{
					ioctl(FD[0], VIDIOC_COREL_MIRRORY, 0);
					gl_fillbox(XX+120,YY+80,120,10,BGDCOLOR);
					gl_write(XX+120,YY+80, "Y Normal");
				}

				YMIRROR++;
			}
			break;

			case 'F':		/* freeze/unfreeze capturing */
			case 'f':
			{
				if (CurrentMenu != 0)break;

				if (CFREEZE == ((CFREEZE/2)*2))
				{
					ioctl(FD[0], VIDIOC_COREL_CFREEZEONOFF, 1);
					gl_fillbox(XX,348,260,80,BGDCOLOR);
					gl_write(XX, 348, "Freeze capture");
				}
				else
				{
					ioctl(FD[0], VIDIOC_COREL_CFREEZEONOFF, 0);
					gl_fillbox(XX,348,260,80,BGDCOLOR);
					gl_write(XX, 348, "Unfreeze capture");
				}
				CFREEZE++;
			}
			break;

			case 'W':		/* Setting capture window size */
			case 'w':
			{
				struct video_window VW;
				struct display_window DW;
				
				if (CurrentMenu != 0)break;

				ioctl(FD[0], VIDIOCGWIN, &VW);
				ioctl(FD[0], VIDIOC_COREL_GDWIN, &DW);

				gl_fillbox(XX,348,260,60,BGDCOLOR);

				/* Getting capture width */
				gl_write(XX,348, " Width:");
				VW.width = get_value(7, 348);
				DW.SrcWidth = VW.width;

				/* Getting capture height */
				gl_write(XX,358, "Height:");
				VW.height = get_value(7, 358);
				DW.SrcHeight = VW.height;

				DW.dest_win.x = 10;
				DW.dest_win.y = 20;

			  /* Stop capturing */
    		ioctl(FD[0], VIDIOCCAPTURE, 0);

				/* Turn off main video display */
				ioctl(FD[0], VIDIOC_COREL_PREVIEW, 0);
				WIN[0] = 0;

				/* Setting channel 1 to be used as video input */
				{
					if (-1 == ioctl(FD[0], VIDIOCSCHAN, CURRENT_VIDEO_CHANNEL))
					{
						fprintf(stderr,"Error! Cannot select channel #%d as video input.\n",
										CURRENT_VIDEO_CHANNEL);
					}
				}

				/* Setting New capture window size */
				if (-1 == ioctl(FD[0], VIDIOCSWIN, &VW))
				{
					fprintf(stderr, "Error setting capture window size.\n\n");
					gl_fillbox(XX,348,260,60,BGDCOLOR);
					gl_write(XX, 348, "Unable to set requested size.");
					break;
				}

				/* Start capturing */
				ioctl(FD[0], VIDIOCCAPTURE, 1);
	
				/* Get the current capture setting */
				ioctl(FD[0], VIDIOCGWIN, &VW);
				DW.dest_win.width = VW.width;
				DW.dest_win.height = VW.height;
				
				/* Setting display window parameter */
				if (-1 == ioctl(FD[0], VIDIOC_COREL_SDWIN, &DW))
				{
					fprintf(stderr, "Error setting displaying window.\n\n");
				}

				/* Enable first video display window */
				ioctl(FD[0], VIDIOC_COREL_PREVIEW, 1);
				WIN[0] = 1;
	
				SubCaptureOn = 0;

				/* Update printed values */
				gl_fillbox(XX,348,260,60,BGDCOLOR);
				gl_fillbox(XX,YY,260,40, BGDCOLOR);
				bzero(TEMPSTRING, TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, "     X: %d", DW.dest_win.x);
				gl_write(XX,YY, TEMPSTRING); bzero(TEMPSTRING,TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, "     Y: %d", DW.dest_win.y);
				gl_write(XX,YY+10, TEMPSTRING); bzero(TEMPSTRING,TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, " Width: %d", DW.dest_win.width);
				gl_write(XX,YY+20, TEMPSTRING); bzero(TEMPSTRING,TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, "Height: %d", DW.dest_win.height);
				gl_write(XX,YY+30, TEMPSTRING); bzero(TEMPSTRING,TEMPSTRINGSIZE);
			}
			break;

			case 'R':		/* Setting capture region */
			case 'r':
			{
				struct video_capture VC;
				struct display_window DW;
				
				if (CurrentMenu != 0)break;

				ioctl(FD[0], VIDIOCSCGWIN, &VC);
				ioctl(FD[0], VIDIOC_COREL_GDWIN, &DW);

				gl_fillbox(XX,348,260,80,BGDCOLOR);

				gl_write(XX,348, "Sub-capture Region:");

				/* Getting input values */
				gl_write(XX,358, "     X:");
				VC.x = get_value(7,358);
				gl_write(XX,368, "     Y:");
				VC.y = get_value(7,368);
				gl_write(XX,378, " Width:");
				VC.width = get_value(7,378);
				gl_write(XX,388, "Height:");
				VC.height = get_value(7,388);

			  /* Stop capturing */
    		ioctl(FD[0], VIDIOCCAPTURE, 0);

				/* Turn off video 1 display */
				ioctl(FD[0], VIDIOC_COREL_PREVIEW, 0);
				WIN[0] = 0;

				/* Setting channel 1 to be used as video input */
				{
					if (-1 == ioctl(FD[0], VIDIOCSCHAN, CURRENT_VIDEO_CHANNEL))
					{
						fprintf(stderr,"Error! Cannot select channel #%d as video input.\n",
										CURRENT_VIDEO_CHANNEL);
					}
				}

				/* Setting sub-capture window */
				{
					if (-1 == ioctl(FD[0], VIDIOCSCSWIN, &VC))
					{
						fprintf(stderr, "Error setting capture window size.\n\n");
					}
				}

				/* Getting the subcapture parameters */
				ioctl(FD[0], VIDIOCSCGWIN, &VC);
				DW.dest_win.x = VC.x + 10;
				DW.dest_win.y = VC.y + 20;
				DW.dest_win.width = VC.width;
				DW.dest_win.height = VC.height;

				/* Start capturing */
				ioctl(FD[0], VIDIOCCAPTURE, 1);
	
				/* Setting display window parameter */
				if (-1 == ioctl(FD[0], VIDIOC_COREL_SDWIN, &DW))
				{
					fprintf(stderr, "Error setting displaying window.\n\n");
				}

				/* Enable first video display window */
				ioctl(FD[0], VIDIOC_COREL_PREVIEW, 1);
				WIN[0] = 1;

				SubCaptureOn = 1;

				/* Update printed values */
				gl_fillbox(XX,348,260,60,BGDCOLOR);
				gl_fillbox(XX,YY,260,40, BGDCOLOR);
				bzero(TEMPSTRING, TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, "     X: %d", DW.dest_win.x);
				gl_write(XX,YY, TEMPSTRING); bzero(TEMPSTRING,TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, "     Y: %d", DW.dest_win.y);
				gl_write(XX,YY+10, TEMPSTRING); bzero(TEMPSTRING,TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, " Width: %d", DW.dest_win.width);
				gl_write(XX,YY+20, TEMPSTRING); bzero(TEMPSTRING,TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, "Height: %d", DW.dest_win.height);
				gl_write(XX,YY+30, TEMPSTRING); bzero(TEMPSTRING,TEMPSTRINGSIZE);
			}
			break;

			case 'g':	/* Choose capture format and get a frame */
			case 'G':
			{
				struct video_picture VP;
				struct video_window VW;
				struct video_capture VC;
				char*  tempstring = (void*)0;
				char   str[]=" RGB24rawdata > RGB24.ppm";
				int    nCurrentWidth	= 0;
				int    nCurrentHeight = 0;
				int    nBytePerPixel = 0;
				int    nDataSize = 0;
				int    filed = 0;
				int    choice = 0;

				if (CurrentMenu != 0)break;

				gl_fillbox(XX,348,260,80,BGDCOLOR);
				gl_write(XX,348, "Choose capture format:");
				gl_write(XX,358, "1 - YUYV");
				gl_write(XX,368, "2 - RGB565");
				gl_write(XX,378, "3 - RGB24");

				ioctl(FD[0], VIDIOCGPICT, &VP);

				/* Getting choice */
				gl_write(XX,398, "Enter choice:");
				while((choice < 1) || (choice > 3))
				{
					gl_fillbox(XX+(13*8),398,158,10, BGDCOLOR);
					choice = get_value(13, 398);
				}
				
				switch(choice)
				{
					default:
					case 1:	/* YUYV */
					{
						VP.palette = VIDEO_PALETTE_YUYV;
						nBytePerPixel = 2;
						filed = open("YUYVrawdata", O_WRONLY|O_CREAT|O_TRUNC, 0666);
					}
					break;
				
					case 2: /* RGB565 */	
					{
						VP.palette = VIDEO_PALETTE_RGB565;
						nBytePerPixel = 2;
						filed = open("RGB565rawdata", O_WRONLY|O_CREAT|O_TRUNC, 0666);
					}
					break;
			
					case 3:	/* RGB24 */
					{
						VP.palette = VIDEO_PALETTE_RGB24;
						nBytePerPixel = 3;
						filed = open("RGB24rawdata", O_WRONLY|O_CREAT|O_TRUNC, 0666);
					}
					break;
				}

				if (0 > filed)
					break;

				ioctl(FD[0], VIDIOCSPICT, &VP);	

				if (SubCaptureOn)
				{
					ioctl(FD[0], VIDIOCSCGWIN, &VC);
					nCurrentWidth = VC.width;
					nCurrentHeight = VC.height;
				}
				else
				{
					ioctl(FD[0], VIDIOCGWIN, &VW);
					nCurrentWidth = VW.width;
					nCurrentHeight = VW.height;
				}

				nDataSize = nCurrentWidth * nCurrentHeight * nBytePerPixel;

				if (nDataSize != read(FD[0], DataBuffer, nDataSize))
				{
					fprintf(stderr, "Error reading data from capture.\n");
				}
				else
				{
					if(nDataSize != write(filed, DataBuffer, nDataSize))
						fprintf(stderr, "Error writing data to file.\n");
					else
						close(filed);
				}

				if (3 == choice)
				{
					tempstring = malloc(100);
					bzero(tempstring,100);
					sprintf(tempstring, "rawtoppm -bgr %d %d ", nCurrentWidth, 
																											nCurrentHeight);
					strcat(tempstring, str);
					system(tempstring);
					free(tempstring);
				}
			
				gl_fillbox(XX,348,260,80,BGDCOLOR);
				gl_write(XX,348,"Done...");
			}
			break;

			case 'a':	/* restore everything back to default setting */
			case 'A':
			{
				struct video_window VW;
				struct display_window DW;
				struct video_picture VP;

				if (CurrentMenu != 0)break;

				ioctl(FD[0], VIDIOCGWIN, &VW);
				ioctl(FD[0], VIDIOC_COREL_GDWIN, &DW);
				ioctl(FD[0], VIDIOCGPICT, &VP);

				gl_fillbox(XX,348,260,80,BGDCOLOR);
				gl_write(XX,348, "Restore default values");

				VW.width = 176;
				VW.height = 144;
				DW.SrcType = 0;
				DW.SrcPalette = VIDEO_PALETTE_YUYV;
				DW.SrcWidth = 176;
				DW.SrcHeight = 144;
				DW.dest_win.x = 10;
				DW.dest_win.y = 20;
				DW.dest_win.width = 176;
				DW.dest_win.height = 144;
				VP.brightness = 50 * 65535 / 100;
				VP.hue = 0;
				VP.colour = 50 * 65535 / 100;
				VP.contrast = 55 * 65535 / 100;
				SubCaptureOn = 0;
			
				/* Printout the new capture width and height */
				{
					gl_fillbox(XX,YY,260,40,BGDCOLOR);
					bzero(TEMPSTRING, TEMPSTRINGSIZE);
					sprintf(TEMPSTRING, "     X: %d", DW.dest_win.x);
					gl_write(XX,YY, TEMPSTRING); bzero(TEMPSTRING,TEMPSTRINGSIZE);
					sprintf(TEMPSTRING, "     Y: %d", DW.dest_win.y);
					gl_write(XX,YY+10, TEMPSTRING); bzero(TEMPSTRING,TEMPSTRINGSIZE);
					sprintf(TEMPSTRING, " Width: %d", DW.dest_win.width);
					gl_write(XX,YY+20, TEMPSTRING); bzero(TEMPSTRING,TEMPSTRINGSIZE);
					sprintf(TEMPSTRING, "Height: %d", DW.dest_win.height);
					gl_write(XX,YY+30, TEMPSTRING); bzero(TEMPSTRING,TEMPSTRINGSIZE);
				}

			  /* Stop capturing */
    		ioctl(FD[0], VIDIOCCAPTURE, 0);

				/* Turn off All video display */
				ioctl(FD[0], VIDIOC_COREL_PREVIEW, 0);
				WIN[0] = 0;
				/* move V2 out of screen to avoid garbage line */
				{
					struct display_window dw;
					ioctl(FD[1], VIDIOC_COREL_GDWIN, &dw);
					dw.dest_win.x = 10;
					dw.dest_win.y = 900;
					ioctl(FD[1], VIDIOC_COREL_SDWIN, &dw);
				}
				ioctl(FD[1], VIDIOC_COREL_PREVIEW, 0);
				WIN[1] = 0;
				ioctl(FD[2], VIDIOC_COREL_PREVIEW, 0);
				WIN[2] = 0;

				/* Setting channel 1 to be used as video input */
				{
					if (-1 == ioctl(FD[0], VIDIOCSCHAN, CURRENT_VIDEO_CHANNEL))
					{
						fprintf(stderr,"Error! Cannot select channel #%d as video input.\n",
										CURRENT_VIDEO_CHANNEL);
					}
				}

				/* Setting image properties */
				{
       		ioctl(FD[0], VIDIOCSPICT, &VP);
				}
			
				/* Unfreeze capture if it was freezed */
				CFREEZE = 0;
				ioctl(FD[0], VIDIOC_COREL_CFREEZEONOFF, 0);
				
				/* Setting New capture window size */
				if (-1 == ioctl(FD[0], VIDIOCSWIN, &VW))
				{
					fprintf(stderr, "Error setting capture window size.\n\n");
				}

				/* Start capturing */
				ioctl(FD[0], VIDIOCCAPTURE, 1);

				/* Return to normal orientation */
				XMIRROR = 0;
				YMIRROR = 0;
				ioctl(FD[0], VIDIOC_COREL_MIRRORX, 0);
				ioctl(FD[0], VIDIOC_COREL_MIRRORY, 0);
				gl_fillbox(XX,YY+80,120,10,BGDCOLOR);
				gl_write(XX, YY+80, "X Normal");
				gl_fillbox(XX+120,YY+80,120,10,BGDCOLOR);
				gl_write(XX+120, YY+80, "Y Normal");

				/* Setting new window parameters */
				ioctl(FD[0], VIDIOC_COREL_SDWIN, &DW);

				/* Enable first video display window */
				ioctl(FD[0], VIDIOC_COREL_PREVIEW, 1);
				WIN[0] = 1;
			}
			break;

			case '/':		/* Simulating what/how well the remote machine will see
										 the captured video */
			{
				struct video_capture  VC;
				struct video_window   VW;
				struct video_picture  VP;
				struct display_window DW;
				int  nSize					= 0;
				char chInput				= 0;
				int  nBytePerPixel	= 0;
				int  nCurrentWidth	= 0;
				int  nCurrentHeight = 0;

				if (CurrentMenu != 0)break;

				if (SubCaptureOn)
				{
					ioctl(FD[0], VIDIOCSCGWIN, &VC);
					nCurrentWidth = VC.width;
					nCurrentHeight = VC.height;
				}
				else
				{
					ioctl(FD[0], VIDIOCGWIN, &VW);
					nCurrentWidth = VW.width;
					nCurrentHeight = VW.height;
				}
		
				/* If the local video window is too big, second window will not
					 be able to display correctly. */
				if ((nCurrentHeight > 288) || (nCurrentWidth > 352))
				{
					gl_fillbox(XX,348,260,80,BGDCOLOR);
					gl_write(XX,348, "Error! Try to shrink the");
					gl_write(XX,358, "       local video window");
				}
				else
				{
  				gl_write(10, 313, "REMOTE MACHINE VIEW");

					gl_fillbox(XX, 348, 260, 80, BGDCOLOR);
					gl_write(XX, 348, "Remote video STARTED");

					/* Getting capture window size */
					ioctl(FD[0], VIDIOCGPICT, &VP);

					switch(VP.palette)
					{
						default:
						case VIDEO_PALETTE_RGB565:
						case VIDEO_PALETTE_YUYV:
						{
							nBytePerPixel = 2;
						}
						break;

						case VIDEO_PALETTE_RGB24:
						{
							nBytePerPixel = 3;
						}
						break;
					}

					DW.SrcType = 1;
					DW.SrcPalette = VP.palette;
					DW.SrcWidth = nCurrentWidth;
					DW.SrcHeight = nCurrentHeight;
					DW.dest_win.x = 10;
					DW.dest_win.y = 328;
					DW.dest_win.width = nCurrentWidth;
					DW.dest_win.height = nCurrentHeight;
					ioctl(FD[1], VIDIOC_COREL_SDWIN, &DW);
					ioctl(FD[1], VIDIOC_COREL_PREVIEW, 1);
					WIN[1] = 1;

					/* Capture data and dump it to second video display window */
					while(1)
					{
						chInput = vga_getkey();

						if ('/' == chInput)
						{
							break;
						}
						else
						{
							nSize = nCurrentWidth * nCurrentHeight * nBytePerPixel;

							if (nSize != read(FD[0], DataBuffer, nSize))
							{
								fprintf(stderr, "error reading data from capture.\n");
							}
							else
							{
								if (nSize != write(FD[1], DataBuffer, nSize))
									fprintf(stderr, "error writing data to V2 window.\n");
							}
						}
					}/* end-while */

					gl_fillbox(XX,348,260,80,BGDCOLOR);
					gl_fillbox(10,313,200,20,BGDCOLOR);
					gl_write(XX,348,"Remote video STOPPED");

					ioctl(FD[1], VIDIOC_COREL_PREVIEW, 0);	/* Turn off video 2 display */
					WIN[1] = 0;
				}/* end-else */
			}
			break;

			case '2':	/* switch to video display menu */
			{
				if (CurrentMenu != 0)break;

				CurrentMenu = 1;
				gl_fillbox(XX,38,264,YY+100-38,BGDCOLOR);
				display_menu();
			}
			break;
			
			case 'o':	/* turn on/off video window(s) */
			case 'O':
			{
				struct display_window dw;
				int window = -1;
				int srctype = -1;
				int defaultx = 0;
				int defaulty = 0;
				int defaultw = 0;
				int defaulth = 0;

				if (CurrentMenu != 1)break;

				gl_fillbox(XX,348,260,80,BGDCOLOR);

				/* Pick a window */
				gl_write(XX, 348, "Choose one:");
				gl_write(XX, 358, "0 - Main window");
				gl_write(XX, 368, "1 - V2 window");
				gl_write(XX, 378, "2 - X2 window");
				gl_write(XX, 398, "Enter choice:");
				while((window < 0) || (window > 2))
				{
					gl_fillbox(XX+(13*8),398,160,10, BGDCOLOR);
					window = get_value(13, 398);
				}

				if (WIN[window] != 0)
				{
					ioctl(FD[window], VIDIOC_COREL_PREVIEW, 0);
					WIN[window] = 0;
					if (1 == window)	/* move V2 out of screen to avoid garbage line */
					{
						ioctl(FD[window], VIDIOC_COREL_GDWIN, &dw);
						dw.dest_win.x = 10;
						dw.dest_win.y = 900;
						ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);
					}
					gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
					bzero(TEMPSTRING, TEMPSTRINGSIZE);
					sprintf(TEMPSTRING, "Video window %d is turned off.", window);
					gl_write(XX, 348, TEMPSTRING);
					break;
				}

				/* Set default values */
				switch(window)
				{
					case 0:
						defaultx = 10;
						defaulty = 20;
						defaultw = 176;
						defaulth = 144;
						break;
					
					case 1:
						defaultx = 10;
						defaulty = 200;
						defaultw = 176;
						defaulth = 144;
						break;

					case 2:
						defaultx = 240;
						defaulty = 20;
						defaultw = 176;
						defaulth = 144;
						break;
				}

				/* Pick type */
				gl_write(XX, 418, "Display data from:");
				gl_write(XX, 428, "0 - Capture");
				gl_write(XX, 438, "1 - Network");
				gl_write(XX, 448, "Enter type:");
				while((srctype < 0) || (srctype > 1))
				{
					gl_fillbox(XX+(11*8),448,176,10, BGDCOLOR);
					srctype = get_value(11, 448);
				}
				dw.SrcType = srctype;
				
				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);

				if (dw.SrcType)
				{
					/* Get SrcWidth and SrcHeight */
					gl_write(XX, 348, " SrcWidth:");
					dw.SrcWidth = get_value(10, 348);
					gl_write(XX, 358, "SrcHeight:");
					dw.SrcHeight = get_value(10, 358);
					gl_fillbox(XX, 348, 260, 20, BGDCOLOR);
				}
				else
				{
					dw.SrcWidth = 0;
					dw.SrcHeight = 0;
				}
				
				/* Get destination window params */
				gl_write(XX, 348, "Destination window:");
				gl_write(XX, 358, "     X:");
				dw.dest_win.x = get_value(7, 358);
				if (!dw.dest_win.x)dw.dest_win.x = defaultx;
				gl_write(XX, 368, "     Y:");
				dw.dest_win.y = get_value(7, 368);
				if (!dw.dest_win.y)dw.dest_win.y = defaulty;
				gl_write(XX, 378, " Width:");
				dw.dest_win.width = get_value(7, 378);
				if (!dw.dest_win.width)dw.dest_win.width = defaultw;
				gl_write(XX, 388, "Height:");
				dw.dest_win.height = get_value(7, 388);
				if (!dw.dest_win.height)dw.dest_win.height = defaulth;

				/* Filling in the rest */
				dw.SrcPalette = VIDEO_PALETTE_YUYV;
				dw.UseChromaKey = 0;
				dw.dest_win.chromakey = 0x00000000;
				dw.dest_win.flags = 0;
				dw.dest_win.clips = (void*)0;
				dw.dest_win.clipcount = 0;

#if 0
printf("BEFORE:: SrcW: %d, SrcH: %d, SrcP: 0x%08X, W: %d, H: %d\n",
				dw.SrcWidth, dw.SrcHeight, dw.SrcPalette, dw.dest_win.width,
				dw.dest_win.height);
#endif
				if (-1 == ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw))
					fprintf(stderr, "Error setting /dev/video%d parameters.\n", window);
#if 0
printf("AFTER:: SrcW: %d, SrcH: %d, SrcP: 0x%08X, W: %d, H: %d\n",
				dw.SrcWidth, dw.SrcHeight, dw.SrcPalette, dw.dest_win.width,
				dw.dest_win.height);
#endif

				ioctl(FD[window], VIDIOC_COREL_PREVIEW, 1);
				WIN[window] = 1;

				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);

				bzero(TEMPSTRING, TEMPSTRINGSIZE);
				sprintf(TEMPSTRING, "Video window %d is turned on", window);
				gl_write(XX, 348, TEMPSTRING);
			}
			break;

			case 'm':	/* Move video window */
			case 'M':
			{
				struct display_window dw;
				char	chInput = 0;
				int		window = -1;
				int		orig_x = 0;
				int		orig_y = 0;

				if (CurrentMenu != 1)break;

				gl_fillbox(XX,348,260,80,BGDCOLOR);
				
				/* Pick a window */
				gl_write(XX, 348, "Choose one:");
				gl_write(XX, 358, "0 - Main window");
				gl_write(XX, 368, "1 - V2 window");
				gl_write(XX, 378, "2 - X2 window");
				gl_write(XX, 398, "Enter choice:");
				while((window < 0) || (window > 2))
				{
					gl_fillbox(XX+(13*8),398,160,10, BGDCOLOR);
					window = get_value(13, 398);
				}

				if (WIN[window] == 0)
				{
					gl_fillbox(XX,348,260,80,BGDCOLOR);
					bzero(TEMPSTRING, TEMPSTRINGSIZE);
					sprintf(TEMPSTRING, "Video window %d is NOT on yet", window);
					gl_write(XX, 348, TEMPSTRING);
					break;
				}

				ioctl(FD[window], VIDIOC_COREL_GDWIN, &dw);
				orig_x = dw.dest_win.x;
				orig_y = dw.dest_win.y;

				gl_fillbox(XX,348,260,200,BGDCOLOR);
				gl_write(XX, 348, "q - quit");
				gl_write(XX, 358, "i - move up");
				gl_write(XX, 368, "k - move down");
				gl_write(XX, 378, "j - move left");
				gl_write(XX, 388, "l - move right");
#if 0
				/* Get new location */
				gl_write(XX, 418, "New location:");
				gl_write(XX, 428, "X:");
				dw.dest_win.x = get_value(2, 428);
				gl_write(XX, 438, "Y:");
				dw.dest_win.y = get_value(2, 438);

				ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);
#endif
				while(1)
				{
					chInput = vga_getkey();
					if ('q' == chInput)
						break;
					else if ('i' == chInput) 
					{
						dw.dest_win.y -= 8;
						ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);
					}
					else if ('k' == chInput)
					{
						dw.dest_win.y += 8;
						ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);
					}
					else if ('j' == chInput)
					{
						dw.dest_win.x -= 10;
						ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);
					}
					else if ('l' == chInput)
					{
						dw.dest_win.x += 10;
						ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);
					}
				}
						
				gl_fillbox(XX,348,260,200,BGDCOLOR);
/*
				dw.dest_win.x = orig_x;
				dw.dest_win.y = orig_y;
				ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);
*/
			}
			break;

			/* Make sure when setting X2's x,y,width,height params, set correctly
					to avoid scaling up out of bound */
			case 'u':	/* scaling up */
			case 'U':
			{
				struct display_window dw;
				struct display_window dw0;
				struct display_window dw1;
				char chInput = 0;
				int window = -1;
				int orig_x = 0;
				int orig_y = 0;
				int orig_w = 0;
				int orig_h = 0;
				int x_limit = 0;
				int y_limit = 0;

				if (CurrentMenu != 1)break;

				gl_fillbox(XX,348,260,80,BGDCOLOR);
				
				/* Pick a window */
				gl_write(XX, 348, "Choose one:");
				gl_write(XX, 358, "0 - Main window");
				gl_write(XX, 368, "1 - V2 window");
				gl_write(XX, 378, "2 - X2 window");
				gl_write(XX, 398, "Enter choice:");
				while((window < 0) || (window > 2))
				{
					gl_fillbox(XX+(13*8),398,160,10, BGDCOLOR);
					window = get_value(13, 398);
				}

				if (WIN[window] == 0)
				{
					gl_fillbox(XX,348,260,80,BGDCOLOR);
					bzero(TEMPSTRING, TEMPSTRINGSIZE);
					sprintf(TEMPSTRING, "Video window %d is NOT on yet", window);
					gl_write(XX, 348, TEMPSTRING);
					break;
				}
				
				gl_write(XX, 418, "Press 'q' to stop scaling");
				gl_write(XX, 428, "Press 'u' to scale up");
				gl_write(XX, 438, "Press 'd' to scale down");
				if ((0 == window) && !WIN[1] && !WIN[2])
				{
					gl_write(XX, 448, "Press 'U' to full screen");
				}

				ioctl(FD[window], VIDIOC_COREL_GDWIN, &dw);
				ioctl(FD[0], VIDIOC_COREL_GDWIN, &dw0);
				if (WIN[1])
					ioctl(FD[1], VIDIOC_COREL_GDWIN, &dw1);

				/* Save some old values */
				orig_x = dw.dest_win.x;
				orig_y = dw.dest_win.y;
				orig_w = dw.dest_win.width;
				orig_h = dw.dest_win.height;

				while(1)
				{
					chInput = vga_getkey();
					if ('q' == chInput)
					{
						vga_hasmode(VideoMode);
						vga_setmode(VideoMode);

						if ((Arg1/3)*3 == Arg1)
							BGDCOLOR = 0x3;	 		/* 8 bits color */
						else if ((Arg1/2)*2 == Arg1)
  						BGDCOLOR = 0x58;	 		/* 24 bits color */
						else
							BGDCOLOR = 0x18C3;			/* 16 bits color */

        		gl_clearscreen(BGDCOLOR);
        		gl_write(10,6,"LOCAL MACHINE VIEW");
        		gl_write(XX-1,6,"VIDEO DEMO");

        		if (!CurrentMenu)
        		{
          		capture_menu();
        		}
        		else
        		{
          		display_menu();
        		}
						break;
					}
					else if ('u' == chInput)
					{
						/* Set new size, take boundaries into consideration */
						x_limit = XX + 264;
						y_limit = YY + 100;
						if ((2 == window) && WIN[1])	
						{
							y_limit = dw0.dest_win.y + dw0.dest_win.height;
						}
						if ((0 == window) && !WIN[1] && WIN[2])
							x_limit = 0;
						if ((1 == window) && WIN[2])
						{
							x_limit = 0;
						}
						if ((0 == window) && WIN[1] && WIN[2])
						{
							x_limit = 0;
							y_limit = dw1.dest_win.y-2;		
						}
						if ((0 == window) && WIN[1] && !WIN[2])
							y_limit = dw1.dest_win.y-2;

						if (dw.dest_win.x + dw.dest_win.width + 5 <= x_limit)
							dw.dest_win.width += 5;
						if (dw.dest_win.y + dw.dest_win.height + 4 <= y_limit)
							dw.dest_win.height += 4;
						ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);
					}
					else if ('d' == chInput)
					{
						if ((dw.dest_win.width - 5) > dw.SrcWidth)
							dw.dest_win.width -= 5;
						if ((dw.dest_win.height -4) > dw.SrcHeight)
							dw.dest_win.height -= 4;
						ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);
					}
					else if ('U' == chInput)
					{
						vga_hasmode(G640x480x64K);
						vga_setmode(G640x480x64K);
						if ((0 == window) && !WIN[1] && !WIN[2])
						{
							/* Full Screen */
							dw.dest_win.x = 0;
							dw.dest_win.y = 0;
							dw.dest_win.width = 640;
							dw.dest_win.height = 480;
							ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);
						}
					}
				}

				/* Back to normal */
				dw.dest_win.x = orig_x;
				dw.dest_win.y = orig_y;
				dw.dest_win.width = orig_w;
				dw.dest_win.height = orig_h;
				ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);
				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
			}
			break;
			
			case 'd':	/* Scale down */
			case 'D':
			{
				struct video_picture vp;
				struct display_window dw;
				struct video_window vw;
				struct video_capture vc;
				char chInput = 0;
				int window = -1;
				int choice = -1;
				int byteperpixel = 0;
				int current_width		= 0;
				int current_height	= 0;
				int datasize = 0;
				int orig_w = 0;
				int orig_h = 0;

				if (CurrentMenu != 1)break;

				/* Pick a window */
				gl_fillbox(XX,348,260,80,BGDCOLOR);
				gl_write(XX, 348, "Choose one:");
				gl_write(XX, 358, "0 - Main window");
				gl_write(XX, 368, "1 - V2 window");
				gl_write(XX, 378, "2 - X2 window");
				gl_write(XX, 398, "Enter choice:");
				while((window < 0) || (window > 2))
				{
					gl_fillbox(XX+(13*8),398,160,10, BGDCOLOR);
					window = get_value(13, 398);
				}
				if (WIN[window] == 0)
				{
					gl_fillbox(XX,348,260,80,BGDCOLOR);
					bzero(TEMPSTRING, TEMPSTRINGSIZE);
					sprintf(TEMPSTRING, "Video window %d is NOT on yet", window);
					gl_write(XX, 348, TEMPSTRING);
					break;
				}

				/* Select capture format (and overlay format) */
				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
				gl_write(XX, 348, "Select capture format:");
				gl_write(XX, 358, "0 - YUYV");
				gl_write(XX, 368, "1 - RGB565");
				gl_write(XX, 378, "2 - RGB24");
				gl_write(XX, 398, "Enter choice:");
				while((choice < 0) || (choice > 2))
				{
					gl_fillbox(XX+(13*8), 398, 160, 10, BGDCOLOR);
					choice = get_value(13, 398);
				}
				ioctl(FD[window], VIDIOCGPICT, &vp);
				switch(choice)
				{
					default:
					case 0:
						vp.palette = VIDEO_PALETTE_YUYV;
						byteperpixel = 2;
						break;
			
					case 1:
						vp.palette = VIDEO_PALETTE_RGB565;
						byteperpixel = 2;
						break;
			
					case 2:
						vp.palette = VIDEO_PALETTE_RGB24;
						byteperpixel = 3;
						break;
				}
				ioctl(FD[window], VIDIOCSPICT, &vp);

				/* Save old values, when scale down is done, need to restore values */
				ioctl(FD[window], VIDIOC_COREL_GDWIN, &dw);
				orig_w = dw.dest_win.width;
				orig_h = dw.dest_win.height;

				/* Find the source data size (how much data we would get each read() */
				if (SubCaptureOn)
				{
					ioctl(FD[0], VIDIOCSCGWIN, &vc);
					current_width = vc.width;
					current_height = vc.height;
				}
				else
				{
					ioctl(FD[0], VIDIOCGWIN, &vw);
					current_width = vw.width;
					current_height = vw.height;
				}
				datasize = byteperpixel * current_width * current_height;

				/* Preset some of the display window attributes */
				ioctl(FD[window], VIDIOC_COREL_GDWIN, &dw);
				dw.SrcType = 1;	/* have to "write" to scale down an image */
				dw.SrcPalette = vp.palette;
				dw.SrcWidth = current_width;
				dw.SrcHeight = current_height;
				ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);

				/* Print sub-menu */
				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
				gl_write(XX, 348, "Press 'q' to quit");
				gl_write(XX, 358, "Press 'd' to scale down");

				while(1)
				{
					chInput = vga_getkey();
					if ('q' == chInput)
						break;
					else if ('d' == chInput)
					{
						ioctl(FD[window], VIDIOC_COREL_GDWIN, &dw);
						if ((dw.dest_win.width/2 >= 20) && (dw.dest_win.height/2 >= 10))
						{
							dw.dest_win.width = dw.dest_win.width / 2;
							dw.dest_win.height = dw.dest_win.height / 2;
							ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);
						}
					}
					if (datasize != read(FD[0], DataBuffer, datasize))
						fprintf(stderr, "Error reading data from capture.\n");
					if (datasize != write(FD[window], DataBuffer, datasize))
						fprintf(stderr, "Error writing data to video window.\n");
				}
			
				/* Restore old values */
				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
				dw.SrcType = 0;
				dw.dest_win.width = orig_w;
				dw.dest_win.height = orig_h;
				ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);	
			}
			break;

			case 'v':	/* partial display freezing */
			case 'V':
			{
				struct video_picture vp;
				struct display_window dw;
				struct video_freeze vf;
				char chInput = 0;
				int byteperpixel = 0;
				int datasize = 0;
				int window = -1;
				int choice = -1;

				if (CurrentMenu != 1)break;

				gl_fillbox(XX,348,260,80,BGDCOLOR);
				
				/* Pick a window */
				gl_write(XX, 348, "Choose one:");
				gl_write(XX, 358, "0 - Main window");
				gl_write(XX, 368, "1 - V2 window");
				gl_write(XX, 378, "2 - X2 window");
				gl_write(XX, 398, "Enter choice:");
				while((window < 0) || (window > 2))
				{
					gl_fillbox(XX+(13*8),398,160,10, BGDCOLOR);
					window = get_value(13, 398);
				}

				if (WIN[window] == 0)
				{
					gl_fillbox(XX,348,260,80,BGDCOLOR);
					bzero(TEMPSTRING, TEMPSTRINGSIZE);
					sprintf(TEMPSTRING, "Video window %d is NOT on yet", window);
					gl_write(XX, 348, TEMPSTRING);
					break;
				}

				/* Select capture format (and overlay format) */
				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
				gl_write(XX, 348, "Select capture format:");
				gl_write(XX, 358, "0 - YUYV");
				gl_write(XX, 368, "1 - RGB565");
				gl_write(XX, 378, "2 - RGB24");
				gl_write(XX, 398, "Enter choice:");
				while((choice < 0) || (choice > 2))
				{
					gl_fillbox(XX+(13*8), 398, 160, 10, BGDCOLOR);
					choice = get_value(13, 398);
				}
				ioctl(FD[0], VIDIOCGPICT, &vp);
				switch(choice)
				{
					default:
					case 0:
						vp.palette = VIDEO_PALETTE_YUYV;
						byteperpixel = 2;
						break;
			
					case 1:
						vp.palette = VIDEO_PALETTE_RGB565;
						byteperpixel = 2;
						break;
			
					case 2:
						vp.palette = VIDEO_PALETTE_RGB24;
						byteperpixel = 3;
						break;
				}
				ioctl(FD[0], VIDIOCSPICT, &vp);

				/* Change Source type of current video window */
				ioctl(FD[window], VIDIOC_COREL_GDWIN, &dw);
				dw.SrcType = 1;		/* have to "write" to do freeze */
				dw.SrcPalette = vp.palette;
				ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);
				
				/* Find the size of the source data */
				datasize = byteperpixel * dw.SrcWidth * dw.SrcHeight;

				/* Getting input */
				gl_write(XX, 418, "Freeze parameters:");
				gl_write(XX, 428, "  Type:");
				vf.type = get_value(7,428);
				gl_write(XX, 438, "     X:");	
				vf.x = get_value(7, 438);
				gl_write(XX, 448, "     Y:");
				vf.y = get_value(7, 448);
				gl_write(XX, 458, " Width:");
				vf.width = get_value(7, 458);
				gl_write(XX, 468, "Height:");
				vf.height = get_value(7, 468);

				ioctl(FD[window], VIDIOC_COREL_SDFREEZE, &vf);

				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
				gl_write(XX, 348, "Press 'q' to quit");
				gl_write(XX, 358, "Press 'v' to freeze");
				gl_write(XX, 368, "Press 'V' to unfreeze");
				gl_write(XX, 378, "Press 'n' to set new area");

				while(1)
				{
					chInput = vga_getkey();

					if ('q' == chInput)
						break;

					if ('v' == chInput)
					{
						ioctl(FD[window], VIDIOC_COREL_DFREEZEONOFF, 1);
					}
					if ('V' == chInput)
					{
						ioctl(FD[window], VIDIOC_COREL_DFREEZEONOFF, 0);
					}
					if ('n' == chInput)
					{
						//ioctl(FD[window], VIDIOC_COREL_DFREEZEONOFF, 0);
						gl_write(XX, 418, "Freeze parameters:");
						gl_write(XX, 428, "  Type:");
						vf.type = get_value(7,428);
						gl_write(XX, 438, "     X:");	
						vf.x = get_value(7, 438);
						gl_write(XX, 448, "     Y:");
						vf.y = get_value(7, 448);
						gl_write(XX, 458, " Width:");
						vf.width = get_value(7, 458);
						gl_write(XX, 468, "Height:");
						vf.height = get_value(7, 468);
						ioctl(FD[window], VIDIOC_COREL_SDFREEZE, &vf);
						gl_fillbox(XX, 418, 260, 70, BGDCOLOR);
						//ioctl(FD[window], VIDIOC_COREL_DFREEZEONOFF, 1);
					}	

					if (datasize != read(FD[0], DataBuffer, datasize))
						fprintf(stderr, "Erorr reading data from capture.\n");
					if (datasize != write(FD[window], DataBuffer, datasize))
						fprintf(stderr, "Error writing data to video window.\n");
				}

				/* Back to normal */
				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
				ioctl(FD[window], VIDIOC_COREL_DFREEZEONOFF, 0);
				dw.SrcType = 0;
				ioctl(FD[window], VIDIOC_COREL_SDWIN, &dw);
			}
			break;

			case 'p':	/* PIP */
			case 'P':
			{
				struct display_window dw1;
				struct display_window dw2;
				struct video_window vw;
				struct video_capture vc;
				struct video_picture vp;
				struct video_pip vpip1;
				struct video_pip vpip2;
				char chInput = 0;
				int byteperpixel = 0;
				int current_width = 0;
				int current_height = 0;
				int datasize = 0;
				int window1 = -1;
				int window2 = -1;
				int choice = -1;

				if (CurrentMenu != 1)break;

				gl_fillbox(XX,348,260,80,BGDCOLOR);
				
				/* Pick 2 windows */
				gl_write(XX, 348, "Choose BACKGROUND window:");
				gl_write(XX, 358, "0 - Main window");
				gl_write(XX, 368, "1 - V2 window");
				gl_write(XX, 378, "2 - X2 window");
				gl_write(XX, 398, "Enter choice:");
				while((window1 < 0) || (window1 > 2))
				{
					gl_fillbox(XX+(13*8),398,160,10, BGDCOLOR);
					window1 = get_value(13, 398);
				}

				if (WIN[window1] == 0)
				{
					gl_fillbox(XX,348,260,200,BGDCOLOR);
					bzero(TEMPSTRING, TEMPSTRINGSIZE);
					sprintf(TEMPSTRING, "Video window %d is NOT on yet", window1);
					gl_write(XX, 348, TEMPSTRING);
					break;
				}

				gl_write(XX, 418, "Choose FOREGROUND window:");
				gl_write(XX, 428, "0 - Main window");
				gl_write(XX, 438, "1 - V2 window");
				gl_write(XX, 448, "2 - X2 window");
				gl_write(XX, 458, "Enter choice:");
				while((window2 < 0) || (window2 > 2) || (window2 == window1))
				{
					gl_fillbox(XX+(13*8),458,160,10, BGDCOLOR);
					window2 = get_value(13, 458);
				}

				if (WIN[window2] == 0)
				{
					gl_fillbox(XX,348,260,200,BGDCOLOR);
					bzero(TEMPSTRING, TEMPSTRINGSIZE);
					sprintf(TEMPSTRING, "Video window %d is NOT on yet", window2);
					gl_write(XX, 348, TEMPSTRING);
					break;
				}

				/* Select capture format (and overlay format) */
				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
				gl_write(XX, 348, "Select capture format:");
				gl_write(XX, 358, "0 - YUYV");
				gl_write(XX, 368, "1 - RGB565");
				gl_write(XX, 378, "2 - RGB24");
				gl_write(XX, 398, "Enter choice:");
				while((choice < 0) || (choice > 2))
				{
					gl_fillbox(XX+(13*8), 398, 160, 10, BGDCOLOR);
					choice = get_value(13, 398);
				}
				ioctl(FD[0], VIDIOCGPICT, &vp);
				switch(choice)
				{
					default:
					case 0:
						vp.palette = VIDEO_PALETTE_YUYV;
						byteperpixel = 2;
						break;
			
					case 1:
						vp.palette = VIDEO_PALETTE_RGB565;
						byteperpixel = 2;
						break;
			
					case 2:
						vp.palette = VIDEO_PALETTE_RGB24;
						byteperpixel = 3;
						break;
				}
				ioctl(FD[0], VIDIOCSPICT, &vp);

				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
				gl_write(XX, 348, "PIP Type:");
				vpip1.type = get_value(9, 348);
				vpip2.type = vpip1.type;
				
				/* Find out source data size */
				if (SubCaptureOn)
				{
					ioctl(FD[0], VIDIOCSCGWIN, &vc);
					current_width = vc.width;
					current_height = vc.height;
				}
				else
				{
					ioctl(FD[0], VIDIOCGWIN, &vw);
					current_width = vw.width;
					current_height = vw.height;
				}
				datasize = byteperpixel * current_width * current_height;

				if (!vpip1.type)		/* hardware pip */
				{
					vpip1.order = 0;	/* background */
					vpip2.order = 1;	/* foreground */
					gl_write(XX, 368, "PIP area:");
					gl_write(XX, 378, "     X:");
					vpip1.x = get_value(7, 378);
					gl_write(XX, 388, "     Y:");
					vpip1.y = get_value(7, 388);
					gl_write(XX, 398, " Width:");
					vpip1.width = get_value(7, 398);
					gl_write(XX, 408, "Height:");
					vpip1.height = get_value(7, 408);
					vpip2.x = vpip1.x;				/* these values must be identical */
					vpip2.y = vpip1.y;
					if (vpip1.width < 22) vpip1.width = 22;
					if (vpip1.height < 18) vpip1.height = 18;
					vpip2.width = vpip1.width;
					vpip2.height = vpip1.height;

					/* Set up source window */
					ioctl(FD[window2], VIDIOC_COREL_GDWIN, &dw2);
					dw2.SrcType = 1;
					dw2.SrcPalette = vp.palette;
					dw2.SrcWidth = current_width;
					dw2.SrcHeight = current_height;
					dw2.dest_win.width = vpip2.width;
					dw2.dest_win.height = vpip2.height;
					ioctl(FD[window2], VIDIOC_COREL_SDWIN, &dw2);
					if (dw2.dest_win.width < vpip2.width)
						vpip2.width = dw2.dest_win.width;
					if (dw2.dest_win.height < vpip2.height)
						vpip2.height = dw2.dest_win.height;
					vpip1.width = vpip2.width;
					vpip1.height = vpip2.height;
					ioctl(FD[window2], VIDIOC_COREL_SPIP,&vpip2); 

					/* Set up background window */
					ioctl(FD[window1], VIDIOC_COREL_GDWIN, &dw1);
					dw1.SrcType = 0;
					ioctl(FD[window1], VIDIOC_COREL_SDWIN, &dw1);	
					ioctl(FD[window1], VIDIOC_COREL_SPIP,&vpip1); 
				}
				else	/* software pip */
				{
					gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
					vpip1.order = 0;	/* background */
					vpip2.order = 1;	/* foreground */
					gl_write(XX, 348, "PIP area:");
					gl_write(XX, 358, "     X:");
					vpip1.x = get_value(7, 358);
					gl_write(XX, 368, "     Y:");
					vpip1.y = get_value(7, 368);
					gl_write(XX, 378, " Width:");
					vpip1.width = get_value(7, 378);
					gl_write(XX, 388, "Height:");
					vpip1.height = get_value(7, 388);
					vpip2.x = vpip1.x;					/* values must be identical */
					vpip2.y = vpip1.y;
					if (vpip1.width < 22) vpip1.width = 22;
					if (vpip1.height < 18) vpip1.height = 18;
					vpip2.width = vpip1.width;
					vpip2.height = vpip1.height;
					
					/* Set up foreground window */
					ioctl(FD[window2], VIDIOC_COREL_GDWIN, &dw2);
					dw2.SrcType = 1;
					dw2.SrcPalette = vp.palette;
					dw2.SrcWidth = current_width;
					dw2.SrcHeight = current_height;
					dw2.dest_win.width = vpip2.width;
					dw2.dest_win.height = vpip2.height;
					ioctl(FD[window2], VIDIOC_COREL_SDWIN, &dw2);
					if (dw2.dest_win.width < vpip2.width)
						vpip2.width = dw2.dest_win.width;
					if (dw2.dest_win.height < vpip2.height)
						vpip2.height = dw2.dest_win.height;
					vpip1.width = vpip2.width;
					vpip1.height = vpip2.height;
					ioctl(FD[window2], VIDIOC_COREL_SPIP,&vpip2);

					/* Set up background window */
					ioctl(FD[window1], VIDIOC_COREL_GDWIN, &dw1);
					dw1.SrcType = 1;	
					dw1.SrcPalette = vp.palette;
					dw1.SrcWidth = current_width;
					dw1.SrcHeight = current_height;	
					dw1.dest_win.width = current_width;
					dw1.dest_win.height = current_height;
					ioctl(FD[window1], VIDIOC_COREL_SDWIN, &dw1);
					ioctl(FD[window1], VIDIOC_COREL_SPIP,&vpip1);
				}

				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
				gl_write(XX, 348, "Press 'q' to quit");
				gl_write(XX, 358, "Press 'p' to start PIP");
				gl_write(XX, 368, "Press 'P' to stop PIP");
				while(1)
				{
					chInput = vga_getkey();
					if ('q' == chInput)
					{
						ioctl(FD[window1], VIDIOC_COREL_PIPONOFF, 0);
						ioctl(FD[window2], VIDIOC_COREL_PIPONOFF, 0);
						break;
					}
					else if ('p' == chInput)
					{
						ioctl(FD[window1], VIDIOC_COREL_PIPONOFF, 1);
						ioctl(FD[window2], VIDIOC_COREL_PIPONOFF, 1);
					}
					else if ('P' == chInput)
					{		
						ioctl(FD[window1], VIDIOC_COREL_PIPONOFF, 0);
						ioctl(FD[window2], VIDIOC_COREL_PIPONOFF, 0);
					}

					if (datasize != read(FD[0], DataBuffer, datasize))
						fprintf(stderr, "Error reading data from capture.\n");
					if (vpip1.type && vpip2.type)
					{
						if (datasize != write(FD[window1], DataBuffer, datasize))
							fprintf(stderr, "Error writing data to window 1.\n");
					}
					if (datasize != write(FD[window2], DataBuffer, datasize))
						fprintf(stderr, "Error writing data to window 2.\n");
				}

				/* clean up */
				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
			}
			break;
	
			case 'i':	/* iconic video */
			{
				struct video_icon vi;
				struct display_window dw1;
				struct display_window dw2;
				struct video_window vw;
				struct video_capture vc;
				struct video_picture vp;
				char chInput = 0;
				int current_width = 0;
				int current_height = 0;
				int datasize = 0;
				int byteperpixel = 0;
				int window1 = -1;
				int window2 = -1;
				int choice = -1;
				int nTemp = 0;
				int orig_srcwidth[2];
				int orig_srcheight[2];
				int orig_destx[2];
				int orig_desty[2];
				int orig_destwidth[2];
				int orig_destheight[2];
				int orig_srctype[2];
				int orig_srcpalette[2];

				if (CurrentMenu != 1)break;

				gl_fillbox(XX,348,260,80,BGDCOLOR);

				/* Pick 2 windows */
				gl_write(XX, 348, "Choose Src window:");
				gl_write(XX, 358, "0 - Main window");
				gl_write(XX, 368, "1 - V2 window");
				gl_write(XX, 378, "2 - X2 window");
				gl_write(XX, 398, "Enter choice:");
				while((window1 < 0) || (window1 > 2))
				{
					gl_fillbox(XX+(13*8),398,160,10, BGDCOLOR);
					window1 = get_value(13, 398);
				}

				if (WIN[window1] == 0)
				{
					gl_fillbox(XX,348,260,200,BGDCOLOR);
					bzero(TEMPSTRING, TEMPSTRINGSIZE);
					sprintf(TEMPSTRING, "Video window %d is NOT on yet", window1);
					gl_write(XX, 348, TEMPSTRING);
					break;
				}
				vi.src = window1;

				gl_write(XX, 418, "Choose Destination window:");
				gl_write(XX, 428, "0 - Main window");
				gl_write(XX, 438, "1 - V2 window");
				gl_write(XX, 448, "2 - X2 window");
				gl_write(XX, 458, "Enter choice:");
				while((window2 < 0) || (window2 > 2) || (window2 == window1))
				{
					gl_fillbox(XX+(13*8),458,160,10, BGDCOLOR);
					window2 = get_value(13, 458);
				}

				if (WIN[window2] == 0)
				{
					gl_fillbox(XX,348,260,200,BGDCOLOR);
					bzero(TEMPSTRING, TEMPSTRINGSIZE);
					sprintf(TEMPSTRING, "Video window %d is NOT on yet", window2);
					gl_write(XX, 348, TEMPSTRING);
					break;
				}

				/* Select capture format (and overlay format) */
				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
				gl_write(XX, 348, "Select capture format:");
				gl_write(XX, 358, "0 - YUYV");
				gl_write(XX, 368, "1 - RGB565");
				gl_write(XX, 378, "2 - RGB24");
				gl_write(XX, 398, "Enter choice:");
				while((choice < 0) || (choice > 2))
				{
					gl_fillbox(XX+(13*8), 398, 160, 10, BGDCOLOR);
					choice = get_value(13, 398);
				}
				ioctl(FD[0], VIDIOCGPICT, &vp);
				switch(choice)
				{
					default:
					case 0:
						vp.palette = VIDEO_PALETTE_YUYV;
						byteperpixel = 2;
						break;
			
					case 1:
						vp.palette = VIDEO_PALETTE_RGB565;
						byteperpixel = 2;
						break;
			
					case 2:
						vp.palette = VIDEO_PALETTE_RGB24;
						byteperpixel = 3;
						break;
				}
				ioctl(FD[0], VIDIOCSPICT, &vp);

				gl_fillbox(XX,348,260,200,BGDCOLOR);
				gl_write(XX, 348, "Number of icons:");
				vi.total_num = get_value(16, 348);
				if ((vi.total_num <= 0) || (vi.total_num > 6))
					vi.total_num = 6;
				gl_write(XX, 368, " Width:");
				vi.width = get_value(7, 368);
				gl_write(XX, 378, "Height:");
				vi.height = get_value(7, 378);

				/* Find out source data size */
				if (SubCaptureOn)
				{
					ioctl(FD[0], VIDIOCSCGWIN, &vc);
					current_width = vc.width;
					current_height = vc.height;
				}
				else
				{
					ioctl(FD[0], VIDIOCGWIN, &vw);
					current_width = vw.width;
					current_height = vw.height;
				}
				datasize = byteperpixel * current_width * current_height;

				/* save old values */
				ioctl(FD[window1], VIDIOC_COREL_GDWIN, &dw1);
				orig_srctype[0] = dw1.SrcType;
				orig_srcpalette[0] = dw1.SrcPalette;
				orig_srcwidth[0] = dw1.SrcWidth;
				orig_srcheight[0] = dw1.SrcHeight;
				orig_destx[0] = dw1.dest_win.x;
				orig_desty[0] = dw1.dest_win.y;
//printf("window 1  x: %d  y: %d\n", dw1.dest_win.x, dw1.dest_win.y);
				orig_destwidth[0] = dw1.dest_win.width;
				orig_destheight[0] = dw1.dest_win.height;
				ioctl(FD[window2], VIDIOC_COREL_GDWIN, &dw2);
				orig_srctype[1] = dw2.SrcType;
				orig_srcpalette[1] = dw2.SrcPalette;
				orig_srcwidth[1]	= dw2.SrcWidth;
				orig_srcheight[1] = dw2.SrcHeight;
				orig_destx[1]			= dw2.dest_win.x;
				orig_desty[1]			= dw2.dest_win.y;
//printf("window 2  x: %d  y: %d\n", dw2.dest_win.x, dw2.dest_win.y);
				orig_destwidth[1] = dw2.dest_win.width;
				orig_destheight[1] = dw2.dest_win.height;
				
				/* Set up icon parameters */
				if (0 != ioctl(FD[window2], VIDIOC_COREL_SICON, &vi))
					fprintf(stderr, "Problem setting icons.\n");

				/* Get icon parameters */
				ioctl(FD[window2], VIDIOC_COREL_GICON, &vi);
//printf(" vi.width is: %d\n", vi.width);
//printf("vi.height is: %d\n", vi.height);

				/* Setting up src window */
				ioctl(FD[window1], VIDIOC_COREL_GDWIN, &dw1);
				dw1.SrcType = 1;
				dw1.SrcPalette = vp.palette;
				dw1.SrcWidth = current_width;
				dw1.SrcHeight = current_height;
				dw1.dest_win.width = vi.width;
				dw1.dest_win.height = vi.height;
				ioctl(FD[window1], VIDIOC_COREL_SDWIN, &dw1);
		
				/* Turn on icon video */
				ioctl(FD[window2], VIDIOC_COREL_ICONONOFF, 1);
				
				/* Set up destination window */
				ioctl(FD[window2], VIDIOC_COREL_GDWIN, &dw2);
				dw2.SrcType = 1;
				dw2.SrcPalette = vp.palette;
				ioctl(FD[window2], VIDIOC_COREL_SDWIN, &dw2);
		
				gl_fillbox(XX,348,260,200,BGDCOLOR);
				gl_write(XX, 348, "Press 'q' to quit");
				gl_write(XX, 358, "Press '1'-'6' to select window");
				gl_write(XX, 368, "Press 'a' to start/stop all");
				gl_write(XX+80, 378, "windows");
				while(1)
				{
					chInput = vga_getkey();
					if ('q' == chInput)
					{
						ioctl(FD[window2], VIDIOC_COREL_ICONONOFF, 0);
						break;
					}
					else if ('a' == chInput)
					{
						while(1)
						{
							chInput = vga_getkey();
							if ('a' == chInput)
								break;
							else
							{
								for (nTemp = 1; nTemp <= vi.total_num; nTemp++)
								{
									ioctl(FD[window2], VIDIOC_COREL_ICONSELECT,(int)nTemp);
									if (datasize != read(FD[0], DataBuffer, datasize))
										fprintf(stderr, "Error reading capture data.\n");
									if (datasize != write(FD[window1], DataBuffer, datasize))
										fprintf(stderr, "Error writing data to window 1.\n");
								}
							}
						}
					}
					else
					{
						if ((chInput > 48) && (chInput <= vi.total_num+48))
						{
							ioctl(FD[window2], VIDIOC_COREL_ICONSELECT,(int)(chInput - 48));
							if (datasize != read(FD[0], DataBuffer, datasize))
								fprintf(stderr, "Error reading capture data.\n");
							if (datasize != write(FD[window1], DataBuffer, datasize))
								fprintf(stderr, "Error writing data to window 1.\n");
						}
					}
				}

				/* clean up */
				gl_fillbox(XX,348,260,200,BGDCOLOR);
				
				/* Restore old values */
				ioctl(FD[window2], VIDIOC_COREL_GDWIN, &dw2);
				dw2.SrcType			= orig_srctype[1];
				dw2.SrcPalette	= orig_srcpalette[1];
				dw2.SrcWidth		= orig_srcwidth[1];
				dw2.SrcHeight		= orig_srcheight[1];
				dw2.dest_win.x			= orig_destx[1];
				dw2.dest_win.y			= orig_desty[1];
				dw2.dest_win.width	= orig_destwidth[1];
				dw2.dest_win.height	= orig_destheight[1];
				ioctl(FD[window2], VIDIOC_COREL_SDWIN, &dw2);

				ioctl(FD[window1], VIDIOC_COREL_GDWIN, &dw1);
				dw1.SrcType			= orig_srctype[0];
				dw1.SrcPalette	= orig_srcpalette[0];
				dw1.SrcWidth		= orig_srcwidth[0];
				dw1.SrcHeight		= orig_srcheight[0];
				dw1.dest_win.x			= orig_destx[0];
				dw1.dest_win.y			= orig_desty[0];
				dw1.dest_win.width	= orig_destwidth[0];
				dw1.dest_win.height	= orig_destheight[0];
				ioctl(FD[window1], VIDIOC_COREL_SDWIN, &dw1);
			}
			break;

			case 't':	/* shift video */
			{
				struct video_shift vs;
				struct display_window dw1;
				struct display_window dw2;
				struct video_window vw;
				struct video_capture vc;
				struct video_picture vp;
				char chInput = 0;
				int byteperpixel = 0;
				int current_width = 0;
				int current_height = 0;
				int datasize = 0;
				int window1 = -1;
				int window2 = -1;
				int choice  = -1;

				if (CurrentMenu != 1)break;

				gl_fillbox(XX,348,260,80,BGDCOLOR);

				/* Pick 2 windows */
				gl_write(XX, 348, "Choose Src window:");
				gl_write(XX, 358, "0 - Main window");
				gl_write(XX, 368, "1 - V2 window");
				gl_write(XX, 378, "2 - X2 window");
				gl_write(XX, 398, "Enter choice:");
				while((window1 < 0) || (window1 > 2))
				{
					gl_fillbox(XX+(13*8),398,160,10, BGDCOLOR);
					window1 = get_value(13, 398);
				}

				if (WIN[window1] == 0)
				{
					gl_fillbox(XX,348,260,200,BGDCOLOR);
					bzero(TEMPSTRING, TEMPSTRINGSIZE);
					sprintf(TEMPSTRING, "Video window %d is NOT on yet", window1);
					gl_write(XX, 348, TEMPSTRING);
					break;
				}
				vs.src = window1;

				gl_write(XX, 418, "Choose Destination window:");
				gl_write(XX, 428, "0 - Main window");
				gl_write(XX, 438, "1 - V2 window");
				gl_write(XX, 448, "2 - X2 window");
				gl_write(XX, 458, "Enter choice:");
				while((window2 < 0) || (window2 > 2) || (window2 == window1))
				{
					gl_fillbox(XX+(13*8),458,160,10, BGDCOLOR);
					window2 = get_value(13, 458);
				}

				if (WIN[window2] == 0)
				{
					gl_fillbox(XX,348,260,200,BGDCOLOR);
					bzero(TEMPSTRING, TEMPSTRINGSIZE);
					sprintf(TEMPSTRING, "Video window %d is NOT on yet", window2);
					gl_write(XX, 348, TEMPSTRING);
					break;
				}

				/* Select capture format (and overlay format) */
				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
				gl_write(XX, 348, "Select capture format:");
				gl_write(XX, 358, "0 - YUYV");
				gl_write(XX, 368, "1 - RGB565");
				gl_write(XX, 378, "2 - RGB24");
				gl_write(XX, 398, "Enter choice:");
				while((choice < 0) || (choice > 2))
				{
					gl_fillbox(XX+(13*8), 398, 160, 10, BGDCOLOR);
					choice = get_value(13, 398);
				}
				ioctl(FD[0], VIDIOCGPICT, &vp);
				switch(choice)
				{
					default:
					case 0:
						vp.palette = VIDEO_PALETTE_YUYV;
						byteperpixel = 2;
						break;
			
					case 1:
						vp.palette = VIDEO_PALETTE_RGB565;
						byteperpixel = 2;
						break;
			
					case 2:
						vp.palette = VIDEO_PALETTE_RGB24;
						byteperpixel = 3;
						break;
				}
				ioctl(FD[0], VIDIOCSPICT, &vp);

				/* Find out source data size */
				if (SubCaptureOn)
				{
					ioctl(FD[0], VIDIOCSCGWIN, &vc);
					current_width = vc.width;
					current_height = vc.height;
				}
				else
				{
					ioctl(FD[0], VIDIOCGWIN, &vw);
					current_width = vw.width;
					current_height = vw.height;
				}
				datasize = byteperpixel * current_width * current_height;

				gl_fillbox(XX,348,260,200,BGDCOLOR);
				gl_write(XX, 348, "Area to be shifted:");
				gl_write(XX, 358, "     X:");
				vs.x = get_value(7, 358);
				gl_write(XX, 368, "     Y:");
				vs.y = get_value(7, 368);
				gl_write(XX, 378, " Width:");
				vs.width = get_value(7, 378);
				/* Width must be multiple of 8 */
				vs.width = (vs.width / 8) * 8;
				gl_write(XX, 388, "Height:");
				vs.height = get_value(7, 388);

				/* Set up source window parameter */
				ioctl(FD[window1], VIDIOC_COREL_GDWIN, &dw1);
				dw1.SrcType = 1;
				dw1.SrcPalette = vp.palette;
				dw1.SrcWidth = current_width;
				dw1.SrcHeight = current_height;
				dw1.dest_win.width = current_width;
				dw1.dest_win.height = current_height;
				ioctl(FD[window1], VIDIOC_COREL_SDWIN, &dw1);

				/* Setup shift video parameter */
				ioctl(FD[window2], VIDIOC_COREL_SSHIFT, &vs);

				/* Setup destination window parameters */
				ioctl(FD[window2], VIDIOC_COREL_GDWIN, &dw2);
				dw2.SrcType = 1;
				dw2.SrcPalette = vp.palette;
				dw2.dest_win.width = 176;		/* defaulted to zoom to this size */
				dw2.dest_win.height = 144;
				ioctl(FD[window2], VIDIOC_COREL_SDWIN, &dw2);
				
				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
				gl_write(XX, 348, "Press 'q' to quit");
				gl_write(XX, 358, "Press 't' to start shift");
				while(1)
				{
					chInput = vga_getkey();
					if ('q' == chInput)
					{
						ioctl(FD[window2], VIDIOC_COREL_SHIFTONOFF, 0);
						break;
					}
					else if ('t' == chInput)
					{
						ioctl(FD[window2], VIDIOC_COREL_SHIFTONOFF, 1);
					}
				
					if (datasize != read(FD[0], DataBuffer, datasize))
						fprintf(stderr, "Error reading capture data.\n");
					if (datasize != write(FD[window1], DataBuffer, datasize))
						fprintf(stderr, "Error writing data to source window.\n");
				}

				/* Clean up */
				gl_fillbox(XX, 348, 260, 200, BGDCOLOR);
			}
			break;

			case '1': /* switch to video capture menu */
			{
				if (CurrentMenu != 1)break;

				CurrentMenu = 0;
				gl_fillbox(XX,38,264,YY+100-38,BGDCOLOR);
				capture_menu();
			}
			break;

		}/* end-switch */
	}/* end-while */

	/* Cleaning up, quiting */
	ioctl(FD[0], VIDIOCCAPTURE, 0);					/* close capture */
	ioctl(FD[1], VIDIOC_COREL_PREVIEW, 0);	/* close V2 */
	ioctl(FD[2], VIDIOC_COREL_PREVIEW, 0);	/* close X2 */
	ioctl(FD[0], VIDIOC_COREL_PREVIEW, 0);	/* close main */
	WIN[0] = 0;
	WIN[1] = 0;
	WIN[2] = 0;

	close(FD[2]);
	close(FD[1]);
	close(FD[0]);

	free(TEMPSTRING);
	free(DataBuffer);

	close_graph();
	return 0;
}

/*
 *	F U N C T I O N S
 */
int init_graph(int VideoMode)
{
	if (vga_init() != 0)
	{
		printf("vga_init failed\n");
		return -1;
	}

	if (vga_hasmode(VideoMode) != 1)
	{
		printf("vga_hasmode failed\n");
		return -1;
	}
	
	if (vga_setmode(VideoMode) != 0)
	{
		printf("vga_setmode failed\n");
		return -1;
	}
	
	gl_setcontextvga(VideoMode);	
	return 0;
}

void close_graph()
{
	vga_setmode(TEXT);
}

void capture_menu()
{
	struct display_window dw;
	int temp = 0;

	gl_write(XX, 38, "Video Capture Menu");
	gl_write(XX, 48, "============================");
	gl_write(XX, 58, "q - Quit the test program");
	gl_write(XX, 68, "B - Brightness +  2%");
	gl_write(XX, 78, "b - Brightness -  2%");
	gl_write(XX, 88, "H - Hue        +  2%");
	gl_write(XX, 98, "h - Hue        -  2%");
	gl_write(XX,108, "S - Saturation +  2%");
	gl_write(XX,118, "s - Saturation -  2%");
	gl_write(XX,128, "C - Contrast   +  2%");
	gl_write(XX,138, "c - Contrast   -  2%");
	gl_write(XX,148, "x - Enable/Disable X Mirror");
	gl_write(XX,158, "y - Enable/Disable Y Mirror");
	gl_write(XX,168, "f - Freeze/Unfreeze capture");
	gl_write(XX,178, "w - Set capture window size");
	gl_write(XX,188, "r - Set capture region");
	gl_write(XX,198, "g - Get one frame");
	gl_write(XX,208, "a - Restore default settings");
#if 0
	gl_write(XX,208, "Z - Zoom-In example");
	gl_write(XX,218, "z - Undo Zoom In");
#endif
	gl_write(XX,228, "/ - Start/Stop remote video");
	gl_write(XX,238, "2 - Video Display menu");
	gl_write(XX-16,248, "Esc - Refresh screen");

	gl_write(XX,328, "Events");
	gl_write(XX,338, "============================");

	if (-1 == ioctl(FD[0], VIDIOC_COREL_GDWIN, &dw))
		fprintf(stderr, "Error getting display window parameters.\n");

	gl_write(XX,YY-10,"Main video window"); 

	memset((void*)TEMPSTRING,0,TEMPSTRINGSIZE);
	temp = sprintf(TEMPSTRING, "     X: %d", dw.dest_win.x);
	fprintf(stderr, "%d", temp);		/* problem with sprintf?! this line
																							has to be here */
	gl_write(XX,YY, TEMPSTRING); 

	memset((void*)TEMPSTRING,0,TEMPSTRINGSIZE);
	temp = sprintf(TEMPSTRING, "     Y: %d", dw.dest_win.y);
	fprintf(stderr, "%d", temp);
	gl_write(XX,YY+10, TEMPSTRING); 

	memset((void*)TEMPSTRING,0,TEMPSTRINGSIZE);
	temp = sprintf(TEMPSTRING, " Width: %d", dw.dest_win.width);
	fprintf(stderr, "%d", temp);
	gl_write(XX,YY+20, TEMPSTRING); 

	memset((void*)TEMPSTRING,0,TEMPSTRINGSIZE);
	temp = sprintf(TEMPSTRING, "Height: %d", dw.dest_win.height);
	fprintf(stderr, "%d\n", temp);
	gl_write(XX,YY+30, TEMPSTRING); 

	memset((void*)TEMPSTRING,0,TEMPSTRINGSIZE);
				
	if (XMIRROR == ((XMIRROR /2)*2))
	{
		gl_fillbox(XX,YY+80,120,10,BGDCOLOR);
		gl_write(XX, YY+80, "X Normal");
	}
	else
	{
		gl_fillbox(XX,YY+80,120,10,BGDCOLOR);
		gl_write(XX, YY+80, "X Mirroring");
	}
	if (YMIRROR == ((YMIRROR /2)*2))
	{
		gl_fillbox(XX+120,YY+80,120,10,BGDCOLOR);
		gl_write(XX+120, YY+80, "Y Normal");
	}
	else
	{
		gl_fillbox(XX+120,YY+80,120,10,BGDCOLOR);
		gl_write(XX+120, YY+80, "Y Mirroring");
	}
}

void display_menu()
{
	gl_write(XX, 38, "Video Display Menu");
	gl_write(XX, 48, "============================");
	gl_write(XX, 58, "q - Quit the test program");
	gl_write(XX, 68, "o - On/Off video window(s)");
	gl_write(XX, 78, "m - Move individual window");
	gl_write(XX, 88, "u - Scale up display");
	gl_write(XX, 98, "d - Scale down display");
	gl_write(XX,108, "v - Dispaly freezing");
	gl_write(XX,118, "p - Picture-In-Picture");
	gl_write(XX,128, "i - Iconic video (R)");
	gl_write(XX,138, "t - Shift video");
	gl_write(XX,148, "");
	gl_write(XX,158, "");
	gl_write(XX,168, "");
	gl_write(XX,178, "");
	gl_write(XX,188, "");
	gl_write(XX,198, "");
	gl_write(XX,208, "");
	gl_write(XX,238, "1 - Video Capture menu");
	gl_write(XX-16,248, "Esc - Refresh screen");
	gl_write(XX,328, "Events");
	gl_write(XX,338, "============================");
}

int get_value(int messagelen, int ypos)
{
	char output[2];
	char chInput     = 0;
	int  nNextSpace  = 8;
	int  nTempValue  = 0;
	int  nInputValue = 0;

	messagelen *= 8;
	output[1] = '\0';

	while(1)
	{
		chInput = vga_getkey();
		if (0 != chInput)
		{
			if ('\n' == chInput)
				break;
			else if (('' == chInput) && (nNextSpace != 8))
			{
				nInputValue -= nTempValue;	
				nInputValue /= 10;
				nNextSpace -= 8;
				gl_fillbox((XX+messagelen+nNextSpace), ypos, 10, 10, BGDCOLOR);
			}
			else if (((int)chInput >= 48) && ((int)chInput <= 57))
			{
				output[0] = chInput;
				gl_write((XX+messagelen+nNextSpace), ypos, output);
				nNextSpace += 8;
				nTempValue = (int)chInput - 48;
				nInputValue *= 10;
				nInputValue += nTempValue;
			}
		}
	}	
	
	return(nInputValue);
}

#if 0		
/**************************** Not used ***************************/
#if 0
	/* Testing icon function */
	{
		struct video_icon VideoIcon;
		VideoIcon.total_num = 1;
		VideoIcon.width = 88;
		VideoIcon.height = 72;
		VideoIcon.src = 1;	/* V2 window as input source */
		ioctl(FD[0], VIDIOC_COREL_SICON, &VideoIcon);
		ioctl(FD[0], VIDIOC_COREL_ICONONOFF, 1);
	}
#endif

#if 0
	/* Testing shift video function */
	{
		struct video_shift VideoShift;
		VideoShift.x = 0;
		VideoShift.y = 0;
		VideoShift.width = 176;
		VideoShift.height = 144;
		VideoShift.src = 0;	/* Main window as input source */
		ioctl(FD[1], VIDIOC_COREL_SSHIFT, &VideoShift);	
		/* V2 is shifted video destination */
		ioctl(FD[1], VIDIOC_COREL_SHIFTONOFF, 1);
	}
#endif

#if 0
	/* Set overlay window V2 parameters */
	{
		struct display_window DWin;

		DWin.SrcType = 1;	/* 0-local 1-remote */
		DWin.SrcPalette = VIDEO_PALETTE_YUYV;
		DWin.SrcWidth = -999;
		DWin.SrcHeight = -999;
		DWin.UseChromaKey = 0;	/* use window key for now */
		DWin.dest_win.x = 10;
		DWin.dest_win.y = 300;
		DWin.dest_win.width = 88;
		DWin.dest_win.height = 72; 

		if (-1 == ioctl(FD[1], VIDIOC_COREL_SDWIN, &DWin))
		{
			fprintf(stderr, "Error setting display window params.\n");
		}
	}
#endif

#if 0
	/* Set overlay window X2 parameters */
	{
		struct display_window DWin;

		DWin.SrcType = 0;	/* 0-local 1-remote */
		DWin.SrcPalette = VIDEO_PALETTE_YUYV;
		DWin.SrcWidth = 176;
		DWin.SrcHeight = 144;
		DWin.UseChromaKey = 0;	/* use window key for now */
		DWin.dest_win.x = 200; 
		DWin.dest_win.y = 20; 
		DWin.dest_win.width = 44; 
		DWin.dest_win.height = 36;
		DWin.dest_win.chromakey = 0x000000;

		if (-1 == ioctl(FD[2], VIDIOC_COREL_SDWIN, &DWin))
		{
			fprintf(stderr, "Error setting display window params.\n");
		}
	}
#endif

#if 0
	while('q' != vga_getkey()){;}

	/* Start V2 overlay display */
	{
		if (-1 == ioctl(FD[1], VIDIOC_COREL_PREVIEW, 1))
		{
			fprintf(stderr, "Error starting preview.\n");
		}
	}
#endif

#if 0
	while('q' != vga_getkey()){;}

	/* Start X2 overlay display */
	{
		if (-1 == ioctl(FD[2], VIDIOC_COREL_PREVIEW, 1))
		{
			fprintf(stderr, "Error starting preview.\n");
		}
	}
#endif

#if 0		// Testing PIPs 
	while('q' != vga_getkey()){;}
	{
		struct video_pip VideoPip;
		VideoPip.x = 0;			
		VideoPip.y = 144;	
		VideoPip.width = 176;
		VideoPip.height = 144;
		VideoPip.type = 1;		/* second kind of PIP */
		VideoPip.order = 1;		/* V2 window act as foreground data */

		ioctl(FD[1], VIDIOC_COREL_SPIP, &VideoPip);
		ioctl(FD[1], VIDIOC_COREL_PIPONOFF, 1);
	}
	{
		struct video_pip VideoPip;
		VideoPip.type = 1;		/* second kind of PIP */
		VideoPip.order = 0;		/* Main window act as background data */
		VideoPip.x = 0;
		VideoPip.y = 0;
		VideoPip.width = 88;
		VideoPip.height = 72;

		ioctl(FD[0], VIDIOC_COREL_SPIP, &VideoPip);
		ioctl(FD[0], VIDIOC_COREL_PIPONOFF, 1);
	}
#endif

#if 0	// Testing Display freeze
	{
		struct video_freeze VideoFreeze;
		VideoFreeze.x = 88;
		VideoFreeze.y = 0;
		VideoFreeze.width = 88;
		VideoFreeze.height = 72;
		VideoFreeze.type = 0;

		ioctl(FD[0], VIDIOC_COREL_SDFREEZE, &VideoFreeze);
	}
#endif

	while('q' != vga_getkey())
	{
#if 0
		if (-1 == read(FD[0], DataBuffer, 176*144*2))
		{
			fprintf(stderr, "Error reading capture data.\n");
		}

		if (-1 == write(FD[0], DataBuffer, 176*144*2))
		{
			fprintf(stderr, "Error writing data to Main window.\n");
		}
#endif

//		ioctl(FD[0], VIDIOC_COREL_DFREEZEONOFF, 1);

#if 0
		if (counter > 1)	counter = 1;
		ioctl(FD[0], VIDIOC_COREL_ICONSELECT, counter);
		counter++;

		if (-1 == write(FD[1], DataBuffer, 176*144*2))
		{
			fprintf(stderr, "Error writing data to V2 window.\n");
		}
#endif

#if 0
		if (-1 == write(FD[2], DataBuffer, 176*144*2))
		{
			fprintf(stderr, "Error writing data to X2 window.\n");
		}
#endif
	}

#if 0
	filed = open("RGB16data", O_CREAT|O_WRONLY, 0644);
	if (0 > filed)	
		fprintf(stderr, "Error opening RGB16data file.\n");
	else
	{
		if (202752/*304128*/ != read(FD[0], RGB16data, 202752/*304128*/))
		{
			fprintf(stderr, "Error reading capture data.\n");
		}
		else if (202752/*304128*/ != write(filed, RGB16data, 202752/*304128*/))	
		{
			fprintf(stderr, "Error writing capture data to file.\n");	
		}
	}
#endif

#if 0
	while('q' != vga_getkey()){;}
	ioctl(FD[0], VIDIOC_COREL_PIPONOFF, 0);
	ioctl(FD[1], VIDIOC_COREL_PIPONOFF, 0);
	while('q' != vga_getkey()){;}
#endif
#if 0
	ioctl(FD[0], VIDIOC_COREL_ICONONOFF, 0);
#endif
#if 0
	ioctl(FD[0], VIDIOC_COREL_DFREEZEONOFF, 0);
#endif
#if 0
	ioctl(FD[1], VIDIOC_COREL_SHIFTONOFF, 0);
#endif


#if 0
	/* Get image properties */
	{
		struct video_picture VideoPicture;

		if (-1 != ioctl(FD[0], VIDIOCGPICT, &VideoPicture))
		{
			printf("\n");
			printf("brightness = %d\n", VideoPicture.brightness);
			printf("hue        = %d\n", VideoPicture.hue);
			printf("colour     = %d\n", VideoPicture.colour);
      printf("contrast   = %d\n", VideoPicture.contrast);
			printf("whiteness  = %d\n", VideoPicture.whiteness);
			if (VideoPicture.palette == VIDEO_PALETTE_YUYV)
				printf("palette    = %s\n", "VIDEO_PALETTE_YUYV");
			else if (VideoPicture.palette == VIDEO_PALETTE_RGB565)
				printf("palette    = %s\n", "VIDEO_PALETTE_RGB565");
			else if (VideoPicture.palette == VIDEO_PALETTE_RGB24)
				printf("palette    = %s\n", "VIDEO_PALETTE_RGB24");
		}
		else
			fprintf(stderr, "Error getting image properties.\n");
	}
#endif

#if 0
	/* Getting video capabilities */
	{
		struct video_capability video_cap;

		if (-1 == ioctl(FD[0], VIDIOCGCAP, &video_cap))
		{
			fprintf(stderr, "Error getting video capabilities.\n");
		}

		printf("\n");
		printf("Video interface name: %s\n", video_cap.name);
		printf("               Types: 0x%04x\n", video_cap.type);
		printf("            Channels: %d\n", video_cap.channels);
		printf("              Audios: %d\n", video_cap.audios);
		printf("   Max capture width: %d\n", video_cap.maxwidth);
		printf("  Max capture height: %d\n", video_cap.maxheight);
		printf("   Min capture width: %d\n", video_cap.minwidth);
		printf("  Min capture height: %d\n", video_cap.minheight);
	}
#endif

#if 0
	/* Get channel properties */
	{
		struct video_channel VideoChannel;

		VideoChannel.channel = CURRENT_VIDEO_CHANNEL;

		if (-1 != ioctl(FD[0], VIDIOCGCHAN, &VideoChannel))
		{
			printf("\n");
			printf("   Channel #: %d\n", VideoChannel.channel);
			printf("Channel name: %s\n", VideoChannel.name);
			printf("      tuners: %d\n", VideoChannel.tuners);
			printf("       flags: %d\n", VideoChannel.flags);
			printf("        type: 0x%04x\n", VideoChannel.type);
			printf("        norm: %d\n", VideoChannel.norm);
		}
		else
		{
			fprintf(stderr, "Error getting info for channel #%d\n", 
							VideoChannel.channel);
		}
	}
#endif

	/* Getting unit numbers */
	{
		struct video_unit VideoUnit;
		
		if (-1 == ioctl(FD[0], VIDIOCGUNIT, &VideoUnit))
		{
			fprintf(stderr, "Error getting video unit values.\n");
		}
		
		printf("\n");
		printf("The unit number of associated devices:\n");
		printf("   video: %d\n", VideoUnit.video);
		printf("     vbi: %d\n", VideoUnit.vbi);
		printf("   radio: %d\n", VideoUnit.radio);
		printf("   audio: %d\n", VideoUnit.audio);
		printf("teletext: %d\n", VideoUnit.teletext);
	}

	/* Getting capture window size */
	{
		struct video_window VideoWindow;
		
		ioctl(FD[0], VIDIOCGWIN, &VideoWindow);
		
		printf("\n");
		printf("Capture window parameters:\n");
		printf("      Capture x: %d\n", VideoWindow.x);
		printf("      Capture y: %d\n", VideoWindow.y);
    printf("  Capture width: %d\n", VideoWindow.width);
    printf(" Capture height: %d\n", VideoWindow.height);
    printf("      Chromakey: 0x%06X\n", VideoWindow.chromakey);
		printf("          flags: %d\n", VideoWindow.flags);
	}

	/* Set sub capture area */
	{
		struct video_capture VideoCapture;
	
		VideoCapture.x = 0;
		VideoCapture.y = 0;
		VideoCapture.width = 176;
		VideoCapture.height = 144;

		ioctl(fd, VIDIOCSCSWIN, &VideoCapture);
	}

			case 'Z':		/* Set capture region and Zoom into it */
			{
				int	 nNewCaptureWidth = 0;
				int  nNewCaptureHeight = 0;
				int  nNextSpace  = 8;
				char chInput     = 0;
				char output[2];
				char TEMPSTRING[20];
				int  nTempValue  = 0;
				int  nInputValue = 0;
	
				int nXSpeedFactor = 0;
				int nYSpeedFactor = 0;
				
				bzero(TEMPSTRING, 20);
				output[1] = '\0';
				gl_fillbox(760,420,260,80,0x00000007);

				if ((gCaptureWidth > 352) || (gCaptureHeight > 288))
				{
					gl_write(760,420, "Zooming not supported in");
					gl_write(760,430, "this capture resolution.");
					break;
				}
				else
				{
					gl_write(760,420, "Zooming the Capture Region: ");
				}

				/* Getting clip X */
				gl_write(760,430, "Clip X:");

				while(1)
				{
					chInput = vga_getkey();
					if (0 != chInput)
					{
						if ('\n' == chInput)
							break;
            else if (('' == chInput) && (nNextSpace != 8))
            {
              nInputValue -= nTempValue;
              nInputValue /= 10;
              nNextSpace -= 8;
              gl_fillbox((816 + nNextSpace), 430, 10, 10, 0x00000007);
            }
            else if (((int)chInput >= 48) && ((int)chInput <= 57))
						{
							output[0] = chInput;
							gl_write((816 + nNextSpace), 430, output);
							nNextSpace += 8;

							nTempValue = (int)chInput - 48;
							nInputValue *= 10;
							nInputValue += nTempValue;
						}
					}
				}	
				gClipX = nInputValue;

				/* Getting clip Y */
				chInput = 0;
				nTempValue = 0;
				nInputValue = 0;
				nNextSpace = 8;

				gl_write(760,440, "Clip Y:");

				while(1)
				{
					chInput = vga_getkey();
					if (0 != chInput)
					{
						if ('\n' == chInput)
							break;
            else if (('' == chInput) && (nNextSpace != 8))
            {
              nInputValue -= nTempValue;
              nInputValue /= 10;
              nNextSpace -= 8;
              gl_fillbox((816 + nNextSpace), 440, 10, 10, 0x00000007);
            }
            else if (((int)chInput >= 48) && ((int)chInput <= 57))
						{
							output[0] = chInput;
							gl_write((816 + nNextSpace), 440, output);
							nNextSpace += 8;

							nTempValue = (int)chInput - 48;
							nInputValue *= 10;
							nInputValue += nTempValue;
						}
					}
				}	
	
				gClipY = nInputValue;

				nNextSpace  = 8;
				chInput     = 0;
				nTempValue  = 0;
				nInputValue = 0;
				
				/* Getting clip width */
				gl_write(760,450, " Width:");

				while(1)
				{
					chInput = vga_getkey();
					if (0 != chInput)
					{
						if ('\n' == chInput)
							break;
            else if (('' == chInput) && (nNextSpace != 8))
            {
              nInputValue -= nTempValue;
              nInputValue /= 10;
              nNextSpace -= 8;
              gl_fillbox((816 + nNextSpace), 450, 10, 10, 0x00000007);
            }
            else if (((int)chInput >= 48) && ((int)chInput <= 57))
						{
							output[0] = chInput;
							gl_write((816 + nNextSpace), 450, output);
							nNextSpace += 8;

							nTempValue = (int)chInput - 48;
							nInputValue *= 10;
							nInputValue += nTempValue;
						}
					}
				}	
				gClipWidth = nInputValue;

				/* Getting clip height */
				chInput = 0;
				nTempValue = 0;
				nInputValue = 0;
				nNextSpace = 8;

				gl_write(760,460, "Height:");

				while(1)
				{
					chInput = vga_getkey();
					if (0 != chInput)
					{
						if ('\n' == chInput)
							break;
            else if (('' == chInput) && (nNextSpace != 8))
            {
              nInputValue -= nTempValue;
              nInputValue /= 10;
              nNextSpace -= 8;
              gl_fillbox((816 + nNextSpace), 460, 10, 10, 0x00000007);
            }
            else if (((int)chInput >= 48) && ((int)chInput <= 57))
						{
							output[0] = chInput;
							gl_write((816 + nNextSpace), 460, output);
							nNextSpace += 8;

							nTempValue = (int)chInput - 48;
							nInputValue *= 10;
							nInputValue += nTempValue;
						}
					}
				}	
	
				gClipHeight = nInputValue;

				gCaptureClipCount = 1;

				/* Validating the input values */
				if (gClipWidth > gCaptureWidth)
					gClipWidth = gCaptureWidth;
				if (gClipHeight > gCaptureHeight)
					gClipHeight = gCaptureHeight;
				if ((gClipX+4) > gCaptureWidth)
					gClipX = 0;
				if ((gClipY+2) > gCaptureHeight)
					gClipY = 0;

				{
					int exit = 0;
					int Level1ZoomMaxX = 352;
					int Level1ZoomMaxY = 288;
					int ZoomClipWidth  = gCaptureWidth;	
					int ZoomClipHeight = gCaptureHeight;
					int nFinalClipX    = 0;
				  int nFinalClipY    = 0;
					int nClipXinc			 = 0;
					int nClipYinc			 = 0;
					int X							 = 0;
					int Y							 = 0;
					int CW						 = gCaptureWidth;
					int CH 						 = gCaptureHeight;

					nXSpeedFactor = 8;	/* have to be multiple of 4 */
					nYSpeedFactor = 6;	/* have to be multiple of 2 */
					nFinalClipX = (gClipX * (Level1ZoomMaxX - ZoomClipWidth)) /
												(ZoomClipWidth - gClipWidth);
					nFinalClipY = (gClipY * (Level1ZoomMaxY - ZoomClipHeight)) /
												(ZoomClipHeight - gClipHeight);
#if 0
					nClipXinc = (nFinalClipX - gClipX) /
											((Level1ZoomMaxX - ZoomClipWidth) / nXSpeedFactor);
					nClipYinc = (nFinalClipY - gClipY) /
											((Level1ZoomMaxY - ZoomClipHeight) / nYSpeedFactor);
#endif
					nClipXinc = nFinalClipX /
											((Level1ZoomMaxX - ZoomClipWidth) / nXSpeedFactor);
					nClipYinc = nFinalClipY /
											((Level1ZoomMaxY - ZoomClipHeight) / nYSpeedFactor);

//printf("nFinalClipX: %d  nFinalClipY: %d  nClipXinc: %d  nClipYinc: %d\n",
//				nFinalClipX, nFinalClipY, nClipXinc, nClipYinc);

					while(2 != exit)
					{
						if (CW >= Level1ZoomMaxX)
						{
							CW = Level1ZoomMaxX;
							exit++;
						}
						else
						{
							CW += nXSpeedFactor;	
							X += nClipXinc;
						}

						if (CH >= Level1ZoomMaxY)
						{
							CH = Level1ZoomMaxY;
							exit++;
						}
						else
						{
							CH += nYSpeedFactor;
							Y += nClipYinc;
						}


				//ioctl(fd, DISPLAY1_OFF, 0);
						/* Setting image properties */
						{
							struct video_picture VideoPicture;

        			VideoPicture.brightness = gBrightness * 65535 / 100;
        			VideoPicture.hue = gHue * 65535 / 100;
        			VideoPicture.colour = gColor * 65535 / 100;
        			VideoPicture.contrast = gContrast * 65535 /100;
        			VideoPicture.palette = VIDEO_PALETTE_YUYV;

        			ioctl(fd, VIDIOCSPICT, &VideoPicture);
						}
				
						/* Setting New capture window size */
						{
							struct video_window VideoWindow;

							VideoWindow.width = CW; 
							VideoWindow.height = CH;
							VideoWindow.clips[0].x = X;
							VideoWindow.clips[0].y = Y; 
							VideoWindow.clips[0].width = ZoomClipWidth;
							VideoWindow.clips[0].height = ZoomClipHeight;
							VideoWindow.clipcount = 1; 

							if (-1 == ioctl(fd, VIDIOCSWIN, &VideoWindow))
							{
								fprintf(stderr, "Error setting capture window size.\n\n");
							}
						}
				//ioctl(fd, DISPLAY1_ON, 0);

						/* A small delay */
						{int i=0;for(i=0;i<3500000;i++);}

					}/*end-while*/
				}

				ioctl(fd, DISPLAY1_OFF, 0);
				ioctl(fd, DISPLAY1_ON, 0);

				gnZoomCount++;
			}
			break;

			case 'z':		// Undo Zooming In
			{
				int nNewCaptureWidth = 0;
				int nNewCaptureHeight = 0;

				if (gnZoomCount != 0)
				{	
					// Getting capture window size
					{
						struct video_window VideoWindow;

 		       	gl_fillbox(760,420,260,100,0x00000007);
    		//  gl_write(760,420, "============================");
    			  gl_write(760,420, "Undo Zoom In");

				#if 0
						ioctl(fd, VIDIOCGWIN, &VideoWindow);

						nNewCaptureWidth = VideoWindow.clips[0].width;
						nNewCaptureHeight = VideoWindow.clips[0].height;
						gClipX = (VideoWindow.clips[0].x * nNewCaptureWidth) / 
											gCaptureWidth;
						gClipY = (VideoWindow.clips[0].y * nNewCaptureHeight) /
											gCaptureHeight;
						gClipWidth = (nNewCaptureWidth * nNewCaptureWidth) /
													gCaptureWidth;
						gClipHeight = (nNewCaptureHeight * nNewCaptureHeight) /
													gCaptureHeight;
						gCaptureWidth = nNewCaptureWidth;
						gCaptureHeight = nNewCaptureHeight;
				#endif
						gCaptureClipCount = 0;
					}

					// Printout the new capture width and height
					{
						char TEMPSTRING[20];
						bzero(TEMPSTRING, 20);
 		       	gl_fillbox(760,660,260,30,0x00000007);
    		//  gl_write(760,420, "============================");
						sprintf(TEMPSTRING, " Width: %d", gCaptureWidth);
        		gl_write(760,660, TEMPSTRING);
						bzero(TEMPSTRING, 20);
						sprintf(TEMPSTRING, "Height: %d", gCaptureHeight);
						gl_write(760,670, TEMPSTRING);
					}

			  	/* Stop capturing */
    			ioctl(fd, VIDIOCCAPTURE, &nStop);

					/* Turn off video 1 display */
					ioctl(fd, DISPLAY1_OFF, 0);

					/* Shutdown capture portion in the device driver */
					//ioctl(fd, VIDIOCSHUTDOWN, 0);

					/* Initialize capture portion of the video driver */
					{
						int nSize = 704*486*2;
						//ioctl(fd, VIDIOCINIT, &nSize); 
					}

					/* Setting channel to be used as video input */
					{
						struct video_channel VideoChannel;
 
						VideoChannel.channel = 1;
	
						ioctl(fd, VIDIOCSCHAN, &VideoChannel);
					}

					/* Setting image properties */
					{
						struct video_picture VideoPicture;

        		VideoPicture.brightness = gBrightness * 65535 / 100;
        		VideoPicture.hue = gHue * 65535 / 100;
        		VideoPicture.colour = gColor * 65535 / 100;
        		VideoPicture.contrast = gContrast * 65535 /100;
        		VideoPicture.palette = VIDEO_PALETTE_YUYV;

        		ioctl(fd, VIDIOCSPICT, &VideoPicture);
					}
				
					/* Setting New capture window size */
					{
						struct video_window VideoWindow;

						VideoWindow.width = gCaptureWidth;
						VideoWindow.height = gCaptureHeight;
						VideoWindow.clips[0].x = gClipX;
						VideoWindow.clips[0].y = gClipY; 
						VideoWindow.clips[0].width = gClipWidth;
						VideoWindow.clips[0].height = gClipHeight;
						VideoWindow.clipcount = gCaptureClipCount;

						if (-1 == ioctl(fd, VIDIOCSWIN, &VideoWindow))
						{
							fprintf(stderr, "Error setting capture window size.\n\n");
						}
					}

					/* Start capturing */
					ioctl(fd, VIDIOCCAPTURE, &nStart);
	
					/* Enable first video display window */
					ioctl(fd, DISPLAY1_ON, 0);
		
					gnZoomCount--;
				}/* end-if */
				else
				{
 		      gl_fillbox(760,420,260,100,0x00000007);
				}
			}
			break;
	
/**************************** End *********************************/
#endif 
